// 
// DocumentReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import XMLCoder
import ViewPresentable

public struct DocumentReference: Codable, Sendable, Hashable  {
    public var id: SendableWrapper<Identifier>
    public var copyIndicator: SendableWrapper<Indicator>?
    public var uuid: SendableWrapper<Identifier>?
    public var issueDate: XMLDate?
    public var issueTime: XMLDate?
    public var documentTypeCode: SendableWrapper<Code>?
    public var documentType: SendableWrapper<Text>?
    public var xPath: [SendableWrapper<Text>] // This array should have at least one element
    public var languageID: SendableWrapper<Identifier>?
    public var localeCode: SendableWrapper<Code>?
    public var versionID: SendableWrapper<Identifier>?
    public var documentStatusCode: SendableWrapper<Code>?
    public var documentDescription: [SendableWrapper<Text>] // This array should have at least one element
    public var attachment: SendableWrapper<Attachment>?
    public var validityPeriod: SendableWrapper<Period>?
    public var issuerParty: SendableWrapper<Party>?
    public var resultOfVerification: SendableWrapper<ResultOfVerification>?

    public init(id: SendableWrapper<Identifier> = SendableWrapper(Identifier.empty()),
         copyIndicator: SendableWrapper<Indicator>? = nil,
         uuid: SendableWrapper<Identifier>? = nil,
         issueDate: XMLDate? = nil,
         issueTime: XMLDate? = nil,
         documentTypeCode: SendableWrapper<Code>? = nil,
         documentType: SendableWrapper<Text>? = nil,
         xPath: [SendableWrapper<Text>] = [],
         languageID: SendableWrapper<Identifier>? = nil,
         localeCode: SendableWrapper<Code>? = nil,
         versionID: SendableWrapper<Identifier>? = nil,
         documentStatusCode: SendableWrapper<Code>? = nil,
         documentDescription: [SendableWrapper<Text>] = [],
         attachment: SendableWrapper<Attachment>? = nil,
         validityPeriod: SendableWrapper<Period>? = nil,
         issuerParty: SendableWrapper<Party>? = nil,
         resultOfVerification: SendableWrapper<ResultOfVerification>? = nil) {
        self.id = id
        self.copyIndicator = copyIndicator
        self.uuid = uuid
        self.issueDate = issueDate
        self.issueTime = issueTime
        self.documentTypeCode = documentTypeCode
        self.documentType = documentType
        self.xPath = xPath
        self.languageID = languageID
        self.localeCode = localeCode
        self.versionID = versionID
        self.documentStatusCode = documentStatusCode
        self.documentDescription = documentDescription
        self.attachment = attachment
        self.validityPeriod = validityPeriod
        self.issuerParty = issuerParty
        self.resultOfVerification = resultOfVerification
    }
    
    public enum CodingKeys: String, CodingKey {
        case id = "ID"
        case copyIndicator = "CopyIndicator"
        case uuid = "UUID"
        case issueDate = "IssueDate"
        case issueTime = "IssueTime"
        case documentTypeCode = "DocumentTypeCode"
        case documentType = "DocumentType"
        case xPath = "XPath"
        case languageID = "LanguageID"
        case localeCode = "LocaleCode"
        case versionID = "VersionID"
        case documentStatusCode = "DocumentStatusCode"
        case documentDescription = "DocumentDescription"
        case attachment = "Attachment"
        case validityPeriod = "ValidityPeriod"
        case issuerParty = "IssuerParty"
        case resultOfVerification = "ResultOfVerification"
    }
}

extension DocumentReference: CreateEmpty {
    public nonisolated static func empty() -> DocumentReference {
        DocumentReference()
    }
}
