// 
// Invoice.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import XMLCoder
import ViewPresentable

public struct Invoice: Codable, Sendable, Hashable  {
    public var ublVersionID: SendableWrapper<Identifier>?
    public var customizationID: SendableWrapper<Identifier>?
    public var profileID: SendableWrapper<Identifier>?
    public var profileExecutionID: SendableWrapper<Identifier>?
    public var id: SendableWrapper<Identifier>
    public var copyIndicator: SendableWrapper<Indicator>?
    public var uuid: SendableWrapper<Identifier>?
    public var issueDate: XMLDate
    public var issueTime: XMLDate?
    public var dueDate: XMLDate?
    public var invoiceTypeCode: SendableWrapper<Code>?
    public var note: [SendableWrapper<Text>] // This array should have at least one element
    public var taxPointDate: XMLDate?
    public var documentCurrencyCode: SendableWrapper<Code>?
    public var taxCurrencyCode: SendableWrapper<Code>?
    public var pricingCurrencyCode: SendableWrapper<Code>?
    public var paymentCurrencyCode: SendableWrapper<Code>?
    public var paymentAlternativeCurrencyCode: SendableWrapper<Code>?
    public var accountingCostCode: SendableWrapper<Code>?
    public var accountingCost: SendableWrapper<Text>?
    public var lineCountNumeric: SendableWrapper<Numeric>?
    public var buyerReference: SendableWrapper<Text>?
    public var invoicePeriod: [SendableWrapper<Period>] // This array should have at least one element
    public var orderReference: SendableWrapper<OrderReference>?
    public var billingReference: [SendableWrapper<BillingReference>] // This array should have at least one element
    public var despatchDocumentReference: [SendableWrapper<DocumentReference>] // This array should have at least one element
    public var receiptDocumentReference: [SendableWrapper<DocumentReference>] // This array should have at least one element
    public var statementDocumentReference: [SendableWrapper<DocumentReference>] // This array should have at least one element
    public var originatorDocumentReference: [SendableWrapper<DocumentReference>] // This array should have at least one element
    public var contractDocumentReference: [SendableWrapper<DocumentReference>] // This array should have at least one element
    public var additionalDocumentReference: [SendableWrapper<DocumentReference>] // This array should have at least one element
    public var projectReference: [SendableWrapper<ProjectReference>] // This array should have at least one element
    public var signature: [SendableWrapper<Signature>] // This array should have at least one element
    public var accountingSupplierParty: SendableWrapper<SupplierParty>
    public var accountingCustomerParty: SendableWrapper<CustomerParty>
    public var payeeParty: SendableWrapper<Party>?
    public var buyerCustomerParty: SendableWrapper<CustomerParty>?
    public var sellerSupplierParty: SendableWrapper<SupplierParty>?
    public var taxRepresentativeParty: SendableWrapper<Party>?
    public var delivery: [SendableWrapper<Delivery>] // This array should have at least one element
    public var deliveryTerms: SendableWrapper<DeliveryTerms>?
    public var paymentMeans: [SendableWrapper<PaymentMeans>] // This array should have at least one element
    public var paymentTerms: [SendableWrapper<PaymentTerms>] // This array should have at least one element
    public var prepaidPayment: [SendableWrapper<Payment>] // This array should have at least one element
    public var allowanceCharge: [SendableWrapper<AllowanceCharge>] // This array should have at least one element
    public var taxExchangeRate: SendableWrapper<ExchangeRate>?
    public var pricingExchangeRate: SendableWrapper<ExchangeRate>?
    public var paymentExchangeRate: SendableWrapper<ExchangeRate>?
    public var paymentAlternativeExchangeRate: SendableWrapper<ExchangeRate>?
    public var taxTotal: [SendableWrapper<TaxTotal>] // This array should have at least one element
    public var withholdingTaxTotal: [SendableWrapper<TaxTotal>] // This array should have at least one element
    public var legalMonetaryTotal: SendableWrapper<MonetaryTotal>
    public var invoiceLine: [SendableWrapper<InvoiceLine>]

    public init(ublVersionID: SendableWrapper<Identifier>? = nil,
         customizationID: SendableWrapper<Identifier>? = nil,
         profileID: SendableWrapper<Identifier>? = nil,
         profileExecutionID: SendableWrapper<Identifier>? = nil,
         id: SendableWrapper<Identifier> = SendableWrapper(Identifier.empty()),
         copyIndicator: SendableWrapper<Indicator>? = nil,
         uuid: SendableWrapper<Identifier>? = nil,
         issueDate: XMLDate = XMLDate(date: Date()),
         issueTime: XMLDate? = nil,
         dueDate: XMLDate? = nil,
         invoiceTypeCode: SendableWrapper<Code>? = nil,
         note: [SendableWrapper<Text>] = [],
         taxPointDate: XMLDate? = nil,
         documentCurrencyCode: SendableWrapper<Code>? = nil,
         taxCurrencyCode: SendableWrapper<Code>? = nil,
         pricingCurrencyCode: SendableWrapper<Code>? = nil,
         paymentCurrencyCode: SendableWrapper<Code>? = nil,
         paymentAlternativeCurrencyCode: SendableWrapper<Code>? = nil,
         accountingCostCode: SendableWrapper<Code>? = nil,
         accountingCost: SendableWrapper<Text>? = nil,
         lineCountNumeric: SendableWrapper<Numeric>? = nil,
         buyerReference: SendableWrapper<Text>? = nil,
         invoicePeriod: [SendableWrapper<Period>] = [],
         orderReference: SendableWrapper<OrderReference>? = nil,
         billingReference: [SendableWrapper<BillingReference>] = [],
         despatchDocumentReference: [SendableWrapper<DocumentReference>] = [],
         receiptDocumentReference: [SendableWrapper<DocumentReference>] = [],
         statementDocumentReference: [SendableWrapper<DocumentReference>] = [],
         originatorDocumentReference: [SendableWrapper<DocumentReference>] = [],
         contractDocumentReference: [SendableWrapper<DocumentReference>] = [],
         additionalDocumentReference: [SendableWrapper<DocumentReference>] = [],
         projectReference: [SendableWrapper<ProjectReference>] = [],
         signature: [SendableWrapper<Signature>] = [],
         accountingSupplierParty: SendableWrapper<SupplierParty> = SendableWrapper(SupplierParty.empty()),
         accountingCustomerParty: SendableWrapper<CustomerParty> = SendableWrapper(CustomerParty.empty()),
         payeeParty: SendableWrapper<Party>? = nil,
         buyerCustomerParty: SendableWrapper<CustomerParty>? = nil,
         sellerSupplierParty: SendableWrapper<SupplierParty>? = nil,
         taxRepresentativeParty: SendableWrapper<Party>? = nil,
         delivery: [SendableWrapper<Delivery>] = [],
         deliveryTerms: SendableWrapper<DeliveryTerms>? = nil,
         paymentMeans: [SendableWrapper<PaymentMeans>] = [],
         paymentTerms: [SendableWrapper<PaymentTerms>] = [],
         prepaidPayment: [SendableWrapper<Payment>] = [],
         allowanceCharge: [SendableWrapper<AllowanceCharge>] = [],
         taxExchangeRate: SendableWrapper<ExchangeRate>? = nil,
         pricingExchangeRate: SendableWrapper<ExchangeRate>? = nil,
         paymentExchangeRate: SendableWrapper<ExchangeRate>? = nil,
         paymentAlternativeExchangeRate: SendableWrapper<ExchangeRate>? = nil,
         taxTotal: [SendableWrapper<TaxTotal>] = [],
         withholdingTaxTotal: [SendableWrapper<TaxTotal>] = [],
         legalMonetaryTotal: SendableWrapper<MonetaryTotal> = SendableWrapper(MonetaryTotal.empty()),
         invoiceLine: [SendableWrapper<InvoiceLine>] = []) {
        self.ublVersionID = ublVersionID
        self.customizationID = customizationID
        self.profileID = profileID
        self.profileExecutionID = profileExecutionID
        self.id = id
        self.copyIndicator = copyIndicator
        self.uuid = uuid
        self.issueDate = issueDate
        self.issueTime = issueTime
        self.dueDate = dueDate
        self.invoiceTypeCode = invoiceTypeCode
        self.note = note
        self.taxPointDate = taxPointDate
        self.documentCurrencyCode = documentCurrencyCode
        self.taxCurrencyCode = taxCurrencyCode
        self.pricingCurrencyCode = pricingCurrencyCode
        self.paymentCurrencyCode = paymentCurrencyCode
        self.paymentAlternativeCurrencyCode = paymentAlternativeCurrencyCode
        self.accountingCostCode = accountingCostCode
        self.accountingCost = accountingCost
        self.lineCountNumeric = lineCountNumeric
        self.buyerReference = buyerReference
        self.invoicePeriod = invoicePeriod
        self.orderReference = orderReference
        self.billingReference = billingReference
        self.despatchDocumentReference = despatchDocumentReference
        self.receiptDocumentReference = receiptDocumentReference
        self.statementDocumentReference = statementDocumentReference
        self.originatorDocumentReference = originatorDocumentReference
        self.contractDocumentReference = contractDocumentReference
        self.additionalDocumentReference = additionalDocumentReference
        self.projectReference = projectReference
        self.signature = signature
        self.accountingSupplierParty = accountingSupplierParty
        self.accountingCustomerParty = accountingCustomerParty
        self.payeeParty = payeeParty
        self.buyerCustomerParty = buyerCustomerParty
        self.sellerSupplierParty = sellerSupplierParty
        self.taxRepresentativeParty = taxRepresentativeParty
        self.delivery = delivery
        self.deliveryTerms = deliveryTerms
        self.paymentMeans = paymentMeans
        self.paymentTerms = paymentTerms
        self.prepaidPayment = prepaidPayment
        self.allowanceCharge = allowanceCharge
        self.taxExchangeRate = taxExchangeRate
        self.pricingExchangeRate = pricingExchangeRate
        self.paymentExchangeRate = paymentExchangeRate
        self.paymentAlternativeExchangeRate = paymentAlternativeExchangeRate
        self.taxTotal = taxTotal
        self.withholdingTaxTotal = withholdingTaxTotal
        self.legalMonetaryTotal = legalMonetaryTotal
        self.invoiceLine = invoiceLine
    }
    
    public enum CodingKeys: String, CodingKey {
        case ublVersionID = "UBLVersionID"
        case customizationID = "CustomizationID"
        case profileID = "ProfileID"
        case profileExecutionID = "ProfileExecutionID"
        case id = "ID"
        case copyIndicator = "CopyIndicator"
        case uuid = "UUID"
        case issueDate = "IssueDate"
        case issueTime = "IssueTime"
        case dueDate = "DueDate"
        case invoiceTypeCode = "InvoiceTypeCode"
        case note = "Note"
        case taxPointDate = "TaxPointDate"
        case documentCurrencyCode = "DocumentCurrencyCode"
        case taxCurrencyCode = "TaxCurrencyCode"
        case pricingCurrencyCode = "PricingCurrencyCode"
        case paymentCurrencyCode = "PaymentCurrencyCode"
        case paymentAlternativeCurrencyCode = "PaymentAlternativeCurrencyCode"
        case accountingCostCode = "AccountingCostCode"
        case accountingCost = "AccountingCost"
        case lineCountNumeric = "LineCountNumeric"
        case buyerReference = "BuyerReference"
        case invoicePeriod = "InvoicePeriod"
        case orderReference = "OrderReference"
        case billingReference = "BillingReference"
        case despatchDocumentReference = "DespatchDocumentReference"
        case receiptDocumentReference = "ReceiptDocumentReference"
        case statementDocumentReference = "StatementDocumentReference"
        case originatorDocumentReference = "OriginatorDocumentReference"
        case contractDocumentReference = "ContractDocumentReference"
        case additionalDocumentReference = "AdditionalDocumentReference"
        case projectReference = "ProjectReference"
        case signature = "Signature"
        case accountingSupplierParty = "AccountingSupplierParty"
        case accountingCustomerParty = "AccountingCustomerParty"
        case payeeParty = "PayeeParty"
        case buyerCustomerParty = "BuyerCustomerParty"
        case sellerSupplierParty = "SellerSupplierParty"
        case taxRepresentativeParty = "TaxRepresentativeParty"
        case delivery = "Delivery"
        case deliveryTerms = "DeliveryTerms"
        case paymentMeans = "PaymentMeans"
        case paymentTerms = "PaymentTerms"
        case prepaidPayment = "PrepaidPayment"
        case allowanceCharge = "AllowanceCharge"
        case taxExchangeRate = "TaxExchangeRate"
        case pricingExchangeRate = "PricingExchangeRate"
        case paymentExchangeRate = "PaymentExchangeRate"
        case paymentAlternativeExchangeRate = "PaymentAlternativeExchangeRate"
        case taxTotal = "TaxTotal"
        case withholdingTaxTotal = "WithholdingTaxTotal"
        case legalMonetaryTotal = "LegalMonetaryTotal"
        case invoiceLine = "InvoiceLine"
    }
}

extension Invoice: CreateEmpty {
    public nonisolated static func empty() -> Invoice {
        Invoice()
    }
}
