// 
// InvoiceLine.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import XMLCoder
import ViewPresentable

public struct InvoiceLine: Codable, Sendable, Hashable  {
    public var id: SendableWrapper<Identifier>
    public var uuid: SendableWrapper<Identifier>?
    public var note: [SendableWrapper<Text>] // This array should have at least one element
    public var invoicedQuantity: SendableWrapper<Quantity>?
    public var lineExtensionAmount: SendableWrapper<Amount>
    public var taxPointDate: XMLDate?
    public var accountingCostCode: SendableWrapper<Code>?
    public var accountingCost: SendableWrapper<Text>?
    public var paymentPurposeCode: SendableWrapper<Code>?
    public var freeOfChargeIndicator: SendableWrapper<Indicator>?
    public var invoicePeriod: [SendableWrapper<Period>] // This array should have at least one element
    public var orderLineReference: [SendableWrapper<OrderLineReference>] // This array should have at least one element
    public var despatchLineReference: [SendableWrapper<LineReference>] // This array should have at least one element
    public var receiptLineReference: [SendableWrapper<LineReference>] // This array should have at least one element
    public var billingReference: [SendableWrapper<BillingReference>] // This array should have at least one element
    public var documentReference: [SendableWrapper<DocumentReference>] // This array should have at least one element
    public var pricingReference: SendableWrapper<PricingReference>?
    public var originatorParty: SendableWrapper<Party>?
    public var delivery: [SendableWrapper<Delivery>] // This array should have at least one element
    public var paymentTerms: [SendableWrapper<PaymentTerms>] // This array should have at least one element
    public var allowanceCharge: [SendableWrapper<AllowanceCharge>] // This array should have at least one element
    public var taxTotal: [SendableWrapper<TaxTotal>] // This array should have at least one element
    public var withholdingTaxTotal: [SendableWrapper<TaxTotal>] // This array should have at least one element
    public var item: SendableWrapper<Item>
    public var price: SendableWrapper<Price>?
    public var deliveryTerms: SendableWrapper<DeliveryTerms>?
    public var subInvoiceLine: [SendableWrapper<InvoiceLine>] // This array should have at least one element
    public var itemPriceExtension: SendableWrapper<PriceExtension>?

    public init(id: SendableWrapper<Identifier> = SendableWrapper(Identifier.empty()),
         uuid: SendableWrapper<Identifier>? = nil,
         note: [SendableWrapper<Text>] = [],
         invoicedQuantity: SendableWrapper<Quantity>? = nil,
         lineExtensionAmount: SendableWrapper<Amount> = SendableWrapper(Amount.empty()),
         taxPointDate: XMLDate? = nil,
         accountingCostCode: SendableWrapper<Code>? = nil,
         accountingCost: SendableWrapper<Text>? = nil,
         paymentPurposeCode: SendableWrapper<Code>? = nil,
         freeOfChargeIndicator: SendableWrapper<Indicator>? = nil,
         invoicePeriod: [SendableWrapper<Period>] = [],
         orderLineReference: [SendableWrapper<OrderLineReference>] = [],
         despatchLineReference: [SendableWrapper<LineReference>] = [],
         receiptLineReference: [SendableWrapper<LineReference>] = [],
         billingReference: [SendableWrapper<BillingReference>] = [],
         documentReference: [SendableWrapper<DocumentReference>] = [],
         pricingReference: SendableWrapper<PricingReference>? = nil,
         originatorParty: SendableWrapper<Party>? = nil,
         delivery: [SendableWrapper<Delivery>] = [],
         paymentTerms: [SendableWrapper<PaymentTerms>] = [],
         allowanceCharge: [SendableWrapper<AllowanceCharge>] = [],
         taxTotal: [SendableWrapper<TaxTotal>] = [],
         withholdingTaxTotal: [SendableWrapper<TaxTotal>] = [],
         item: SendableWrapper<Item> = SendableWrapper(Item.empty()),
         price: SendableWrapper<Price>? = nil,
         deliveryTerms: SendableWrapper<DeliveryTerms>? = nil,
         subInvoiceLine: [SendableWrapper<InvoiceLine>] = [],
         itemPriceExtension: SendableWrapper<PriceExtension>? = nil) {
        self.id = id
        self.uuid = uuid
        self.note = note
        self.invoicedQuantity = invoicedQuantity
        self.lineExtensionAmount = lineExtensionAmount
        self.taxPointDate = taxPointDate
        self.accountingCostCode = accountingCostCode
        self.accountingCost = accountingCost
        self.paymentPurposeCode = paymentPurposeCode
        self.freeOfChargeIndicator = freeOfChargeIndicator
        self.invoicePeriod = invoicePeriod
        self.orderLineReference = orderLineReference
        self.despatchLineReference = despatchLineReference
        self.receiptLineReference = receiptLineReference
        self.billingReference = billingReference
        self.documentReference = documentReference
        self.pricingReference = pricingReference
        self.originatorParty = originatorParty
        self.delivery = delivery
        self.paymentTerms = paymentTerms
        self.allowanceCharge = allowanceCharge
        self.taxTotal = taxTotal
        self.withholdingTaxTotal = withholdingTaxTotal
        self.item = item
        self.price = price
        self.deliveryTerms = deliveryTerms
        self.subInvoiceLine = subInvoiceLine
        self.itemPriceExtension = itemPriceExtension
    }
    
    public enum CodingKeys: String, CodingKey {
        case id = "ID"
        case uuid = "UUID"
        case note = "Note"
        case invoicedQuantity = "InvoicedQuantity"
        case lineExtensionAmount = "LineExtensionAmount"
        case taxPointDate = "TaxPointDate"
        case accountingCostCode = "AccountingCostCode"
        case accountingCost = "AccountingCost"
        case paymentPurposeCode = "PaymentPurposeCode"
        case freeOfChargeIndicator = "FreeOfChargeIndicator"
        case invoicePeriod = "InvoicePeriod"
        case orderLineReference = "OrderLineReference"
        case despatchLineReference = "DespatchLineReference"
        case receiptLineReference = "ReceiptLineReference"
        case billingReference = "BillingReference"
        case documentReference = "DocumentReference"
        case pricingReference = "PricingReference"
        case originatorParty = "OriginatorParty"
        case delivery = "Delivery"
        case paymentTerms = "PaymentTerms"
        case allowanceCharge = "AllowanceCharge"
        case taxTotal = "TaxTotal"
        case withholdingTaxTotal = "WithholdingTaxTotal"
        case item = "Item"
        case price = "Price"
        case deliveryTerms = "DeliveryTerms"
        case subInvoiceLine = "SubInvoiceLine"
        case itemPriceExtension = "ItemPriceExtension"
    }
}

extension InvoiceLine: CreateEmpty {
    public nonisolated static func empty() -> InvoiceLine {
        InvoiceLine()
    }
}
