// 
// TransportationService.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import XMLCoder
import ViewPresentable

public struct TransportationService: Codable, Sendable, Hashable  {
    public var transportServiceCode: SendableWrapper<Code>
    public var tariffClassCode: SendableWrapper<Code>?
    public var priority: SendableWrapper<Text>?
    public var freightRateClassCode: SendableWrapper<Code>?
    public var transportationServiceDescription: [SendableWrapper<Text>] // This array should have at least one element
    public var transportationServiceDetailsURI: SendableWrapper<Identifier>?
    public var nominationDate: XMLDate?
    public var nominationTime: XMLDate?
    public var name: SendableWrapper<Text>?
    public var sequenceNumeric: SendableWrapper<Numeric>?
    public var transportEquipment: [SendableWrapper<TransportEquipment>] // This array should have at least one element
    public var supportedTransportEquipment: [SendableWrapper<TransportEquipment>] // This array should have at least one element
    public var unsupportedTransportEquipment: [SendableWrapper<TransportEquipment>] // This array should have at least one element
    public var commodityClassification: [SendableWrapper<CommodityClassification>] // This array should have at least one element
    public var supportedCommodityClassification: [SendableWrapper<CommodityClassification>] // This array should have at least one element
    public var unsupportedCommodityClassification: [SendableWrapper<CommodityClassification>] // This array should have at least one element
    public var totalCapacityDimension: SendableWrapper<Dimension>?
    public var shipmentStage: [SendableWrapper<ShipmentStage>] // This array should have at least one element
    public var transportEvent: [SendableWrapper<TransportEvent>] // This array should have at least one element
    public var responsibleTransportServiceProviderParty: SendableWrapper<Party>?
    public var environmentalEmission: [SendableWrapper<EnvironmentalEmission>] // This array should have at least one element
    public var estimatedDurationPeriod: SendableWrapper<Period>?
    public var scheduledServiceFrequency: [SendableWrapper<ServiceFrequency>] // This array should have at least one element

    public init(transportServiceCode: SendableWrapper<Code> = SendableWrapper(Code.empty()),
         tariffClassCode: SendableWrapper<Code>? = nil,
         priority: SendableWrapper<Text>? = nil,
         freightRateClassCode: SendableWrapper<Code>? = nil,
         transportationServiceDescription: [SendableWrapper<Text>] = [],
         transportationServiceDetailsURI: SendableWrapper<Identifier>? = nil,
         nominationDate: XMLDate? = nil,
         nominationTime: XMLDate? = nil,
         name: SendableWrapper<Text>? = nil,
         sequenceNumeric: SendableWrapper<Numeric>? = nil,
         transportEquipment: [SendableWrapper<TransportEquipment>] = [],
         supportedTransportEquipment: [SendableWrapper<TransportEquipment>] = [],
         unsupportedTransportEquipment: [SendableWrapper<TransportEquipment>] = [],
         commodityClassification: [SendableWrapper<CommodityClassification>] = [],
         supportedCommodityClassification: [SendableWrapper<CommodityClassification>] = [],
         unsupportedCommodityClassification: [SendableWrapper<CommodityClassification>] = [],
         totalCapacityDimension: SendableWrapper<Dimension>? = nil,
         shipmentStage: [SendableWrapper<ShipmentStage>] = [],
         transportEvent: [SendableWrapper<TransportEvent>] = [],
         responsibleTransportServiceProviderParty: SendableWrapper<Party>? = nil,
         environmentalEmission: [SendableWrapper<EnvironmentalEmission>] = [],
         estimatedDurationPeriod: SendableWrapper<Period>? = nil,
         scheduledServiceFrequency: [SendableWrapper<ServiceFrequency>] = []) {
        self.transportServiceCode = transportServiceCode
        self.tariffClassCode = tariffClassCode
        self.priority = priority
        self.freightRateClassCode = freightRateClassCode
        self.transportationServiceDescription = transportationServiceDescription
        self.transportationServiceDetailsURI = transportationServiceDetailsURI
        self.nominationDate = nominationDate
        self.nominationTime = nominationTime
        self.name = name
        self.sequenceNumeric = sequenceNumeric
        self.transportEquipment = transportEquipment
        self.supportedTransportEquipment = supportedTransportEquipment
        self.unsupportedTransportEquipment = unsupportedTransportEquipment
        self.commodityClassification = commodityClassification
        self.supportedCommodityClassification = supportedCommodityClassification
        self.unsupportedCommodityClassification = unsupportedCommodityClassification
        self.totalCapacityDimension = totalCapacityDimension
        self.shipmentStage = shipmentStage
        self.transportEvent = transportEvent
        self.responsibleTransportServiceProviderParty = responsibleTransportServiceProviderParty
        self.environmentalEmission = environmentalEmission
        self.estimatedDurationPeriod = estimatedDurationPeriod
        self.scheduledServiceFrequency = scheduledServiceFrequency
    }
    
    public enum CodingKeys: String, CodingKey {
        case transportServiceCode = "TransportServiceCode"
        case tariffClassCode = "TariffClassCode"
        case priority = "Priority"
        case freightRateClassCode = "FreightRateClassCode"
        case transportationServiceDescription = "TransportationServiceDescription"
        case transportationServiceDetailsURI = "TransportationServiceDetailsURI"
        case nominationDate = "NominationDate"
        case nominationTime = "NominationTime"
        case name = "Name"
        case sequenceNumeric = "SequenceNumeric"
        case transportEquipment = "TransportEquipment"
        case supportedTransportEquipment = "SupportedTransportEquipment"
        case unsupportedTransportEquipment = "UnsupportedTransportEquipment"
        case commodityClassification = "CommodityClassification"
        case supportedCommodityClassification = "SupportedCommodityClassification"
        case unsupportedCommodityClassification = "UnsupportedCommodityClassification"
        case totalCapacityDimension = "TotalCapacityDimension"
        case shipmentStage = "ShipmentStage"
        case transportEvent = "TransportEvent"
        case responsibleTransportServiceProviderParty = "ResponsibleTransportServiceProviderParty"
        case environmentalEmission = "EnvironmentalEmission"
        case estimatedDurationPeriod = "EstimatedDurationPeriod"
        case scheduledServiceFrequency = "ScheduledServiceFrequency"
    }
}

extension TransportationService: CreateEmpty {
    public nonisolated static func empty() -> TransportationService {
        TransportationService()
    }
}
