// 
// Address.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Address: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .addressTypeCode,
        .addressFormatCode,
        .postbox,
        .floor,
        .room,
        .streetName,
        .additionalStreetName,
        .blockName,
        .buildingName,
        .buildingNumber,
        .inhouseMail,
        .department,
        .markAttention,
        .markCare,
        .plotIdentification,
        .citySubdivisionName,
        .cityName,
        .postalZone,
        .countrySubentity,
        .countrySubentityCode,
        .region,
        .district,
        .timezoneOffset,
        .addressLine,
        .country,
        .locationCoordinate
]
    public var hasNilValue: Bool {(id == nil)
        || (addressTypeCode == nil)
        || (addressFormatCode == nil)
        || (postbox == nil)
        || (floor == nil)
        || (room == nil)
        || (streetName == nil)
        || (additionalStreetName == nil)
        || (blockName == nil)
        || (buildingName == nil)
        || (buildingNumber == nil)
        || (inhouseMail == nil)
        || (department == nil)
        || (markAttention == nil)
        || (markCare == nil)
        || (plotIdentification == nil)
        || (citySubdivisionName == nil)
        || (cityName == nil)
        || (postalZone == nil)
        || (countrySubentity == nil)
        || (countrySubentityCode == nil)
        || (region == nil)
        || (district == nil)
        || (timezoneOffset == nil)
        || (country == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .addressTypeCode: return "Address Type Code"
            case .addressFormatCode: return "Address Format Code"
            case .postbox: return "Postbox"
            case .floor: return "Floor"
            case .room: return "Room"
            case .streetName: return "Street Name"
            case .additionalStreetName: return "Additional Street Name"
            case .blockName: return "Block Name"
            case .buildingName: return "Building Name"
            case .buildingNumber: return "Building Number"
            case .inhouseMail: return "Inhouse Mail"
            case .department: return "Department"
            case .markAttention: return "Mark Attention"
            case .markCare: return "Mark Care"
            case .plotIdentification: return "Plot Identification"
            case .citySubdivisionName: return "City Subdivision Name"
            case .cityName: return "City Name"
            case .postalZone: return "Postal Zone"
            case .countrySubentity: return "Country Subentity"
            case .countrySubentityCode: return "Country Subentity Code"
            case .region: return "Region"
            case .district: return "District"
            case .timezoneOffset: return "Timezone Offset"
            case .addressLine: return "Address Line"
            case .country: return "Country"
            case .locationCoordinate: return "Location Coordinate"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this address within an agreed scheme of address identifiers."
            case .addressTypeCode: return "A mutually agreed code signifying the type of this address."
            case .addressFormatCode: return "A mutually agreed code signifying the format of this address."
            case .postbox: return "A post office box number registered for postal delivery by a postal service provider."
            case .floor: return "An identifiable floor of a building."
            case .room: return "An identifiable room, suite, or apartment of a building."
            case .streetName: return "The name of the street, road, avenue, way, etc. to which the number of the building is attached."
            case .additionalStreetName: return "An additional street name used to further clarify the address."
            case .blockName: return "The name of the block (an area surrounded by streets and usually containing several buildings) in which this address is located."
            case .buildingName: return "The name of a building."
            case .buildingNumber: return "The number of a building within the street."
            case .inhouseMail: return "The specific identifable location within a building where mail is delivered."
            case .department: return "The department of the addressee."
            case .markAttention: return "The name, expressed as text, of a person or department in an organization to whose attention incoming mail is directed; corresponds to the printed forms \"for the attention of\", \"FAO\", and ATTN:\"."
            case .markCare: return "The name, expressed as text, of a person or organization at this address into whose care incoming mail is entrusted; corresponds to the printed forms \"care of\" and \"c/o\"."
            case .plotIdentification: return "An identifier (e.g., a parcel number) for the piece of land associated with this address."
            case .citySubdivisionName: return "The name of the subdivision of a city, town, or village in which this address is located, such as the name of its district or borough."
            case .cityName: return "The name of a city, town, or village."
            case .postalZone: return "The postal identifier for this address according to the relevant national postal service, such as a ZIP code or Post Code."
            case .countrySubentity: return "The political or administrative division of a country in which this address is located, such as the name of its county, province, or state, expressed as text."
            case .countrySubentityCode: return "The political or administrative division of a country in which this address is located, such as a county, province, or state, expressed as a code (typically nationally agreed)."
            case .region: return "The recognized geographic or economic region or group of countries in which this address is located."
            case .district: return "The district or geographical division of a country or region in which this address is located."
            case .timezoneOffset: return "The time zone in which this address is located (as an offset from Universal Coordinated Time (UTC)) at the time of exchange."
            case .addressLine: return "An unstructured address line."
            case .country: return "The country in which this address is situated."
            case .locationCoordinate: return "The geographical coordinates of this address."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .addressTypeCode: result = addressTypeCode?.search(_search)
                case .addressFormatCode: result = addressFormatCode?.search(_search)
                case .postbox: result = postbox?.search(_search)
                case .floor: result = floor?.search(_search)
                case .room: result = room?.search(_search)
                case .streetName: result = streetName?.search(_search)
                case .additionalStreetName: result = additionalStreetName?.search(_search)
                case .blockName: result = blockName?.search(_search)
                case .buildingName: result = buildingName?.search(_search)
                case .buildingNumber: result = buildingNumber?.search(_search)
                case .inhouseMail: result = inhouseMail?.search(_search)
                case .department: result = department?.search(_search)
                case .markAttention: result = markAttention?.search(_search)
                case .markCare: result = markCare?.search(_search)
                case .plotIdentification: result = plotIdentification?.search(_search)
                case .citySubdivisionName: result = citySubdivisionName?.search(_search)
                case .cityName: result = cityName?.search(_search)
                case .postalZone: result = postalZone?.search(_search)
                case .countrySubentity: result = countrySubentity?.search(_search)
                case .countrySubentityCode: result = countrySubentityCode?.search(_search)
                case .region: result = region?.search(_search)
                case .district: result = district?.search(_search)
                case .timezoneOffset: result = timezoneOffset?.search(_search)
                case .addressLine: result = addressLine.compactMap({ $0.search(_search) }).first
                case .country: result = country?.search(_search)
                case .locationCoordinate: result = locationCoordinate.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .addressTypeCode: presentableView(for: addressTypeCode, title: _title, details: _details, show: show, search: _search)
                case .addressFormatCode: presentableView(for: addressFormatCode, title: _title, details: _details, show: show, search: _search)
                case .postbox: presentableView(for: postbox, title: _title, details: _details, show: show, search: _search)
                case .floor: presentableView(for: floor, title: _title, details: _details, show: show, search: _search)
                case .room: presentableView(for: room, title: _title, details: _details, show: show, search: _search)
                case .streetName: presentableView(for: streetName, title: _title, details: _details, show: show, search: _search)
                case .additionalStreetName: presentableView(for: additionalStreetName, title: _title, details: _details, show: show, search: _search)
                case .blockName: presentableView(for: blockName, title: _title, details: _details, show: show, search: _search)
                case .buildingName: presentableView(for: buildingName, title: _title, details: _details, show: show, search: _search)
                case .buildingNumber: presentableView(for: buildingNumber, title: _title, details: _details, show: show, search: _search)
                case .inhouseMail: presentableView(for: inhouseMail, title: _title, details: _details, show: show, search: _search)
                case .department: presentableView(for: department, title: _title, details: _details, show: show, search: _search)
                case .markAttention: presentableView(for: markAttention, title: _title, details: _details, show: show, search: _search)
                case .markCare: presentableView(for: markCare, title: _title, details: _details, show: show, search: _search)
                case .plotIdentification: presentableView(for: plotIdentification, title: _title, details: _details, show: show, search: _search)
                case .citySubdivisionName: presentableView(for: citySubdivisionName, title: _title, details: _details, show: show, search: _search)
                case .cityName: presentableView(for: cityName, title: _title, details: _details, show: show, search: _search)
                case .postalZone: presentableView(for: postalZone, title: _title, details: _details, show: show, search: _search)
                case .countrySubentity: presentableView(for: countrySubentity, title: _title, details: _details, show: show, search: _search)
                case .countrySubentityCode: presentableView(for: countrySubentityCode, title: _title, details: _details, show: show, search: _search)
                case .region: presentableView(for: region, title: _title, details: _details, show: show, search: _search)
                case .district: presentableView(for: district, title: _title, details: _details, show: show, search: _search)
                case .timezoneOffset: presentableView(for: timezoneOffset, title: _title, details: _details, show: show, search: _search)
                case .addressLine: presentableView(for: addressLine.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .country: presentableView(for: country, title: _title, details: _details, show: show, search: _search)
                case .locationCoordinate: presentableView(for: locationCoordinate.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Address: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = addressTypeCode?.search(string) { return result }
        if let result = addressFormatCode?.search(string) { return result }
        if let result = postbox?.search(string) { return result }
        if let result = floor?.search(string) { return result }
        if let result = room?.search(string) { return result }
        if let result = streetName?.search(string) { return result }
        if let result = additionalStreetName?.search(string) { return result }
        if let result = blockName?.search(string) { return result }
        if let result = buildingName?.search(string) { return result }
        if let result = buildingNumber?.search(string) { return result }
        if let result = inhouseMail?.search(string) { return result }
        if let result = department?.search(string) { return result }
        if let result = markAttention?.search(string) { return result }
        if let result = markCare?.search(string) { return result }
        if let result = plotIdentification?.search(string) { return result }
        if let result = citySubdivisionName?.search(string) { return result }
        if let result = cityName?.search(string) { return result }
        if let result = postalZone?.search(string) { return result }
        if let result = countrySubentity?.search(string) { return result }
        if let result = countrySubentityCode?.search(string) { return result }
        if let result = region?.search(string) { return result }
        if let result = district?.search(string) { return result }
        if let result = timezoneOffset?.search(string) { return result }
        if let result = addressLine.compactMap({ $0.search(string)  }).first { return result }
        if let result = country?.search(string) { return result }
        if let result = locationCoordinate.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
