// 
// AllowanceCharge.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.AllowanceCharge: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .chargeIndicator,
        .allowanceChargeReasonCode,
        .allowanceChargeReason,
        .multiplierFactorNumeric,
        .prepaidIndicator,
        .sequenceNumeric,
        .amount,
        .baseAmount,
        .accountingCostCode,
        .accountingCost,
        .perUnitAmount,
        .taxCategory,
        .taxTotal,
        .paymentMeans
]
    public var hasNilValue: Bool {(id == nil)
        || (allowanceChargeReasonCode == nil)
        || (multiplierFactorNumeric == nil)
        || (prepaidIndicator == nil)
        || (sequenceNumeric == nil)
        || (baseAmount == nil)
        || (accountingCostCode == nil)
        || (accountingCost == nil)
        || (perUnitAmount == nil)
        || (taxTotal == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .chargeIndicator: return "Charge Indicator"
            case .allowanceChargeReasonCode: return "Allowance Charge Reason Code"
            case .allowanceChargeReason: return "Allowance Charge Reason"
            case .multiplierFactorNumeric: return "Multiplier Factor"
            case .prepaidIndicator: return "Prepaid Indicator"
            case .sequenceNumeric: return "Sequence"
            case .amount: return "Amount"
            case .baseAmount: return "Base Amount"
            case .accountingCostCode: return "Accounting Cost Code"
            case .accountingCost: return "Accounting Cost"
            case .perUnitAmount: return "Per Unit Amount"
            case .taxCategory: return "Tax Category"
            case .taxTotal: return "Tax Total"
            case .paymentMeans: return "Payment Means"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this allowance or charge."
            case .chargeIndicator: return "An indicator that this AllowanceCharge describes a charge (true) or a discount (false)."
            case .allowanceChargeReasonCode: return "A mutually agreed code signifying the reason for this allowance or charge."
            case .allowanceChargeReason: return "The reason for this allowance or charge."
            case .multiplierFactorNumeric: return "A number by which the base amount is multiplied to calculate the actual amount of this allowance or charge."
            case .prepaidIndicator: return "An indicator that this allowance or charge is prepaid (true) or not (false)."
            case .sequenceNumeric: return "A number indicating the order of this allowance or charge in the sequence of calculations applied when there are multiple allowances or charges."
            case .amount: return "The monetary amount of this allowance or charge to be applied."
            case .baseAmount: return "The monetary amount to which the multiplier factor is applied in calculating the amount of this allowance or charge."
            case .accountingCostCode: return "The accounting cost centre used by the buyer to account for this allowance or charge, expressed as a code."
            case .accountingCost: return "The accounting cost centre used by the buyer to account for this allowance or charge, expressed as text."
            case .perUnitAmount: return "The allowance or charge per item; the total allowance or charge is calculated by multiplying the per unit amount by the quantity of items, either at the level of the individual transaction line or for the total number of items in the document, depending on the context in which it appears."
            case .taxCategory: return "A tax category applicable to this allowance or charge."
            case .taxTotal: return "The total of all the taxes applicable to this allowance or charge."
            case .paymentMeans: return "A means of payment for this allowance or charge."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .chargeIndicator: result = chargeIndicator.search(_search)
                case .allowanceChargeReasonCode: result = allowanceChargeReasonCode?.search(_search)
                case .allowanceChargeReason: result = allowanceChargeReason.compactMap({ $0.search(_search) }).first
                case .multiplierFactorNumeric: result = multiplierFactorNumeric?.search(_search)
                case .prepaidIndicator: result = prepaidIndicator?.search(_search)
                case .sequenceNumeric: result = sequenceNumeric?.search(_search)
                case .amount: result = amount.search(_search)
                case .baseAmount: result = baseAmount?.search(_search)
                case .accountingCostCode: result = accountingCostCode?.search(_search)
                case .accountingCost: result = accountingCost?.search(_search)
                case .perUnitAmount: result = perUnitAmount?.search(_search)
                case .taxCategory: result = taxCategory.compactMap({ $0.search(_search) }).first
                case .taxTotal: result = taxTotal?.search(_search)
                case .paymentMeans: result = paymentMeans.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .chargeIndicator: presentableView(for: chargeIndicator, title: _title, details: _details, show: show, search: _search)
                case .allowanceChargeReasonCode: presentableView(for: allowanceChargeReasonCode, title: _title, details: _details, show: show, search: _search)
                case .allowanceChargeReason: presentableView(for: allowanceChargeReason.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .multiplierFactorNumeric: presentableView(for: multiplierFactorNumeric, title: _title, details: _details, show: show, search: _search)
                case .prepaidIndicator: presentableView(for: prepaidIndicator, title: _title, details: _details, show: show, search: _search)
                case .sequenceNumeric: presentableView(for: sequenceNumeric, title: _title, details: _details, show: show, search: _search)
                case .amount: presentableView(for: amount, title: _title, details: _details, show: show, search: _search)
                case .baseAmount: presentableView(for: baseAmount, title: _title, details: _details, show: show, search: _search)
                case .accountingCostCode: presentableView(for: accountingCostCode, title: _title, details: _details, show: show, search: _search)
                case .accountingCost: presentableView(for: accountingCost, title: _title, details: _details, show: show, search: _search)
                case .perUnitAmount: presentableView(for: perUnitAmount, title: _title, details: _details, show: show, search: _search)
                case .taxCategory: presentableView(for: taxCategory.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .taxTotal: presentableView(for: taxTotal, title: _title, details: _details, show: show, search: _search)
                case .paymentMeans: presentableView(for: paymentMeans.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.AllowanceCharge: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = chargeIndicator.search(string) { return result }
        if let result = allowanceChargeReasonCode?.search(string) { return result }
        if let result = allowanceChargeReason.compactMap({ $0.search(string)  }).first { return result }
        if let result = multiplierFactorNumeric?.search(string) { return result }
        if let result = prepaidIndicator?.search(string) { return result }
        if let result = sequenceNumeric?.search(string) { return result }
        if let result = amount.search(string) { return result }
        if let result = baseAmount?.search(string) { return result }
        if let result = accountingCostCode?.search(string) { return result }
        if let result = accountingCost?.search(string) { return result }
        if let result = perUnitAmount?.search(string) { return result }
        if let result = taxCategory.compactMap({ $0.search(string)  }).first { return result }
        if let result = taxTotal?.search(string) { return result }
        if let result = paymentMeans.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
