// 
// Attachment.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Attachment: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .embeddedDocumentBinaryObject,
        .externalReference
]
    public var hasNilValue: Bool {(embeddedDocumentBinaryObject == nil)
        || (externalReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .embeddedDocumentBinaryObject: return "Embedded Document"
            case .externalReference: return "External Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .embeddedDocumentBinaryObject: return "A binary large object containing an attached document."
            case .externalReference: return "A reference to an attached document that is external to the document(s) being exchanged."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .embeddedDocumentBinaryObject: result = embeddedDocumentBinaryObject?.search(_search)
                case .externalReference: result = externalReference?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .embeddedDocumentBinaryObject: presentableView(for: embeddedDocumentBinaryObject, title: _title, details: _details, show: show, search: _search)
                case .externalReference: presentableView(for: externalReference, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Attachment: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = embeddedDocumentBinaryObject?.search(string) { return result }
        if let result = externalReference?.search(string) { return result }
        return nil
    }
}
