// 
// BillingReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.BillingReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .invoiceDocumentReference,
        .selfBilledInvoiceDocumentReference,
        .creditNoteDocumentReference,
        .selfBilledCreditNoteDocumentReference,
        .debitNoteDocumentReference,
        .reminderDocumentReference,
        .additionalDocumentReference,
        .billingReferenceLine
]
    public var hasNilValue: Bool {(invoiceDocumentReference == nil)
        || (selfBilledInvoiceDocumentReference == nil)
        || (creditNoteDocumentReference == nil)
        || (selfBilledCreditNoteDocumentReference == nil)
        || (debitNoteDocumentReference == nil)
        || (reminderDocumentReference == nil)
        || (additionalDocumentReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .invoiceDocumentReference: return "Invoice Document Reference"
            case .selfBilledInvoiceDocumentReference: return "Self Billed Invoice Document Reference"
            case .creditNoteDocumentReference: return "Credit Note Document Reference"
            case .selfBilledCreditNoteDocumentReference: return "Self Billed Credit Note Document Reference"
            case .debitNoteDocumentReference: return "Debit Note Document Reference"
            case .reminderDocumentReference: return "Reminder Document Reference"
            case .additionalDocumentReference: return "Additional Document Reference"
            case .billingReferenceLine: return "Billing Reference Line"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .invoiceDocumentReference: return "A reference to an invoice."
            case .selfBilledInvoiceDocumentReference: return "A reference to a self billed invoice."
            case .creditNoteDocumentReference: return "A reference to a credit note."
            case .selfBilledCreditNoteDocumentReference: return "A reference to a self billed credit note."
            case .debitNoteDocumentReference: return "A reference to a debit note."
            case .reminderDocumentReference: return "A reference to a billing reminder."
            case .additionalDocumentReference: return "A reference to an additional document."
            case .billingReferenceLine: return "A reference to a transaction line in the billing document."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .invoiceDocumentReference: result = invoiceDocumentReference?.search(_search)
                case .selfBilledInvoiceDocumentReference: result = selfBilledInvoiceDocumentReference?.search(_search)
                case .creditNoteDocumentReference: result = creditNoteDocumentReference?.search(_search)
                case .selfBilledCreditNoteDocumentReference: result = selfBilledCreditNoteDocumentReference?.search(_search)
                case .debitNoteDocumentReference: result = debitNoteDocumentReference?.search(_search)
                case .reminderDocumentReference: result = reminderDocumentReference?.search(_search)
                case .additionalDocumentReference: result = additionalDocumentReference?.search(_search)
                case .billingReferenceLine: result = billingReferenceLine.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .invoiceDocumentReference: presentableView(for: invoiceDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .selfBilledInvoiceDocumentReference: presentableView(for: selfBilledInvoiceDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .creditNoteDocumentReference: presentableView(for: creditNoteDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .selfBilledCreditNoteDocumentReference: presentableView(for: selfBilledCreditNoteDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .debitNoteDocumentReference: presentableView(for: debitNoteDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .reminderDocumentReference: presentableView(for: reminderDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .additionalDocumentReference: presentableView(for: additionalDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .billingReferenceLine: presentableView(for: billingReferenceLine.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.BillingReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = invoiceDocumentReference?.search(string) { return result }
        if let result = selfBilledInvoiceDocumentReference?.search(string) { return result }
        if let result = creditNoteDocumentReference?.search(string) { return result }
        if let result = selfBilledCreditNoteDocumentReference?.search(string) { return result }
        if let result = debitNoteDocumentReference?.search(string) { return result }
        if let result = reminderDocumentReference?.search(string) { return result }
        if let result = additionalDocumentReference?.search(string) { return result }
        if let result = billingReferenceLine.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
