// 
// BinaryObject.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.BinaryObject: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .value,
        .format,
        .mimeCode,
        .encodingCode,
        .characterSetCode,
        .uri,
        .filename
]
    public var hasNilValue: Bool {(format == nil)
        || (mimeCode == nil)
        || (encodingCode == nil)
        || (characterSetCode == nil)
        || (uri == nil)
        || (filename == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .value: return "Value"
            case .format: return "Format"
            case .mimeCode: return "Mime Code"
            case .encodingCode: return "Encoding Code"
            case .characterSetCode: return "Character Set Code"
            case .uri: return "Uri"
            case .filename: return "Filename"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .value: return nil
            case .format: return nil
            case .mimeCode: return nil
            case .encodingCode: return nil
            case .characterSetCode: return nil
            case .uri: return nil
            case .filename: return nil
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .value: result = value.search(_search)
                case .format: result = format?.search(_search)
                case .mimeCode: result = mimeCode?.search(_search)
                case .encodingCode: result = encodingCode?.search(_search)
                case .characterSetCode: result = characterSetCode?.search(_search)
                case .uri: result = uri?.search(_search)
                case .filename: result = filename?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .value: anyView(for: value, title: _title, details: _details, show: show, search: _search)
                case .format: anyView(for: format, title: _title, details: _details, show: show, search: _search)
                case .mimeCode: anyView(for: mimeCode, title: _title, details: _details, show: show, search: _search)
                case .encodingCode: anyView(for: encodingCode, title: _title, details: _details, show: show, search: _search)
                case .characterSetCode: anyView(for: characterSetCode, title: _title, details: _details, show: show, search: _search)
                case .uri: anyView(for: uri, title: _title, details: _details, show: show, search: _search)
                case .filename: anyView(for: filename, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.BinaryObject: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = value.search(string) { return result }
        if let result = format?.search(string) { return result }
        if let result = mimeCode?.search(string) { return result }
        if let result = encodingCode?.search(string) { return result }
        if let result = characterSetCode?.search(string) { return result }
        if let result = uri?.search(string) { return result }
        if let result = filename?.search(string) { return result }
        return nil
    }
}
