// 
// CardAccount.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.CardAccount: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .primaryAccountNumberID,
        .networkID,
        .cardTypeCode,
        .validityStartDate,
        .expiryDate,
        .issuerID,
        .issueNumberID,
        .cV2ID,
        .cardChipCode,
        .chipApplicationID,
        .holderName
]
    public var hasNilValue: Bool {(cardTypeCode == nil)
        || (validityStartDate == nil)
        || (expiryDate == nil)
        || (issuerID == nil)
        || (issueNumberID == nil)
        || (cV2ID == nil)
        || (cardChipCode == nil)
        || (chipApplicationID == nil)
        || (holderName == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .primaryAccountNumberID: return "Primary Account Number"
            case .networkID: return "Network"
            case .cardTypeCode: return "Card Type Code"
            case .validityStartDate: return "Validity Start Date"
            case .expiryDate: return "Expiry Date"
            case .issuerID: return "Issuer"
            case .issueNumberID: return "Issue Number"
            case .cV2ID: return "CV2"
            case .cardChipCode: return "Card Chip Code"
            case .chipApplicationID: return "Chip Application"
            case .holderName: return "Holder"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .primaryAccountNumberID: return "An identifier of the card (e.g., the Primary Account Number (PAN))."
            case .networkID: return "An identifier for the financial service network provider of the card."
            case .cardTypeCode: return "A mutually agreed code signifying the type of card. Examples of types are \"debit\", \"credit\" and \"purchasing\""
            case .validityStartDate: return "The date from which the card is valid."
            case .expiryDate: return "The date on which the card expires."
            case .issuerID: return "An identifier for the institution issuing the card."
            case .issueNumberID: return "An identifier for the card, specified by the issuer."
            case .cV2ID: return "An identifier for the Card Verification Value (often found on the reverse of the card itself)."
            case .cardChipCode: return "A mutually agreed code to distinguish between CHIP and MAG STRIPE cards."
            case .chipApplicationID: return "An identifier on the chip card for the application that provides the quoted information; an AID (application ID)."
            case .holderName: return "The name of the cardholder."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .primaryAccountNumberID: result = primaryAccountNumberID.search(_search)
                case .networkID: result = networkID.search(_search)
                case .cardTypeCode: result = cardTypeCode?.search(_search)
                case .validityStartDate: result = validityStartDate?.search(_search)
                case .expiryDate: result = expiryDate?.search(_search)
                case .issuerID: result = issuerID?.search(_search)
                case .issueNumberID: result = issueNumberID?.search(_search)
                case .cV2ID: result = cV2ID?.search(_search)
                case .cardChipCode: result = cardChipCode?.search(_search)
                case .chipApplicationID: result = chipApplicationID?.search(_search)
                case .holderName: result = holderName?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .primaryAccountNumberID: presentableView(for: primaryAccountNumberID, title: _title, details: _details, show: show, search: _search)
                case .networkID: presentableView(for: networkID, title: _title, details: _details, show: show, search: _search)
                case .cardTypeCode: presentableView(for: cardTypeCode, title: _title, details: _details, show: show, search: _search)
                case .validityStartDate: anyView(for: validityStartDate, title: _title, details: _details, show: show, search: _search)
                case .expiryDate: anyView(for: expiryDate, title: _title, details: _details, show: show, search: _search)
                case .issuerID: presentableView(for: issuerID, title: _title, details: _details, show: show, search: _search)
                case .issueNumberID: presentableView(for: issueNumberID, title: _title, details: _details, show: show, search: _search)
                case .cV2ID: presentableView(for: cV2ID, title: _title, details: _details, show: show, search: _search)
                case .cardChipCode: presentableView(for: cardChipCode, title: _title, details: _details, show: show, search: _search)
                case .chipApplicationID: presentableView(for: chipApplicationID, title: _title, details: _details, show: show, search: _search)
                case .holderName: presentableView(for: holderName, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.CardAccount: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = validityStartDate?.search(string) { return result }
        if let result = expiryDate?.search(string) { return result }
        if let result = primaryAccountNumberID.search(string) { return result }
        if let result = networkID.search(string) { return result }
        if let result = cardTypeCode?.search(string) { return result }
        if let result = issuerID?.search(string) { return result }
        if let result = issueNumberID?.search(string) { return result }
        if let result = cV2ID?.search(string) { return result }
        if let result = cardChipCode?.search(string) { return result }
        if let result = chipApplicationID?.search(string) { return result }
        if let result = holderName?.search(string) { return result }
        return nil
    }
}
