// 
// Certificate.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Certificate: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .certificateTypeCode,
        .certificateType,
        .remarks,
        .issuerParty,
        .documentReference,
        .signature
]
    public var hasNilValue: Bool {
        false
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .certificateTypeCode: return "Certificate Type Code"
            case .certificateType: return "Certificate Type"
            case .remarks: return "Remarks"
            case .issuerParty: return "Issuer Party"
            case .documentReference: return "Document Reference"
            case .signature: return "Signature"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this certificate."
            case .certificateTypeCode: return "The type of this certificate, expressed as a code. The type specifies what array it belongs to, e.g.. Environmental, security, health improvement etc."
            case .certificateType: return "The type of this certificate, expressed as a code. The type specifies what array it belongs to, e.g.. Environmental, security, health improvement etc."
            case .remarks: return "Remarks by the applicant for this certificate."
            case .issuerParty: return "The authorized organization that issued this certificate, the provider of the certificate."
            case .documentReference: return "A reference to a document relevant to this certificate or an application for this certificate."
            case .signature: return "A signature applied to this certificate."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .certificateTypeCode: result = certificateTypeCode.search(_search)
                case .certificateType: result = certificateType.search(_search)
                case .remarks: result = remarks.compactMap({ $0.search(_search) }).first
                case .issuerParty: result = issuerParty.search(_search)
                case .documentReference: result = documentReference.compactMap({ $0.search(_search) }).first
                case .signature: result = signature.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .certificateTypeCode: presentableView(for: certificateTypeCode, title: _title, details: _details, show: show, search: _search)
                case .certificateType: presentableView(for: certificateType, title: _title, details: _details, show: show, search: _search)
                case .remarks: presentableView(for: remarks.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .issuerParty: presentableView(for: issuerParty, title: _title, details: _details, show: show, search: _search)
                case .documentReference: presentableView(for: documentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .signature: presentableView(for: signature.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Certificate: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id.search(string) { return result }
        if let result = certificateTypeCode.search(string) { return result }
        if let result = certificateType.search(string) { return result }
        if let result = remarks.compactMap({ $0.search(string)  }).first { return result }
        if let result = issuerParty.search(string) { return result }
        if let result = documentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = signature.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
