// 
// CommodityClassification.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.CommodityClassification: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .natureCode,
        .cargoTypeCode,
        .commodityCode,
        .itemClassificationCode
]
    public var hasNilValue: Bool {(natureCode == nil)
        || (cargoTypeCode == nil)
        || (commodityCode == nil)
        || (itemClassificationCode == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .natureCode: return "Nature Code"
            case .cargoTypeCode: return "Cargo Type Code"
            case .commodityCode: return "Commodity Code"
            case .itemClassificationCode: return "Item Classification Code"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .natureCode: return "A code defined by a specific maintenance agency signifying the high-level nature of the commodity."
            case .cargoTypeCode: return "A mutually agreed code signifying the type of cargo for purposes of commodity classification."
            case .commodityCode: return "The harmonized international commodity code for cross border and regulatory (customs and trade statistics) purposes."
            case .itemClassificationCode: return "A code signifying the trade classification of the commodity."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .natureCode: result = natureCode?.search(_search)
                case .cargoTypeCode: result = cargoTypeCode?.search(_search)
                case .commodityCode: result = commodityCode?.search(_search)
                case .itemClassificationCode: result = itemClassificationCode?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .natureCode: presentableView(for: natureCode, title: _title, details: _details, show: show, search: _search)
                case .cargoTypeCode: presentableView(for: cargoTypeCode, title: _title, details: _details, show: show, search: _search)
                case .commodityCode: presentableView(for: commodityCode, title: _title, details: _details, show: show, search: _search)
                case .itemClassificationCode: presentableView(for: itemClassificationCode, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.CommodityClassification: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = natureCode?.search(string) { return result }
        if let result = cargoTypeCode?.search(string) { return result }
        if let result = commodityCode?.search(string) { return result }
        if let result = itemClassificationCode?.search(string) { return result }
        return nil
    }
}
