// 
// Communication.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Communication: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .channelCode,
        .channel,
        .value
]
    public var hasNilValue: Bool {(channelCode == nil)
        || (channel == nil)
        || (value == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .channelCode: return "Channel Code"
            case .channel: return "Channel"
            case .value: return "Value"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .channelCode: return "The method of communication, expressed as a code."
            case .channel: return "The method of communication, expressed as text."
            case .value: return "An identifying value (phone number, email address, etc.) for this channel of communication"
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .channelCode: result = channelCode?.search(_search)
                case .channel: result = channel?.search(_search)
                case .value: result = value?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .channelCode: presentableView(for: channelCode, title: _title, details: _details, show: show, search: _search)
                case .channel: presentableView(for: channel, title: _title, details: _details, show: show, search: _search)
                case .value: presentableView(for: value, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Communication: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = channelCode?.search(string) { return result }
        if let result = channel?.search(string) { return result }
        if let result = value?.search(string) { return result }
        return nil
    }
}
