// 
// Condition.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Condition: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .attributeID,
        .measure,
        .description,
        .minimumMeasure,
        .maximumMeasure,
        .value,
        .languageID,
        .languageLocaleID
]
    public var hasNilValue: Bool {(measure == nil)
        || (minimumMeasure == nil)
        || (maximumMeasure == nil)
        || (languageID == nil)
        || (languageLocaleID == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .attributeID: return "Attribute Identifier"
            case .measure: return "Measure"
            case .description: return "Description"
            case .minimumMeasure: return "Minimum Measure"
            case .maximumMeasure: return "Maximum Measure"
            case .value: return "Value"
            case .languageID: return "Language Id"
            case .languageLocaleID: return "Language Locale Id"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .attributeID: return "An identifier for the attribute that applies to the condition."
            case .measure: return "The measurement value."
            case .description: return "Text describing the attribute that applies to the condition."
            case .minimumMeasure: return "The minimum value in a range of measurement for this condition."
            case .maximumMeasure: return "The maximum value in a range of measurement for this condition."
            case .value: return nil
            case .languageID: return nil
            case .languageLocaleID: return nil
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .attributeID: result = attributeID.search(_search)
                case .measure: result = measure?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
                case .minimumMeasure: result = minimumMeasure?.search(_search)
                case .maximumMeasure: result = maximumMeasure?.search(_search)
                case .value: result = value.search(_search)
                case .languageID: result = languageID?.search(_search)
                case .languageLocaleID: result = languageLocaleID?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .attributeID: presentableView(for: attributeID, title: _title, details: _details, show: show, search: _search)
                case .measure: presentableView(for: measure, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .minimumMeasure: presentableView(for: minimumMeasure, title: _title, details: _details, show: show, search: _search)
                case .maximumMeasure: presentableView(for: maximumMeasure, title: _title, details: _details, show: show, search: _search)
                case .value: anyView(for: value, title: _title, details: _details, show: show, search: _search)
                case .languageID: anyView(for: languageID, title: _title, details: _details, show: show, search: _search)
                case .languageLocaleID: anyView(for: languageLocaleID, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Condition: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = value.search(string) { return result }
        if let result = languageID?.search(string) { return result }
        if let result = languageLocaleID?.search(string) { return result }
        if let result = attributeID.search(string) { return result }
        if let result = measure?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        if let result = minimumMeasure?.search(string) { return result }
        if let result = maximumMeasure?.search(string) { return result }
        return nil
    }
}
