// 
// Contact.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Contact: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .name,
        .telephone,
        .telefax,
        .electronicMail,
        .note,
        .otherCommunication
]
    public var hasNilValue: Bool {(id == nil)
        || (name == nil)
        || (telephone == nil)
        || (telefax == nil)
        || (electronicMail == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .name: return "Name"
            case .telephone: return "Telephone"
            case .telefax: return "Telefax"
            case .electronicMail: return "Electronic Mail"
            case .note: return "Note"
            case .otherCommunication: return "Other Communication"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this contact."
            case .name: return "The name of this contact. It is recommended that this be used for a functional name and not a personal name."
            case .telephone: return "The primary telephone number of this contact."
            case .telefax: return "The primary fax number of this contact."
            case .electronicMail: return "The primary email address of this contact."
            case .note: return "Free-form text conveying information that is not contained explicitly in other structures; in particular, a textual description of the circumstances under which this contact can be used (e.g., \"emergency\" or \"after hours\")."
            case .otherCommunication: return "Another means of communication with this contact."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .name: result = name?.search(_search)
                case .telephone: result = telephone?.search(_search)
                case .telefax: result = telefax?.search(_search)
                case .electronicMail: result = electronicMail?.search(_search)
                case .note: result = note.compactMap({ $0.search(_search) }).first
                case .otherCommunication: result = otherCommunication.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .telephone: presentableView(for: telephone, title: _title, details: _details, show: show, search: _search)
                case .telefax: presentableView(for: telefax, title: _title, details: _details, show: show, search: _search)
                case .electronicMail: presentableView(for: electronicMail, title: _title, details: _details, show: show, search: _search)
                case .note: presentableView(for: note.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .otherCommunication: presentableView(for: otherCommunication.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Contact: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = name?.search(string) { return result }
        if let result = telephone?.search(string) { return result }
        if let result = telefax?.search(string) { return result }
        if let result = electronicMail?.search(string) { return result }
        if let result = note.compactMap({ $0.search(string)  }).first { return result }
        if let result = otherCommunication.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
