// 
// Contract.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Contract: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .issueDate,
        .issueTime,
        .nominationDate,
        .nominationTime,
        .contractTypeCode,
        .contractType,
        .note,
        .versionID,
        .description,
        .validityPeriod,
        .contractDocumentReference,
        .nominationPeriod,
        .contractualDelivery
]
    public var hasNilValue: Bool {(id == nil)
        || (issueDate == nil)
        || (issueTime == nil)
        || (nominationDate == nil)
        || (nominationTime == nil)
        || (contractTypeCode == nil)
        || (contractType == nil)
        || (versionID == nil)
        || (validityPeriod == nil)
        || (nominationPeriod == nil)
        || (contractualDelivery == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .issueDate: return "Issue Date"
            case .issueTime: return "Issue Time"
            case .nominationDate: return "Nomination Date"
            case .nominationTime: return "Nomination Time"
            case .contractTypeCode: return "Contract Type Code"
            case .contractType: return "Contract Type"
            case .note: return "Note"
            case .versionID: return "Version"
            case .description: return "Description"
            case .validityPeriod: return "Validity Period"
            case .contractDocumentReference: return "Contract Document Reference"
            case .nominationPeriod: return "Nomination Period"
            case .contractualDelivery: return "Contractual Delivery"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this contract."
            case .issueDate: return "The date on which this contract was issued."
            case .issueTime: return "The time at which this contract was issued."
            case .nominationDate: return "In a transportation contract, the deadline date by which the services referred to in the transport execution plan have to be booked. For example, if this service is a carrier service scheduled for Wednesday 16 February 2011 at 10 a.m. CET, the nomination date might be Tuesday15 February 2011."
            case .nominationTime: return "In a transportation contract, the deadline time by which the services referred to in the transport execution plan have to be booked. For example, if this service is a carrier service scheduled for Wednesday 16 February 2011 at 10 a.m. CET, the nomination date might be Tuesday15 February 2011 and the nomination time 4 p.m. at the latest."
            case .contractTypeCode: return "The type of this contract, expressed as a code, such as \"Cost plus award fee\" and \"Cost plus fixed fee\" from UNCEFACT Contract Type code list."
            case .contractType: return "The type of this contract, expressed as text, such as \"Cost plus award fee\" and \"Cost plus fixed fee\" from UNCEFACT Contract Type code list."
            case .note: return "Free-form text conveying information that is not contained explicitly in other structures."
            case .versionID: return "An identifier for the current version of this contract."
            case .description: return "Text describing this contract."
            case .validityPeriod: return "The period during which this contract is valid."
            case .contractDocumentReference: return "A reference to a contract document."
            case .nominationPeriod: return "In a transportation contract, the period required to book the services specified in the contract before the services can begin."
            case .contractualDelivery: return "In a transportation contract, the delivery of the services required to book the services specified in the contract."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .issueDate: result = issueDate?.search(_search)
                case .issueTime: result = issueTime?.search(_search)
                case .nominationDate: result = nominationDate?.search(_search)
                case .nominationTime: result = nominationTime?.search(_search)
                case .contractTypeCode: result = contractTypeCode?.search(_search)
                case .contractType: result = contractType?.search(_search)
                case .note: result = note.compactMap({ $0.search(_search) }).first
                case .versionID: result = versionID?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
                case .validityPeriod: result = validityPeriod?.search(_search)
                case .contractDocumentReference: result = contractDocumentReference.compactMap({ $0.search(_search) }).first
                case .nominationPeriod: result = nominationPeriod?.search(_search)
                case .contractualDelivery: result = contractualDelivery?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .issueDate: anyView(for: issueDate, title: _title, details: _details, show: show, search: _search)
                case .issueTime: anyView(for: issueTime, title: _title, details: _details, show: show, search: _search)
                case .nominationDate: anyView(for: nominationDate, title: _title, details: _details, show: show, search: _search)
                case .nominationTime: anyView(for: nominationTime, title: _title, details: _details, show: show, search: _search)
                case .contractTypeCode: presentableView(for: contractTypeCode, title: _title, details: _details, show: show, search: _search)
                case .contractType: presentableView(for: contractType, title: _title, details: _details, show: show, search: _search)
                case .note: presentableView(for: note.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .versionID: presentableView(for: versionID, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .validityPeriod: presentableView(for: validityPeriod, title: _title, details: _details, show: show, search: _search)
                case .contractDocumentReference: presentableView(for: contractDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .nominationPeriod: presentableView(for: nominationPeriod, title: _title, details: _details, show: show, search: _search)
                case .contractualDelivery: presentableView(for: contractualDelivery, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Contract: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = issueDate?.search(string) { return result }
        if let result = issueTime?.search(string) { return result }
        if let result = nominationDate?.search(string) { return result }
        if let result = nominationTime?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = contractTypeCode?.search(string) { return result }
        if let result = contractType?.search(string) { return result }
        if let result = note.compactMap({ $0.search(string)  }).first { return result }
        if let result = versionID?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        if let result = validityPeriod?.search(string) { return result }
        if let result = contractDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = nominationPeriod?.search(string) { return result }
        if let result = contractualDelivery?.search(string) { return result }
        return nil
    }
}
