// 
// CorporateRegistrationScheme.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.CorporateRegistrationScheme: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .name,
        .corporateRegistrationTypeCode,
        .jurisdictionRegionAddress
]
    public var hasNilValue: Bool {(id == nil)
        || (name == nil)
        || (corporateRegistrationTypeCode == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .name: return "Name"
            case .corporateRegistrationTypeCode: return "Corporate Registration Type Code"
            case .jurisdictionRegionAddress: return "Jurisdiction Region Address"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this registration scheme."
            case .name: return "The name of this registration scheme."
            case .corporateRegistrationTypeCode: return "A code signifying the type of this registration scheme."
            case .jurisdictionRegionAddress: return "A geographic area in which this registration scheme applies."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .name: result = name?.search(_search)
                case .corporateRegistrationTypeCode: result = corporateRegistrationTypeCode?.search(_search)
                case .jurisdictionRegionAddress: result = jurisdictionRegionAddress.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .corporateRegistrationTypeCode: presentableView(for: corporateRegistrationTypeCode, title: _title, details: _details, show: show, search: _search)
                case .jurisdictionRegionAddress: presentableView(for: jurisdictionRegionAddress.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.CorporateRegistrationScheme: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = name?.search(string) { return result }
        if let result = corporateRegistrationTypeCode?.search(string) { return result }
        if let result = jurisdictionRegionAddress.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
