// 
// CustomerParty.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.CustomerParty: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .customerAssignedAccountID,
        .supplierAssignedAccountID,
        .additionalAccountID,
        .party,
        .deliveryContact,
        .accountingContact,
        .buyerContact
]
    public var hasNilValue: Bool {(customerAssignedAccountID == nil)
        || (supplierAssignedAccountID == nil)
        || (party == nil)
        || (deliveryContact == nil)
        || (accountingContact == nil)
        || (buyerContact == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .customerAssignedAccountID: return "Customer Assigned Account Identifier"
            case .supplierAssignedAccountID: return "Supplier Assigned Account Identifier"
            case .additionalAccountID: return "Additional Account Identifier"
            case .party: return "Party"
            case .deliveryContact: return "Delivery Contact"
            case .accountingContact: return "Accounting Contact"
            case .buyerContact: return "Buyer Contact"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .customerAssignedAccountID: return "An identifier for the customer's account, assigned by the customer itself."
            case .supplierAssignedAccountID: return "An identifier for the customer's account, assigned by the supplier."
            case .additionalAccountID: return "An identifier for the customer's account, assigned by a third party."
            case .party: return "The customer party itself."
            case .deliveryContact: return "A customer contact for deliveries."
            case .accountingContact: return "A customer contact for accounting."
            case .buyerContact: return "A customer contact for purchasing."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .customerAssignedAccountID: result = customerAssignedAccountID?.search(_search)
                case .supplierAssignedAccountID: result = supplierAssignedAccountID?.search(_search)
                case .additionalAccountID: result = additionalAccountID.compactMap({ $0.search(_search) }).first
                case .party: result = party?.search(_search)
                case .deliveryContact: result = deliveryContact?.search(_search)
                case .accountingContact: result = accountingContact?.search(_search)
                case .buyerContact: result = buyerContact?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .customerAssignedAccountID: presentableView(for: customerAssignedAccountID, title: _title, details: _details, show: show, search: _search)
                case .supplierAssignedAccountID: presentableView(for: supplierAssignedAccountID, title: _title, details: _details, show: show, search: _search)
                case .additionalAccountID: presentableView(for: additionalAccountID.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .party: presentableView(for: party, title: _title, details: _details, show: show, search: _search)
                case .deliveryContact: presentableView(for: deliveryContact, title: _title, details: _details, show: show, search: _search)
                case .accountingContact: presentableView(for: accountingContact, title: _title, details: _details, show: show, search: _search)
                case .buyerContact: presentableView(for: buyerContact, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.CustomerParty: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = customerAssignedAccountID?.search(string) { return result }
        if let result = supplierAssignedAccountID?.search(string) { return result }
        if let result = additionalAccountID.compactMap({ $0.search(string)  }).first { return result }
        if let result = party?.search(string) { return result }
        if let result = deliveryContact?.search(string) { return result }
        if let result = accountingContact?.search(string) { return result }
        if let result = buyerContact?.search(string) { return result }
        return nil
    }
}
