// 
// Delivery.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Delivery: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .quantity,
        .minimumQuantity,
        .maximumQuantity,
        .actualDeliveryDate,
        .actualDeliveryTime,
        .latestDeliveryDate,
        .latestDeliveryTime,
        .releaseID,
        .trackingID,
        .deliveryAddress,
        .deliveryLocation,
        .alternativeDeliveryLocation,
        .requestedDeliveryPeriod,
        .promisedDeliveryPeriod,
        .estimatedDeliveryPeriod,
        .carrierParty,
        .deliveryParty,
        .notifyParty,
        .despatch,
        .deliveryTerms,
        .minimumDeliveryUnit,
        .maximumDeliveryUnit,
        .shipment
]
    public var hasNilValue: Bool {(id == nil)
        || (quantity == nil)
        || (minimumQuantity == nil)
        || (maximumQuantity == nil)
        || (actualDeliveryDate == nil)
        || (actualDeliveryTime == nil)
        || (latestDeliveryDate == nil)
        || (latestDeliveryTime == nil)
        || (releaseID == nil)
        || (trackingID == nil)
        || (deliveryAddress == nil)
        || (deliveryLocation == nil)
        || (alternativeDeliveryLocation == nil)
        || (requestedDeliveryPeriod == nil)
        || (promisedDeliveryPeriod == nil)
        || (estimatedDeliveryPeriod == nil)
        || (carrierParty == nil)
        || (deliveryParty == nil)
        || (despatch == nil)
        || (minimumDeliveryUnit == nil)
        || (maximumDeliveryUnit == nil)
        || (shipment == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .quantity: return "Quantity"
            case .minimumQuantity: return "Minimum Quantity"
            case .maximumQuantity: return "Maximum Quantity"
            case .actualDeliveryDate: return "Actual Delivery Date"
            case .actualDeliveryTime: return "Actual Delivery Time"
            case .latestDeliveryDate: return "Latest Delivery Date"
            case .latestDeliveryTime: return "Latest Delivery Time"
            case .releaseID: return "Release"
            case .trackingID: return "Tracking Identifier"
            case .deliveryAddress: return "Delivery Address"
            case .deliveryLocation: return "Delivery Location"
            case .alternativeDeliveryLocation: return "Alternative Delivery Location"
            case .requestedDeliveryPeriod: return "Requested Delivery Period"
            case .promisedDeliveryPeriod: return "Promised Delivery Period"
            case .estimatedDeliveryPeriod: return "Estimated Delivery Period"
            case .carrierParty: return "Carrier Party"
            case .deliveryParty: return "Delivery Party"
            case .notifyParty: return "Notify Party"
            case .despatch: return "Despatch"
            case .deliveryTerms: return "Delivery Terms"
            case .minimumDeliveryUnit: return "Minimum Delivery Unit"
            case .maximumDeliveryUnit: return "Maximum Delivery Unit"
            case .shipment: return "Shipment"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this delivery."
            case .quantity: return "The quantity of items, child consignments, shipments in this delivery."
            case .minimumQuantity: return "The minimum quantity of items, child consignments, shipments in this delivery."
            case .maximumQuantity: return "The maximum quantity of items, child consignments, shipments in this delivery."
            case .actualDeliveryDate: return "The actual date of delivery."
            case .actualDeliveryTime: return "The actual time of delivery."
            case .latestDeliveryDate: return "The latest date of delivery allowed by the buyer."
            case .latestDeliveryTime: return "The latest time of delivery allowed by the buyer."
            case .releaseID: return "An identifier used for approval of access to delivery locations (e.g., port terminals)."
            case .trackingID: return "The delivery Tracking ID (for transport tracking)."
            case .deliveryAddress: return "The delivery address."
            case .deliveryLocation: return "The delivery location."
            case .alternativeDeliveryLocation: return "An alternative delivery location."
            case .requestedDeliveryPeriod: return "The period requested for delivery."
            case .promisedDeliveryPeriod: return "The period promised for delivery."
            case .estimatedDeliveryPeriod: return "The period estimated for delivery."
            case .carrierParty: return "The party responsible for delivering the goods."
            case .deliveryParty: return "The party to whom the goods are delivered."
            case .notifyParty: return "A party to be notified of this delivery."
            case .despatch: return "The despatch (pickup) associated with this delivery."
            case .deliveryTerms: return "Terms and conditions relating to the delivery."
            case .minimumDeliveryUnit: return "The minimum delivery unit for this delivery."
            case .maximumDeliveryUnit: return "The maximum delivery unit for this delivery."
            case .shipment: return "The shipment being delivered."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .quantity: result = quantity?.search(_search)
                case .minimumQuantity: result = minimumQuantity?.search(_search)
                case .maximumQuantity: result = maximumQuantity?.search(_search)
                case .actualDeliveryDate: result = actualDeliveryDate?.search(_search)
                case .actualDeliveryTime: result = actualDeliveryTime?.search(_search)
                case .latestDeliveryDate: result = latestDeliveryDate?.search(_search)
                case .latestDeliveryTime: result = latestDeliveryTime?.search(_search)
                case .releaseID: result = releaseID?.search(_search)
                case .trackingID: result = trackingID?.search(_search)
                case .deliveryAddress: result = deliveryAddress?.search(_search)
                case .deliveryLocation: result = deliveryLocation?.search(_search)
                case .alternativeDeliveryLocation: result = alternativeDeliveryLocation?.search(_search)
                case .requestedDeliveryPeriod: result = requestedDeliveryPeriod?.search(_search)
                case .promisedDeliveryPeriod: result = promisedDeliveryPeriod?.search(_search)
                case .estimatedDeliveryPeriod: result = estimatedDeliveryPeriod?.search(_search)
                case .carrierParty: result = carrierParty?.search(_search)
                case .deliveryParty: result = deliveryParty?.search(_search)
                case .notifyParty: result = notifyParty.compactMap({ $0.search(_search) }).first
                case .despatch: result = despatch?.search(_search)
                case .deliveryTerms: result = deliveryTerms.compactMap({ $0.search(_search) }).first
                case .minimumDeliveryUnit: result = minimumDeliveryUnit?.search(_search)
                case .maximumDeliveryUnit: result = maximumDeliveryUnit?.search(_search)
                case .shipment: result = shipment?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .quantity: presentableView(for: quantity, title: _title, details: _details, show: show, search: _search)
                case .minimumQuantity: presentableView(for: minimumQuantity, title: _title, details: _details, show: show, search: _search)
                case .maximumQuantity: presentableView(for: maximumQuantity, title: _title, details: _details, show: show, search: _search)
                case .actualDeliveryDate: anyView(for: actualDeliveryDate, title: _title, details: _details, show: show, search: _search)
                case .actualDeliveryTime: anyView(for: actualDeliveryTime, title: _title, details: _details, show: show, search: _search)
                case .latestDeliveryDate: anyView(for: latestDeliveryDate, title: _title, details: _details, show: show, search: _search)
                case .latestDeliveryTime: anyView(for: latestDeliveryTime, title: _title, details: _details, show: show, search: _search)
                case .releaseID: presentableView(for: releaseID, title: _title, details: _details, show: show, search: _search)
                case .trackingID: presentableView(for: trackingID, title: _title, details: _details, show: show, search: _search)
                case .deliveryAddress: presentableView(for: deliveryAddress, title: _title, details: _details, show: show, search: _search)
                case .deliveryLocation: presentableView(for: deliveryLocation, title: _title, details: _details, show: show, search: _search)
                case .alternativeDeliveryLocation: presentableView(for: alternativeDeliveryLocation, title: _title, details: _details, show: show, search: _search)
                case .requestedDeliveryPeriod: presentableView(for: requestedDeliveryPeriod, title: _title, details: _details, show: show, search: _search)
                case .promisedDeliveryPeriod: presentableView(for: promisedDeliveryPeriod, title: _title, details: _details, show: show, search: _search)
                case .estimatedDeliveryPeriod: presentableView(for: estimatedDeliveryPeriod, title: _title, details: _details, show: show, search: _search)
                case .carrierParty: presentableView(for: carrierParty, title: _title, details: _details, show: show, search: _search)
                case .deliveryParty: presentableView(for: deliveryParty, title: _title, details: _details, show: show, search: _search)
                case .notifyParty: presentableView(for: notifyParty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .despatch: presentableView(for: despatch, title: _title, details: _details, show: show, search: _search)
                case .deliveryTerms: presentableView(for: deliveryTerms.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .minimumDeliveryUnit: presentableView(for: minimumDeliveryUnit, title: _title, details: _details, show: show, search: _search)
                case .maximumDeliveryUnit: presentableView(for: maximumDeliveryUnit, title: _title, details: _details, show: show, search: _search)
                case .shipment: presentableView(for: shipment, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Delivery: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = actualDeliveryDate?.search(string) { return result }
        if let result = actualDeliveryTime?.search(string) { return result }
        if let result = latestDeliveryDate?.search(string) { return result }
        if let result = latestDeliveryTime?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = quantity?.search(string) { return result }
        if let result = minimumQuantity?.search(string) { return result }
        if let result = maximumQuantity?.search(string) { return result }
        if let result = releaseID?.search(string) { return result }
        if let result = trackingID?.search(string) { return result }
        if let result = deliveryAddress?.search(string) { return result }
        if let result = deliveryLocation?.search(string) { return result }
        if let result = alternativeDeliveryLocation?.search(string) { return result }
        if let result = requestedDeliveryPeriod?.search(string) { return result }
        if let result = promisedDeliveryPeriod?.search(string) { return result }
        if let result = estimatedDeliveryPeriod?.search(string) { return result }
        if let result = carrierParty?.search(string) { return result }
        if let result = deliveryParty?.search(string) { return result }
        if let result = notifyParty.compactMap({ $0.search(string)  }).first { return result }
        if let result = despatch?.search(string) { return result }
        if let result = deliveryTerms.compactMap({ $0.search(string)  }).first { return result }
        if let result = minimumDeliveryUnit?.search(string) { return result }
        if let result = maximumDeliveryUnit?.search(string) { return result }
        if let result = shipment?.search(string) { return result }
        return nil
    }
}
