// 
// DeliveryTerms.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.DeliveryTerms: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .specialTerms,
        .lossRiskResponsibilityCode,
        .lossRisk,
        .amount,
        .deliveryLocation,
        .allowanceCharge
]
    public var hasNilValue: Bool {(id == nil)
        || (lossRiskResponsibilityCode == nil)
        || (amount == nil)
        || (deliveryLocation == nil)
        || (allowanceCharge == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .specialTerms: return "Special Terms"
            case .lossRiskResponsibilityCode: return "Loss Risk Responsibility Code"
            case .lossRisk: return "Loss Risk"
            case .amount: return "Amount"
            case .deliveryLocation: return "Delivery Location"
            case .allowanceCharge: return "Allowance Charge"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this description of delivery terms."
            case .specialTerms: return "A description of any terms or conditions relating to the delivery items."
            case .lossRiskResponsibilityCode: return "A code that identifies one of various responsibilities for loss risk in the execution of the delivery."
            case .lossRisk: return "A description of responsibility for risk of loss in execution of the delivery, expressed as text."
            case .amount: return "The monetary amount covered by these delivery terms."
            case .deliveryLocation: return "The location for the contracted delivery."
            case .allowanceCharge: return "An allowance or charge covered by these delivery terms."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .specialTerms: result = specialTerms.compactMap({ $0.search(_search) }).first
                case .lossRiskResponsibilityCode: result = lossRiskResponsibilityCode?.search(_search)
                case .lossRisk: result = lossRisk.compactMap({ $0.search(_search) }).first
                case .amount: result = amount?.search(_search)
                case .deliveryLocation: result = deliveryLocation?.search(_search)
                case .allowanceCharge: result = allowanceCharge?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .specialTerms: presentableView(for: specialTerms.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .lossRiskResponsibilityCode: presentableView(for: lossRiskResponsibilityCode, title: _title, details: _details, show: show, search: _search)
                case .lossRisk: presentableView(for: lossRisk.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .amount: presentableView(for: amount, title: _title, details: _details, show: show, search: _search)
                case .deliveryLocation: presentableView(for: deliveryLocation, title: _title, details: _details, show: show, search: _search)
                case .allowanceCharge: presentableView(for: allowanceCharge, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.DeliveryTerms: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = specialTerms.compactMap({ $0.search(string)  }).first { return result }
        if let result = lossRiskResponsibilityCode?.search(string) { return result }
        if let result = lossRisk.compactMap({ $0.search(string)  }).first { return result }
        if let result = amount?.search(string) { return result }
        if let result = deliveryLocation?.search(string) { return result }
        if let result = allowanceCharge?.search(string) { return result }
        return nil
    }
}
