// 
// DeliveryUnit.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.DeliveryUnit: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .batchQuantity,
        .consumerUnitQuantity,
        .hazardousRiskIndicator
]
    public var hasNilValue: Bool {(consumerUnitQuantity == nil)
        || (hazardousRiskIndicator == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .batchQuantity: return "Batch Quantity"
            case .consumerUnitQuantity: return "Consumer Unit"
            case .hazardousRiskIndicator: return "Hazardous Risk Indicator"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .batchQuantity: return "The quantity of ordered Items that constitutes a batch for delivery purposes."
            case .consumerUnitQuantity: return "The quantity of units in the Delivery Unit expressed in the units used by the consumer."
            case .hazardousRiskIndicator: return "An indication that the transported goods are subject to an international regulation concerning the carriage of dangerous goods (true) or not (false)."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .batchQuantity: result = batchQuantity.search(_search)
                case .consumerUnitQuantity: result = consumerUnitQuantity?.search(_search)
                case .hazardousRiskIndicator: result = hazardousRiskIndicator?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .batchQuantity: presentableView(for: batchQuantity, title: _title, details: _details, show: show, search: _search)
                case .consumerUnitQuantity: presentableView(for: consumerUnitQuantity, title: _title, details: _details, show: show, search: _search)
                case .hazardousRiskIndicator: presentableView(for: hazardousRiskIndicator, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.DeliveryUnit: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = batchQuantity.search(string) { return result }
        if let result = consumerUnitQuantity?.search(string) { return result }
        if let result = hazardousRiskIndicator?.search(string) { return result }
        return nil
    }
}
