// 
// DependentPriceReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.DependentPriceReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .percent,
        .locationAddress,
        .dependentLineReference
]
    public var hasNilValue: Bool {(percent == nil)
        || (locationAddress == nil)
        || (dependentLineReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .percent: return "Percent"
            case .locationAddress: return "Location Address"
            case .dependentLineReference: return "Dependent Line Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .percent: return "The percentage by which the price of the different item is multiplied to calculate the price of the item."
            case .locationAddress: return "The reference location for this dependent price reference."
            case .dependentLineReference: return "A reference to a line that the price is depended of."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .percent: result = percent?.search(_search)
                case .locationAddress: result = locationAddress?.search(_search)
                case .dependentLineReference: result = dependentLineReference?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .percent: presentableView(for: percent, title: _title, details: _details, show: show, search: _search)
                case .locationAddress: presentableView(for: locationAddress, title: _title, details: _details, show: show, search: _search)
                case .dependentLineReference: presentableView(for: dependentLineReference, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.DependentPriceReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = percent?.search(string) { return result }
        if let result = locationAddress?.search(string) { return result }
        if let result = dependentLineReference?.search(string) { return result }
        return nil
    }
}
