// 
// Despatch.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Despatch: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .requestedDespatchDate,
        .requestedDespatchTime,
        .estimatedDespatchDate,
        .estimatedDespatchTime,
        .actualDespatchDate,
        .actualDespatchTime,
        .guaranteedDespatchDate,
        .guaranteedDespatchTime,
        .releaseID,
        .instructions,
        .despatchAddress,
        .despatchLocation,
        .despatchParty,
        .carrierParty,
        .notifyParty,
        .contact,
        .estimatedDespatchPeriod,
        .requestedDespatchPeriod
]
    public var hasNilValue: Bool {(id == nil)
        || (requestedDespatchDate == nil)
        || (requestedDespatchTime == nil)
        || (estimatedDespatchDate == nil)
        || (estimatedDespatchTime == nil)
        || (actualDespatchDate == nil)
        || (actualDespatchTime == nil)
        || (guaranteedDespatchDate == nil)
        || (guaranteedDespatchTime == nil)
        || (releaseID == nil)
        || (despatchAddress == nil)
        || (despatchLocation == nil)
        || (despatchParty == nil)
        || (carrierParty == nil)
        || (contact == nil)
        || (estimatedDespatchPeriod == nil)
        || (requestedDespatchPeriod == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .requestedDespatchDate: return "Requested Despatch Date"
            case .requestedDespatchTime: return "Requested Despatch Time"
            case .estimatedDespatchDate: return "Estimated Despatch Date"
            case .estimatedDespatchTime: return "Estimated Despatch Time"
            case .actualDespatchDate: return "Actual Despatch Date"
            case .actualDespatchTime: return "Actual Despatch Time"
            case .guaranteedDespatchDate: return "Guaranteed Despatch Date"
            case .guaranteedDespatchTime: return "Guaranteed Despatch Time"
            case .releaseID: return "Release"
            case .instructions: return "Instructions"
            case .despatchAddress: return "Despatch Address"
            case .despatchLocation: return "Despatch Location"
            case .despatchParty: return "Despatch Party"
            case .carrierParty: return "Carrier Party"
            case .notifyParty: return "Notify Party"
            case .contact: return "Contact"
            case .estimatedDespatchPeriod: return "Estimated Despatch Period"
            case .requestedDespatchPeriod: return "Requested Despatch Period"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this despatch event."
            case .requestedDespatchDate: return "The despatch (pickup) date requested, normally by the buyer."
            case .requestedDespatchTime: return "The despatch (pickup) time requested, normally by the buyer."
            case .estimatedDespatchDate: return "The estimated despatch (pickup) date."
            case .estimatedDespatchTime: return "The estimated despatch (pickup) time."
            case .actualDespatchDate: return "The actual despatch (pickup) date."
            case .actualDespatchTime: return "The actual despatch (pickup) time."
            case .guaranteedDespatchDate: return "The date guaranteed for the despatch (pickup)."
            case .guaranteedDespatchTime: return "The time guaranteed for the despatch (pickup)."
            case .releaseID: return "An identifier for the release of the despatch used as security control or cargo control (pick-up)."
            case .instructions: return "Text describing any special instructions applying to the despatch (pickup)."
            case .despatchAddress: return "The address of the despatch (pickup)."
            case .despatchLocation: return "The location of the despatch (pickup)."
            case .despatchParty: return "The party despatching the goods."
            case .carrierParty: return "The party carrying the goods."
            case .notifyParty: return "A party to be notified of this despatch (pickup)."
            case .contact: return "The primary contact for this despatch (pickup)."
            case .estimatedDespatchPeriod: return "The period estimated for the despatch (pickup) of goods."
            case .requestedDespatchPeriod: return "The period requested for the despatch (pickup) of goods."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .requestedDespatchDate: result = requestedDespatchDate?.search(_search)
                case .requestedDespatchTime: result = requestedDespatchTime?.search(_search)
                case .estimatedDespatchDate: result = estimatedDespatchDate?.search(_search)
                case .estimatedDespatchTime: result = estimatedDespatchTime?.search(_search)
                case .actualDespatchDate: result = actualDespatchDate?.search(_search)
                case .actualDespatchTime: result = actualDespatchTime?.search(_search)
                case .guaranteedDespatchDate: result = guaranteedDespatchDate?.search(_search)
                case .guaranteedDespatchTime: result = guaranteedDespatchTime?.search(_search)
                case .releaseID: result = releaseID?.search(_search)
                case .instructions: result = instructions.compactMap({ $0.search(_search) }).first
                case .despatchAddress: result = despatchAddress?.search(_search)
                case .despatchLocation: result = despatchLocation?.search(_search)
                case .despatchParty: result = despatchParty?.search(_search)
                case .carrierParty: result = carrierParty?.search(_search)
                case .notifyParty: result = notifyParty.compactMap({ $0.search(_search) }).first
                case .contact: result = contact?.search(_search)
                case .estimatedDespatchPeriod: result = estimatedDespatchPeriod?.search(_search)
                case .requestedDespatchPeriod: result = requestedDespatchPeriod?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .requestedDespatchDate: anyView(for: requestedDespatchDate, title: _title, details: _details, show: show, search: _search)
                case .requestedDespatchTime: anyView(for: requestedDespatchTime, title: _title, details: _details, show: show, search: _search)
                case .estimatedDespatchDate: anyView(for: estimatedDespatchDate, title: _title, details: _details, show: show, search: _search)
                case .estimatedDespatchTime: anyView(for: estimatedDespatchTime, title: _title, details: _details, show: show, search: _search)
                case .actualDespatchDate: anyView(for: actualDespatchDate, title: _title, details: _details, show: show, search: _search)
                case .actualDespatchTime: anyView(for: actualDespatchTime, title: _title, details: _details, show: show, search: _search)
                case .guaranteedDespatchDate: anyView(for: guaranteedDespatchDate, title: _title, details: _details, show: show, search: _search)
                case .guaranteedDespatchTime: anyView(for: guaranteedDespatchTime, title: _title, details: _details, show: show, search: _search)
                case .releaseID: presentableView(for: releaseID, title: _title, details: _details, show: show, search: _search)
                case .instructions: presentableView(for: instructions.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .despatchAddress: presentableView(for: despatchAddress, title: _title, details: _details, show: show, search: _search)
                case .despatchLocation: presentableView(for: despatchLocation, title: _title, details: _details, show: show, search: _search)
                case .despatchParty: presentableView(for: despatchParty, title: _title, details: _details, show: show, search: _search)
                case .carrierParty: presentableView(for: carrierParty, title: _title, details: _details, show: show, search: _search)
                case .notifyParty: presentableView(for: notifyParty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .contact: presentableView(for: contact, title: _title, details: _details, show: show, search: _search)
                case .estimatedDespatchPeriod: presentableView(for: estimatedDespatchPeriod, title: _title, details: _details, show: show, search: _search)
                case .requestedDespatchPeriod: presentableView(for: requestedDespatchPeriod, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Despatch: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = requestedDespatchDate?.search(string) { return result }
        if let result = requestedDespatchTime?.search(string) { return result }
        if let result = estimatedDespatchDate?.search(string) { return result }
        if let result = estimatedDespatchTime?.search(string) { return result }
        if let result = actualDespatchDate?.search(string) { return result }
        if let result = actualDespatchTime?.search(string) { return result }
        if let result = guaranteedDespatchDate?.search(string) { return result }
        if let result = guaranteedDespatchTime?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = releaseID?.search(string) { return result }
        if let result = instructions.compactMap({ $0.search(string)  }).first { return result }
        if let result = despatchAddress?.search(string) { return result }
        if let result = despatchLocation?.search(string) { return result }
        if let result = despatchParty?.search(string) { return result }
        if let result = carrierParty?.search(string) { return result }
        if let result = notifyParty.compactMap({ $0.search(string)  }).first { return result }
        if let result = contact?.search(string) { return result }
        if let result = estimatedDespatchPeriod?.search(string) { return result }
        if let result = requestedDespatchPeriod?.search(string) { return result }
        return nil
    }
}
