// 
// DespatchLine.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.DespatchLine: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .uuid,
        .note,
        .lineStatusCode,
        .deliveredQuantity,
        .backorderQuantity,
        .backorderReason,
        .outstandingQuantity,
        .outstandingReason,
        .oversupplyQuantity,
        .orderLineReference,
        .documentReference,
        .item,
        .shipment
]
    public var hasNilValue: Bool {(uuid == nil)
        || (lineStatusCode == nil)
        || (deliveredQuantity == nil)
        || (backorderQuantity == nil)
        || (outstandingQuantity == nil)
        || (oversupplyQuantity == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .uuid: return "UUID"
            case .note: return "Note"
            case .lineStatusCode: return "Line Status Code"
            case .deliveredQuantity: return "Delivered Quantity"
            case .backorderQuantity: return "Backorder Quantity"
            case .backorderReason: return "Backorder Reason"
            case .outstandingQuantity: return "Outstanding Quantity"
            case .outstandingReason: return "Outstanding Reason"
            case .oversupplyQuantity: return "Oversupply Quantity"
            case .orderLineReference: return "Order Line Reference"
            case .documentReference: return "Document Reference"
            case .item: return "Item"
            case .shipment: return "Shipment"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this despatch line."
            case .uuid: return "A universally unique identifier for this despatch line."
            case .note: return "Free-form text conveying information that is not contained explicitly in other structures."
            case .lineStatusCode: return "A code signifying the status of this despatch line with respect to its original state."
            case .deliveredQuantity: return "The quantity despatched (picked up)."
            case .backorderQuantity: return "The quantity on back order at the supplier."
            case .backorderReason: return "The reason for the back order."
            case .outstandingQuantity: return "The quantity outstanding (which will follow in a later despatch)."
            case .outstandingReason: return "The reason for the outstanding quantity."
            case .oversupplyQuantity: return "The quantity over-supplied, i.e., the quantity over and above that ordered."
            case .orderLineReference: return "A reference to an order line associated with this despatch line."
            case .documentReference: return "A reference to a document associated with this despatch line."
            case .item: return "The item associated with this despatch line."
            case .shipment: return "A shipment associated with this despatch line."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .note: result = note.compactMap({ $0.search(_search) }).first
                case .lineStatusCode: result = lineStatusCode?.search(_search)
                case .deliveredQuantity: result = deliveredQuantity?.search(_search)
                case .backorderQuantity: result = backorderQuantity?.search(_search)
                case .backorderReason: result = backorderReason.compactMap({ $0.search(_search) }).first
                case .outstandingQuantity: result = outstandingQuantity?.search(_search)
                case .outstandingReason: result = outstandingReason.compactMap({ $0.search(_search) }).first
                case .oversupplyQuantity: result = oversupplyQuantity?.search(_search)
                case .orderLineReference: result = orderLineReference.compactMap({ $0.search(_search) }).first
                case .documentReference: result = documentReference.compactMap({ $0.search(_search) }).first
                case .item: result = item.search(_search)
                case .shipment: result = shipment.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .note: presentableView(for: note.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .lineStatusCode: presentableView(for: lineStatusCode, title: _title, details: _details, show: show, search: _search)
                case .deliveredQuantity: presentableView(for: deliveredQuantity, title: _title, details: _details, show: show, search: _search)
                case .backorderQuantity: presentableView(for: backorderQuantity, title: _title, details: _details, show: show, search: _search)
                case .backorderReason: presentableView(for: backorderReason.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .outstandingQuantity: presentableView(for: outstandingQuantity, title: _title, details: _details, show: show, search: _search)
                case .outstandingReason: presentableView(for: outstandingReason.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .oversupplyQuantity: presentableView(for: oversupplyQuantity, title: _title, details: _details, show: show, search: _search)
                case .orderLineReference: presentableView(for: orderLineReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .documentReference: presentableView(for: documentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .item: presentableView(for: item, title: _title, details: _details, show: show, search: _search)
                case .shipment: presentableView(for: shipment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.DespatchLine: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = note.compactMap({ $0.search(string)  }).first { return result }
        if let result = lineStatusCode?.search(string) { return result }
        if let result = deliveredQuantity?.search(string) { return result }
        if let result = backorderQuantity?.search(string) { return result }
        if let result = backorderReason.compactMap({ $0.search(string)  }).first { return result }
        if let result = outstandingQuantity?.search(string) { return result }
        if let result = outstandingReason.compactMap({ $0.search(string)  }).first { return result }
        if let result = oversupplyQuantity?.search(string) { return result }
        if let result = orderLineReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = documentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = item.search(string) { return result }
        if let result = shipment.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
