// 
// DocumentReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.DocumentReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .copyIndicator,
        .uuid,
        .issueDate,
        .issueTime,
        .documentTypeCode,
        .documentType,
        .xPath,
        .languageID,
        .localeCode,
        .versionID,
        .documentStatusCode,
        .documentDescription,
        .attachment,
        .validityPeriod,
        .issuerParty,
        .resultOfVerification
]
    public var hasNilValue: Bool {(copyIndicator == nil)
        || (uuid == nil)
        || (issueDate == nil)
        || (issueTime == nil)
        || (documentTypeCode == nil)
        || (documentType == nil)
        || (languageID == nil)
        || (localeCode == nil)
        || (versionID == nil)
        || (documentStatusCode == nil)
        || (attachment == nil)
        || (validityPeriod == nil)
        || (issuerParty == nil)
        || (resultOfVerification == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .copyIndicator: return "Copy Indicator"
            case .uuid: return "UUID"
            case .issueDate: return "Issue Date"
            case .issueTime: return "Issue Time"
            case .documentTypeCode: return "Document Type Code"
            case .documentType: return "Document Type"
            case .xPath: return "XPath"
            case .languageID: return "Language"
            case .localeCode: return "Locale Code"
            case .versionID: return "Version"
            case .documentStatusCode: return "Document Status Code"
            case .documentDescription: return "Document Description"
            case .attachment: return "Attachment"
            case .validityPeriod: return "Validity Period"
            case .issuerParty: return "Issuer Party"
            case .resultOfVerification: return "Result Of Verification"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for the referenced document."
            case .copyIndicator: return "An indicator that the referenced document is a copy (true) or the original (false)."
            case .uuid: return "A universally unique identifier for this document reference."
            case .issueDate: return "The date, assigned by the sender of the referenced document, on which the document was issued."
            case .issueTime: return "The time, assigned by the sender of the referenced document, at which the document was issued."
            case .documentTypeCode: return "The type of document being referenced, expressed as a code."
            case .documentType: return "The type of document being referenced, expressed as text."
            case .xPath: return "A reference to another place in the same XML document instance in which DocumentReference appears."
            case .languageID: return "An identifier for the language used in the referenced document."
            case .localeCode: return "A code signifying the locale in which the language in the referenced document is used."
            case .versionID: return "An identifier for the current version of the referenced document."
            case .documentStatusCode: return "A code signifying the status of the reference document with respect to its original state."
            case .documentDescription: return "Text describing the referenced document."
            case .attachment: return "The referenced document as an attachment to the document from which it is referenced."
            case .validityPeriod: return "The period for which this document reference is valid."
            case .issuerParty: return "The party who issued the referenced document."
            case .resultOfVerification: return "The result of an attempt to verify a signature associated with the referenced document."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .copyIndicator: result = copyIndicator?.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .issueDate: result = issueDate?.search(_search)
                case .issueTime: result = issueTime?.search(_search)
                case .documentTypeCode: result = documentTypeCode?.search(_search)
                case .documentType: result = documentType?.search(_search)
                case .xPath: result = xPath.compactMap({ $0.search(_search) }).first
                case .languageID: result = languageID?.search(_search)
                case .localeCode: result = localeCode?.search(_search)
                case .versionID: result = versionID?.search(_search)
                case .documentStatusCode: result = documentStatusCode?.search(_search)
                case .documentDescription: result = documentDescription.compactMap({ $0.search(_search) }).first
                case .attachment: result = attachment?.search(_search)
                case .validityPeriod: result = validityPeriod?.search(_search)
                case .issuerParty: result = issuerParty?.search(_search)
                case .resultOfVerification: result = resultOfVerification?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .copyIndicator: presentableView(for: copyIndicator, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .issueDate: anyView(for: issueDate, title: _title, details: _details, show: show, search: _search)
                case .issueTime: anyView(for: issueTime, title: _title, details: _details, show: show, search: _search)
                case .documentTypeCode: presentableView(for: documentTypeCode, title: _title, details: _details, show: show, search: _search)
                case .documentType: presentableView(for: documentType, title: _title, details: _details, show: show, search: _search)
                case .xPath: presentableView(for: xPath.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .languageID: presentableView(for: languageID, title: _title, details: _details, show: show, search: _search)
                case .localeCode: presentableView(for: localeCode, title: _title, details: _details, show: show, search: _search)
                case .versionID: presentableView(for: versionID, title: _title, details: _details, show: show, search: _search)
                case .documentStatusCode: presentableView(for: documentStatusCode, title: _title, details: _details, show: show, search: _search)
                case .documentDescription: presentableView(for: documentDescription.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .attachment: presentableView(for: attachment, title: _title, details: _details, show: show, search: _search)
                case .validityPeriod: presentableView(for: validityPeriod, title: _title, details: _details, show: show, search: _search)
                case .issuerParty: presentableView(for: issuerParty, title: _title, details: _details, show: show, search: _search)
                case .resultOfVerification: presentableView(for: resultOfVerification, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.DocumentReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = issueDate?.search(string) { return result }
        if let result = issueTime?.search(string) { return result }
        if let result = id.search(string) { return result }
        if let result = copyIndicator?.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = documentTypeCode?.search(string) { return result }
        if let result = documentType?.search(string) { return result }
        if let result = xPath.compactMap({ $0.search(string)  }).first { return result }
        if let result = languageID?.search(string) { return result }
        if let result = localeCode?.search(string) { return result }
        if let result = versionID?.search(string) { return result }
        if let result = documentStatusCode?.search(string) { return result }
        if let result = documentDescription.compactMap({ $0.search(string)  }).first { return result }
        if let result = attachment?.search(string) { return result }
        if let result = validityPeriod?.search(string) { return result }
        if let result = issuerParty?.search(string) { return result }
        if let result = resultOfVerification?.search(string) { return result }
        return nil
    }
}
