// 
// EmissionCalculationMethod.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.EmissionCalculationMethod: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .calculationMethodCode,
        .fullnessIndicationCode,
        .measurementFromLocation,
        .measurementToLocation
]
    public var hasNilValue: Bool {(calculationMethodCode == nil)
        || (fullnessIndicationCode == nil)
        || (measurementFromLocation == nil)
        || (measurementToLocation == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .calculationMethodCode: return "Calculation Method Code"
            case .fullnessIndicationCode: return "Fullness Indication Code"
            case .measurementFromLocation: return "Measurement From Location"
            case .measurementToLocation: return "Measurement To Location"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .calculationMethodCode: return "A code signifying the method used to calculate the emission."
            case .fullnessIndicationCode: return "A code signifying whether a piece of transport equipment is full, partially full, or empty. This indication is used as a parameter when calculating the environmental emission."
            case .measurementFromLocation: return "A start location from which an environmental emission is calculated."
            case .measurementToLocation: return "An end location to which an environmental emission is calculated."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .calculationMethodCode: result = calculationMethodCode?.search(_search)
                case .fullnessIndicationCode: result = fullnessIndicationCode?.search(_search)
                case .measurementFromLocation: result = measurementFromLocation?.search(_search)
                case .measurementToLocation: result = measurementToLocation?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .calculationMethodCode: presentableView(for: calculationMethodCode, title: _title, details: _details, show: show, search: _search)
                case .fullnessIndicationCode: presentableView(for: fullnessIndicationCode, title: _title, details: _details, show: show, search: _search)
                case .measurementFromLocation: presentableView(for: measurementFromLocation, title: _title, details: _details, show: show, search: _search)
                case .measurementToLocation: presentableView(for: measurementToLocation, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.EmissionCalculationMethod: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = calculationMethodCode?.search(string) { return result }
        if let result = fullnessIndicationCode?.search(string) { return result }
        if let result = measurementFromLocation?.search(string) { return result }
        if let result = measurementToLocation?.search(string) { return result }
        return nil
    }
}
