// 
// ExchangeRate.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ExchangeRate: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .sourceCurrencyCode,
        .sourceCurrencyBaseRate,
        .targetCurrencyCode,
        .targetCurrencyBaseRate,
        .exchangeMarketID,
        .calculationRate,
        .mathematicOperatorCode,
        .date,
        .foreignExchangeContract
]
    public var hasNilValue: Bool {(sourceCurrencyBaseRate == nil)
        || (targetCurrencyBaseRate == nil)
        || (exchangeMarketID == nil)
        || (calculationRate == nil)
        || (mathematicOperatorCode == nil)
        || (date == nil)
        || (foreignExchangeContract == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .sourceCurrencyCode: return "Source Currency Code"
            case .sourceCurrencyBaseRate: return "Source Currency Base Rate"
            case .targetCurrencyCode: return "Target Currency Code"
            case .targetCurrencyBaseRate: return "Target Currency Base Rate"
            case .exchangeMarketID: return "Exchange Market Identifier"
            case .calculationRate: return "Calculation Rate"
            case .mathematicOperatorCode: return "Mathematic Operator Code"
            case .date: return "Date"
            case .foreignExchangeContract: return "Foreign Exchange Contract"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .sourceCurrencyCode: return "The reference currency for this exchange rate; the currency from which the exchange is being made."
            case .sourceCurrencyBaseRate: return "In the case of a source currency with denominations of small value, the unit base."
            case .targetCurrencyCode: return "The target currency for this exchange rate; the currency to which the exchange is being made."
            case .targetCurrencyBaseRate: return "In the case of a target currency with denominations of small value, the unit base."
            case .exchangeMarketID: return "An identifier for the currency exchange market used as the source of this exchange rate."
            case .calculationRate: return "The factor applied to the source currency to calculate the target currency."
            case .mathematicOperatorCode: return "A code signifying whether the calculation rate is a multiplier or a divisor."
            case .date: return "The date on which the exchange rate was established."
            case .foreignExchangeContract: return "A contract for foreign exchange."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .sourceCurrencyCode: result = sourceCurrencyCode.search(_search)
                case .sourceCurrencyBaseRate: result = sourceCurrencyBaseRate?.search(_search)
                case .targetCurrencyCode: result = targetCurrencyCode.search(_search)
                case .targetCurrencyBaseRate: result = targetCurrencyBaseRate?.search(_search)
                case .exchangeMarketID: result = exchangeMarketID?.search(_search)
                case .calculationRate: result = calculationRate?.search(_search)
                case .mathematicOperatorCode: result = mathematicOperatorCode?.search(_search)
                case .date: result = date?.search(_search)
                case .foreignExchangeContract: result = foreignExchangeContract?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .sourceCurrencyCode: presentableView(for: sourceCurrencyCode, title: _title, details: _details, show: show, search: _search)
                case .sourceCurrencyBaseRate: presentableView(for: sourceCurrencyBaseRate, title: _title, details: _details, show: show, search: _search)
                case .targetCurrencyCode: presentableView(for: targetCurrencyCode, title: _title, details: _details, show: show, search: _search)
                case .targetCurrencyBaseRate: presentableView(for: targetCurrencyBaseRate, title: _title, details: _details, show: show, search: _search)
                case .exchangeMarketID: presentableView(for: exchangeMarketID, title: _title, details: _details, show: show, search: _search)
                case .calculationRate: presentableView(for: calculationRate, title: _title, details: _details, show: show, search: _search)
                case .mathematicOperatorCode: presentableView(for: mathematicOperatorCode, title: _title, details: _details, show: show, search: _search)
                case .date: anyView(for: date, title: _title, details: _details, show: show, search: _search)
                case .foreignExchangeContract: presentableView(for: foreignExchangeContract, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ExchangeRate: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = date?.search(string) { return result }
        if let result = sourceCurrencyCode.search(string) { return result }
        if let result = sourceCurrencyBaseRate?.search(string) { return result }
        if let result = targetCurrencyCode.search(string) { return result }
        if let result = targetCurrencyBaseRate?.search(string) { return result }
        if let result = exchangeMarketID?.search(string) { return result }
        if let result = calculationRate?.search(string) { return result }
        if let result = mathematicOperatorCode?.search(string) { return result }
        if let result = foreignExchangeContract?.search(string) { return result }
        return nil
    }
}
