// 
// ExternalReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ExternalReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .uri,
        .documentHash,
        .hashAlgorithmMethod,
        .expiryDate,
        .expiryTime,
        .mimeCode,
        .formatCode,
        .encodingCode,
        .characterSetCode,
        .fileName,
        .description
]
    public var hasNilValue: Bool {(uri == nil)
        || (documentHash == nil)
        || (hashAlgorithmMethod == nil)
        || (expiryDate == nil)
        || (expiryTime == nil)
        || (mimeCode == nil)
        || (formatCode == nil)
        || (encodingCode == nil)
        || (characterSetCode == nil)
        || (fileName == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .uri: return "URI"
            case .documentHash: return "Document Hash"
            case .hashAlgorithmMethod: return "Hash Algorithm Method"
            case .expiryDate: return "Expiry Date"
            case .expiryTime: return "Expiry Time"
            case .mimeCode: return "Mime Code"
            case .formatCode: return "Format Code"
            case .encodingCode: return "Encoding Code"
            case .characterSetCode: return "Character Set Code"
            case .fileName: return "File Name"
            case .description: return "Description"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .uri: return "The Uniform Resource Identifier (URI) that identifies the external object as an Internet resource."
            case .documentHash: return "A hash value for the externally stored object."
            case .hashAlgorithmMethod: return "A hash algorithm used to calculate the hash value of the externally stored object."
            case .expiryDate: return "The date on which availability of the resource can no longer be relied upon."
            case .expiryTime: return "The time after which availability of the resource can no longer be relied upon."
            case .mimeCode: return "A code signifying the mime type of the external object."
            case .formatCode: return "A code signifying the format of the external object."
            case .encodingCode: return "A code signifying the encoding/decoding algorithm used with the external object."
            case .characterSetCode: return "A code signifying the character set of an external document."
            case .fileName: return "The file name of the external object."
            case .description: return "Text describing the external object."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .uri: result = uri?.search(_search)
                case .documentHash: result = documentHash?.search(_search)
                case .hashAlgorithmMethod: result = hashAlgorithmMethod?.search(_search)
                case .expiryDate: result = expiryDate?.search(_search)
                case .expiryTime: result = expiryTime?.search(_search)
                case .mimeCode: result = mimeCode?.search(_search)
                case .formatCode: result = formatCode?.search(_search)
                case .encodingCode: result = encodingCode?.search(_search)
                case .characterSetCode: result = characterSetCode?.search(_search)
                case .fileName: result = fileName?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .uri: presentableView(for: uri, title: _title, details: _details, show: show, search: _search)
                case .documentHash: presentableView(for: documentHash, title: _title, details: _details, show: show, search: _search)
                case .hashAlgorithmMethod: presentableView(for: hashAlgorithmMethod, title: _title, details: _details, show: show, search: _search)
                case .expiryDate: anyView(for: expiryDate, title: _title, details: _details, show: show, search: _search)
                case .expiryTime: anyView(for: expiryTime, title: _title, details: _details, show: show, search: _search)
                case .mimeCode: presentableView(for: mimeCode, title: _title, details: _details, show: show, search: _search)
                case .formatCode: presentableView(for: formatCode, title: _title, details: _details, show: show, search: _search)
                case .encodingCode: presentableView(for: encodingCode, title: _title, details: _details, show: show, search: _search)
                case .characterSetCode: presentableView(for: characterSetCode, title: _title, details: _details, show: show, search: _search)
                case .fileName: presentableView(for: fileName, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ExternalReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = expiryDate?.search(string) { return result }
        if let result = expiryTime?.search(string) { return result }
        if let result = uri?.search(string) { return result }
        if let result = documentHash?.search(string) { return result }
        if let result = hashAlgorithmMethod?.search(string) { return result }
        if let result = mimeCode?.search(string) { return result }
        if let result = formatCode?.search(string) { return result }
        if let result = encodingCode?.search(string) { return result }
        if let result = characterSetCode?.search(string) { return result }
        if let result = fileName?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
