// 
// FinancialAccount.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.FinancialAccount: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .name,
        .aliasName,
        .accountTypeCode,
        .accountFormatCode,
        .currencyCode,
        .paymentNote,
        .financialInstitutionBranch,
        .country
]
    public var hasNilValue: Bool {(id == nil)
        || (name == nil)
        || (aliasName == nil)
        || (accountTypeCode == nil)
        || (accountFormatCode == nil)
        || (currencyCode == nil)
        || (financialInstitutionBranch == nil)
        || (country == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .name: return "Name"
            case .aliasName: return "Alias Name"
            case .accountTypeCode: return "Account Type Code"
            case .accountFormatCode: return "Account Format Code"
            case .currencyCode: return "Currency Code"
            case .paymentNote: return "Payment Note"
            case .financialInstitutionBranch: return "Financial Institution Branch"
            case .country: return "Country"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "The identifier for this financial account; the bank account number."
            case .name: return "The name of this financial account."
            case .aliasName: return "An alias for the name of this financial account, to be used in place of the actual account name for security reasons."
            case .accountTypeCode: return "A code signifying the type of this financial account."
            case .accountFormatCode: return "A code signifying the format of this financial account."
            case .currencyCode: return "A code signifying the currency in which this financial account is held."
            case .paymentNote: return "Free-form text applying to the Payment for the owner of this account."
            case .financialInstitutionBranch: return "The branch of the financial institution associated with this financial account."
            case .country: return "The country in which the holder of the financial account is domiciled."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .name: result = name?.search(_search)
                case .aliasName: result = aliasName?.search(_search)
                case .accountTypeCode: result = accountTypeCode?.search(_search)
                case .accountFormatCode: result = accountFormatCode?.search(_search)
                case .currencyCode: result = currencyCode?.search(_search)
                case .paymentNote: result = paymentNote.compactMap({ $0.search(_search) }).first
                case .financialInstitutionBranch: result = financialInstitutionBranch?.search(_search)
                case .country: result = country?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .aliasName: presentableView(for: aliasName, title: _title, details: _details, show: show, search: _search)
                case .accountTypeCode: presentableView(for: accountTypeCode, title: _title, details: _details, show: show, search: _search)
                case .accountFormatCode: presentableView(for: accountFormatCode, title: _title, details: _details, show: show, search: _search)
                case .currencyCode: presentableView(for: currencyCode, title: _title, details: _details, show: show, search: _search)
                case .paymentNote: presentableView(for: paymentNote.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .financialInstitutionBranch: presentableView(for: financialInstitutionBranch, title: _title, details: _details, show: show, search: _search)
                case .country: presentableView(for: country, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.FinancialAccount: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = name?.search(string) { return result }
        if let result = aliasName?.search(string) { return result }
        if let result = accountTypeCode?.search(string) { return result }
        if let result = accountFormatCode?.search(string) { return result }
        if let result = currencyCode?.search(string) { return result }
        if let result = paymentNote.compactMap({ $0.search(string)  }).first { return result }
        if let result = financialInstitutionBranch?.search(string) { return result }
        if let result = country?.search(string) { return result }
        return nil
    }
}
