// 
// GoodsItemContainer.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.GoodsItemContainer: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .quantity,
        .transportEquipment
]
    public var hasNilValue: Bool {(quantity == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .quantity: return "Quantity"
            case .transportEquipment: return "Transport Equipment"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this goods item container."
            case .quantity: return "The number of goods items loaded into or onto one piece of transport equipment as a total consignment or part of a consignment."
            case .transportEquipment: return "A piece of transport equipment used to contain a single goods item."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .quantity: result = quantity?.search(_search)
                case .transportEquipment: result = transportEquipment.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .quantity: presentableView(for: quantity, title: _title, details: _details, show: show, search: _search)
                case .transportEquipment: presentableView(for: transportEquipment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.GoodsItemContainer: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id.search(string) { return result }
        if let result = quantity?.search(string) { return result }
        if let result = transportEquipment.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
