// 
// HazardousGoodsTransit.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.HazardousGoodsTransit: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .transportEmergencyCardCode,
        .packingCriteriaCode,
        .hazardousRegulationCode,
        .inhalationToxicityZoneCode,
        .transportAuthorizationCode,
        .maximumTemperature,
        .minimumTemperature
]
    public var hasNilValue: Bool {(transportEmergencyCardCode == nil)
        || (packingCriteriaCode == nil)
        || (hazardousRegulationCode == nil)
        || (inhalationToxicityZoneCode == nil)
        || (transportAuthorizationCode == nil)
        || (maximumTemperature == nil)
        || (minimumTemperature == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .transportEmergencyCardCode: return "Transport Emergency Card Code"
            case .packingCriteriaCode: return "Packing Criteria Code"
            case .hazardousRegulationCode: return "Hazardous Regulation Code"
            case .inhalationToxicityZoneCode: return "Inhalation Toxicity Zone Code"
            case .transportAuthorizationCode: return "Transport Authorization Code"
            case .maximumTemperature: return "Maximum Temperature"
            case .minimumTemperature: return "Minimum Temperature"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .transportEmergencyCardCode: return "An identifier for a transport emergency card describing the actions to be taken in an emergency in transporting the hazardous goods. It may be the identity number of a hazardous emergency response plan assigned by the appropriate authority."
            case .packingCriteriaCode: return "A code signifying the packaging requirement for transportation of the hazardous goods as assigned by IATA, IMDB, ADR, RID etc."
            case .hazardousRegulationCode: return "A code signifying the set of legal regulations governing the transportation of the hazardous goods."
            case .inhalationToxicityZoneCode: return "A code signifying the Inhalation Toxicity Hazard Zone for the hazardous goods, as defined by the US Department of Transportation."
            case .transportAuthorizationCode: return "A code signifying authorization for the transportation of hazardous cargo."
            case .maximumTemperature: return "The maximum temperature at which the hazardous goods can safely be transported."
            case .minimumTemperature: return "The minimum temperature at which the hazardous goods can safely be transported."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .transportEmergencyCardCode: result = transportEmergencyCardCode?.search(_search)
                case .packingCriteriaCode: result = packingCriteriaCode?.search(_search)
                case .hazardousRegulationCode: result = hazardousRegulationCode?.search(_search)
                case .inhalationToxicityZoneCode: result = inhalationToxicityZoneCode?.search(_search)
                case .transportAuthorizationCode: result = transportAuthorizationCode?.search(_search)
                case .maximumTemperature: result = maximumTemperature?.search(_search)
                case .minimumTemperature: result = minimumTemperature?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .transportEmergencyCardCode: presentableView(for: transportEmergencyCardCode, title: _title, details: _details, show: show, search: _search)
                case .packingCriteriaCode: presentableView(for: packingCriteriaCode, title: _title, details: _details, show: show, search: _search)
                case .hazardousRegulationCode: presentableView(for: hazardousRegulationCode, title: _title, details: _details, show: show, search: _search)
                case .inhalationToxicityZoneCode: presentableView(for: inhalationToxicityZoneCode, title: _title, details: _details, show: show, search: _search)
                case .transportAuthorizationCode: presentableView(for: transportAuthorizationCode, title: _title, details: _details, show: show, search: _search)
                case .maximumTemperature: presentableView(for: maximumTemperature, title: _title, details: _details, show: show, search: _search)
                case .minimumTemperature: presentableView(for: minimumTemperature, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.HazardousGoodsTransit: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = transportEmergencyCardCode?.search(string) { return result }
        if let result = packingCriteriaCode?.search(string) { return result }
        if let result = hazardousRegulationCode?.search(string) { return result }
        if let result = inhalationToxicityZoneCode?.search(string) { return result }
        if let result = transportAuthorizationCode?.search(string) { return result }
        if let result = maximumTemperature?.search(string) { return result }
        if let result = minimumTemperature?.search(string) { return result }
        return nil
    }
}
