// 
// HazardousItem.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.HazardousItem: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .placardNotation,
        .placardEndorsement,
        .additionalInformation,
        .undgCode,
        .emergencyProceduresCode,
        .medicalFirstAidGuideCode,
        .technicalName,
        .categoryName,
        .hazardousCategoryCode,
        .upperOrangeHazardPlacardID,
        .lowerOrangeHazardPlacardID,
        .markingID,
        .hazardClassID,
        .netWeightMeasure,
        .netVolumeMeasure,
        .quantity,
        .contactParty,
        .secondaryHazard,
        .hazardousGoodsTransit,
        .emergencyTemperature,
        .flashpointTemperature,
        .additionalTemperature
]
    public var hasNilValue: Bool {(id == nil)
        || (placardNotation == nil)
        || (placardEndorsement == nil)
        || (undgCode == nil)
        || (emergencyProceduresCode == nil)
        || (medicalFirstAidGuideCode == nil)
        || (technicalName == nil)
        || (categoryName == nil)
        || (hazardousCategoryCode == nil)
        || (upperOrangeHazardPlacardID == nil)
        || (lowerOrangeHazardPlacardID == nil)
        || (markingID == nil)
        || (hazardClassID == nil)
        || (netWeightMeasure == nil)
        || (netVolumeMeasure == nil)
        || (quantity == nil)
        || (contactParty == nil)
        || (emergencyTemperature == nil)
        || (flashpointTemperature == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .placardNotation: return "Placard Notation"
            case .placardEndorsement: return "Placard Endorsement"
            case .additionalInformation: return "Additional Information"
            case .undgCode: return "UNDG Code"
            case .emergencyProceduresCode: return "Emergency Procedures Code"
            case .medicalFirstAidGuideCode: return "Medical First Aid Guide Code"
            case .technicalName: return "Technical Name"
            case .categoryName: return "Category"
            case .hazardousCategoryCode: return "Hazardous Category Code"
            case .upperOrangeHazardPlacardID: return "Upper Orange Hazard Placard Identifier"
            case .lowerOrangeHazardPlacardID: return "Lower Orange Hazard Placard Identifier"
            case .markingID: return "Marking Identifier"
            case .hazardClassID: return "Hazard Class Identifier"
            case .netWeightMeasure: return "Net Weight"
            case .netVolumeMeasure: return "Net Volume"
            case .quantity: return "Quantity"
            case .contactParty: return "Contact Party"
            case .secondaryHazard: return "Secondary Hazard"
            case .hazardousGoodsTransit: return "Hazardous Goods Transit"
            case .emergencyTemperature: return "Emergency Temperature"
            case .flashpointTemperature: return "Flashpoint Temperature"
            case .additionalTemperature: return "Additional Temperature"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this hazardous item."
            case .placardNotation: return "Text of the placard notation corresponding to the hazard class of this hazardous item. Can also be the hazard identification number of the orange placard (upper part) required on the means of transport."
            case .placardEndorsement: return "Text of the placard endorsement that is to be shown on the shipping papers for this hazardous item. Can also be used for the number of the orange placard (lower part) required on the means of transport."
            case .additionalInformation: return "Text providing further information about the hazardous substance."
            case .undgCode: return "The UN code for this kind of hazardous item."
            case .emergencyProceduresCode: return "A code signifying the emergency procedures for this hazardous item."
            case .medicalFirstAidGuideCode: return "A code signifying a medical first aid guide appropriate to this hazardous item."
            case .technicalName: return "The full technical name of a specific hazardous substance contained in this goods item."
            case .categoryName: return "The name of the category of hazard that applies to the Item."
            case .hazardousCategoryCode: return "A code signifying a kind of hazard for a material."
            case .upperOrangeHazardPlacardID: return "The number for the upper part of the orange hazard placard required on the means of transport."
            case .lowerOrangeHazardPlacardID: return "The number for the lower part of the orange hazard placard required on the means of transport."
            case .markingID: return "An identifier to the marking of the Hazardous Item"
            case .hazardClassID: return "An identifier for the hazard class applicable to this hazardous item as defined by the relevant regulation authority (e.g., the IMDG Class Number of the SOLAS Convention of IMO and the ADR/RID Class Number for the road/rail environment)."
            case .netWeightMeasure: return "The net weight of this hazardous item, excluding packaging."
            case .netVolumeMeasure: return "The volume of this hazardous item, excluding packaging and transport equipment."
            case .quantity: return "The quantity of goods items in this hazardous item that are hazardous."
            case .contactParty: return "The individual, group, or body to be contacted in case of a hazardous incident associated with this item."
            case .secondaryHazard: return "A secondary hazard associated with this hazardous item."
            case .hazardousGoodsTransit: return "Information related to the transit of this kind of hazardous goods."
            case .emergencyTemperature: return "The threshold temperature at which emergency procedures apply in the handling of temperature-controlled goods."
            case .flashpointTemperature: return "The flashpoint temperature of this hazardous item; i.e., the lowest temperature at which vapors above a volatile combustible substance ignite in air when exposed to flame."
            case .additionalTemperature: return "Another temperature relevant to the handling of this hazardous item."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .placardNotation: result = placardNotation?.search(_search)
                case .placardEndorsement: result = placardEndorsement?.search(_search)
                case .additionalInformation: result = additionalInformation.compactMap({ $0.search(_search) }).first
                case .undgCode: result = undgCode?.search(_search)
                case .emergencyProceduresCode: result = emergencyProceduresCode?.search(_search)
                case .medicalFirstAidGuideCode: result = medicalFirstAidGuideCode?.search(_search)
                case .technicalName: result = technicalName?.search(_search)
                case .categoryName: result = categoryName?.search(_search)
                case .hazardousCategoryCode: result = hazardousCategoryCode?.search(_search)
                case .upperOrangeHazardPlacardID: result = upperOrangeHazardPlacardID?.search(_search)
                case .lowerOrangeHazardPlacardID: result = lowerOrangeHazardPlacardID?.search(_search)
                case .markingID: result = markingID?.search(_search)
                case .hazardClassID: result = hazardClassID?.search(_search)
                case .netWeightMeasure: result = netWeightMeasure?.search(_search)
                case .netVolumeMeasure: result = netVolumeMeasure?.search(_search)
                case .quantity: result = quantity?.search(_search)
                case .contactParty: result = contactParty?.search(_search)
                case .secondaryHazard: result = secondaryHazard.compactMap({ $0.search(_search) }).first
                case .hazardousGoodsTransit: result = hazardousGoodsTransit.compactMap({ $0.search(_search) }).first
                case .emergencyTemperature: result = emergencyTemperature?.search(_search)
                case .flashpointTemperature: result = flashpointTemperature?.search(_search)
                case .additionalTemperature: result = additionalTemperature.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .placardNotation: presentableView(for: placardNotation, title: _title, details: _details, show: show, search: _search)
                case .placardEndorsement: presentableView(for: placardEndorsement, title: _title, details: _details, show: show, search: _search)
                case .additionalInformation: presentableView(for: additionalInformation.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .undgCode: presentableView(for: undgCode, title: _title, details: _details, show: show, search: _search)
                case .emergencyProceduresCode: presentableView(for: emergencyProceduresCode, title: _title, details: _details, show: show, search: _search)
                case .medicalFirstAidGuideCode: presentableView(for: medicalFirstAidGuideCode, title: _title, details: _details, show: show, search: _search)
                case .technicalName: presentableView(for: technicalName, title: _title, details: _details, show: show, search: _search)
                case .categoryName: presentableView(for: categoryName, title: _title, details: _details, show: show, search: _search)
                case .hazardousCategoryCode: presentableView(for: hazardousCategoryCode, title: _title, details: _details, show: show, search: _search)
                case .upperOrangeHazardPlacardID: presentableView(for: upperOrangeHazardPlacardID, title: _title, details: _details, show: show, search: _search)
                case .lowerOrangeHazardPlacardID: presentableView(for: lowerOrangeHazardPlacardID, title: _title, details: _details, show: show, search: _search)
                case .markingID: presentableView(for: markingID, title: _title, details: _details, show: show, search: _search)
                case .hazardClassID: presentableView(for: hazardClassID, title: _title, details: _details, show: show, search: _search)
                case .netWeightMeasure: presentableView(for: netWeightMeasure, title: _title, details: _details, show: show, search: _search)
                case .netVolumeMeasure: presentableView(for: netVolumeMeasure, title: _title, details: _details, show: show, search: _search)
                case .quantity: presentableView(for: quantity, title: _title, details: _details, show: show, search: _search)
                case .contactParty: presentableView(for: contactParty, title: _title, details: _details, show: show, search: _search)
                case .secondaryHazard: presentableView(for: secondaryHazard.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .hazardousGoodsTransit: presentableView(for: hazardousGoodsTransit.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .emergencyTemperature: presentableView(for: emergencyTemperature, title: _title, details: _details, show: show, search: _search)
                case .flashpointTemperature: presentableView(for: flashpointTemperature, title: _title, details: _details, show: show, search: _search)
                case .additionalTemperature: presentableView(for: additionalTemperature.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.HazardousItem: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = placardNotation?.search(string) { return result }
        if let result = placardEndorsement?.search(string) { return result }
        if let result = additionalInformation.compactMap({ $0.search(string)  }).first { return result }
        if let result = undgCode?.search(string) { return result }
        if let result = emergencyProceduresCode?.search(string) { return result }
        if let result = medicalFirstAidGuideCode?.search(string) { return result }
        if let result = technicalName?.search(string) { return result }
        if let result = categoryName?.search(string) { return result }
        if let result = hazardousCategoryCode?.search(string) { return result }
        if let result = upperOrangeHazardPlacardID?.search(string) { return result }
        if let result = lowerOrangeHazardPlacardID?.search(string) { return result }
        if let result = markingID?.search(string) { return result }
        if let result = hazardClassID?.search(string) { return result }
        if let result = netWeightMeasure?.search(string) { return result }
        if let result = netVolumeMeasure?.search(string) { return result }
        if let result = quantity?.search(string) { return result }
        if let result = contactParty?.search(string) { return result }
        if let result = secondaryHazard.compactMap({ $0.search(string)  }).first { return result }
        if let result = hazardousGoodsTransit.compactMap({ $0.search(string)  }).first { return result }
        if let result = emergencyTemperature?.search(string) { return result }
        if let result = flashpointTemperature?.search(string) { return result }
        if let result = additionalTemperature.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
