// 
// Identifier.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Identifier: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .value,
        .schemeID,
        .schemeName,
        .schemeAgencyID,
        .schemeAgencyName,
        .schemeVersionID,
        .schemeDataURI,
        .schemeURI
]
    public var hasNilValue: Bool {(schemeID == nil)
        || (schemeName == nil)
        || (schemeAgencyID == nil)
        || (schemeAgencyName == nil)
        || (schemeVersionID == nil)
        || (schemeDataURI == nil)
        || (schemeURI == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .value: return "Value"
            case .schemeID: return "Scheme Id"
            case .schemeName: return "Scheme Name"
            case .schemeAgencyID: return "Scheme Agency Id"
            case .schemeAgencyName: return "Scheme Agency Name"
            case .schemeVersionID: return "Scheme Version Id"
            case .schemeDataURI: return "Scheme Data Uri"
            case .schemeURI: return "Scheme Uri"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .value: return nil
            case .schemeID: return nil
            case .schemeName: return nil
            case .schemeAgencyID: return nil
            case .schemeAgencyName: return nil
            case .schemeVersionID: return nil
            case .schemeDataURI: return nil
            case .schemeURI: return nil
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .value: result = value.search(_search)
                case .schemeID: result = schemeID?.search(_search)
                case .schemeName: result = schemeName?.search(_search)
                case .schemeAgencyID: result = schemeAgencyID?.search(_search)
                case .schemeAgencyName: result = schemeAgencyName?.search(_search)
                case .schemeVersionID: result = schemeVersionID?.search(_search)
                case .schemeDataURI: result = schemeDataURI?.search(_search)
                case .schemeURI: result = schemeURI?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .value: anyView(for: value, title: _title, details: _details, show: show, search: _search)
                case .schemeID: anyView(for: schemeID, title: _title, details: _details, show: show, search: _search)
                case .schemeName: anyView(for: schemeName, title: _title, details: _details, show: show, search: _search)
                case .schemeAgencyID: anyView(for: schemeAgencyID, title: _title, details: _details, show: show, search: _search)
                case .schemeAgencyName: anyView(for: schemeAgencyName, title: _title, details: _details, show: show, search: _search)
                case .schemeVersionID: anyView(for: schemeVersionID, title: _title, details: _details, show: show, search: _search)
                case .schemeDataURI: anyView(for: schemeDataURI, title: _title, details: _details, show: show, search: _search)
                case .schemeURI: anyView(for: schemeURI, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Identifier: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = value.search(string) { return result }
        if let result = schemeID?.search(string) { return result }
        if let result = schemeName?.search(string) { return result }
        if let result = schemeAgencyID?.search(string) { return result }
        if let result = schemeAgencyName?.search(string) { return result }
        if let result = schemeVersionID?.search(string) { return result }
        if let result = schemeDataURI?.search(string) { return result }
        if let result = schemeURI?.search(string) { return result }
        return nil
    }
}
