// 
// Indicator.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Indicator: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .value,
        .format
]
    public var hasNilValue: Bool {(format == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .value: return "Value"
            case .format: return "Format"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .value: return nil
            case .format: return nil
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .value: result = value.search(_search)
                case .format: result = format?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .value: anyView(for: value, title: _title, details: _details, show: show, search: _search)
                case .format: anyView(for: format, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Indicator: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = value.search(string) { return result }
        if let result = format?.search(string) { return result }
        return nil
    }
}
