// 
// Invoice.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Invoice: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .ublVersionID,
        .customizationID,
        .profileID,
        .profileExecutionID,
        .id,
        .copyIndicator,
        .uuid,
        .issueDate,
        .issueTime,
        .dueDate,
        .invoiceTypeCode,
        .note,
        .taxPointDate,
        .documentCurrencyCode,
        .taxCurrencyCode,
        .pricingCurrencyCode,
        .paymentCurrencyCode,
        .paymentAlternativeCurrencyCode,
        .accountingCostCode,
        .accountingCost,
        .lineCountNumeric,
        .buyerReference,
        .invoicePeriod,
        .orderReference,
        .billingReference,
        .despatchDocumentReference,
        .receiptDocumentReference,
        .statementDocumentReference,
        .originatorDocumentReference,
        .contractDocumentReference,
        .additionalDocumentReference,
        .projectReference,
        .signature,
        .accountingSupplierParty,
        .accountingCustomerParty,
        .payeeParty,
        .buyerCustomerParty,
        .sellerSupplierParty,
        .taxRepresentativeParty,
        .delivery,
        .deliveryTerms,
        .paymentMeans,
        .paymentTerms,
        .prepaidPayment,
        .allowanceCharge,
        .taxExchangeRate,
        .pricingExchangeRate,
        .paymentExchangeRate,
        .paymentAlternativeExchangeRate,
        .taxTotal,
        .withholdingTaxTotal,
        .legalMonetaryTotal,
        .invoiceLine
]
    public var hasNilValue: Bool {(ublVersionID == nil)
        || (customizationID == nil)
        || (profileID == nil)
        || (profileExecutionID == nil)
        || (copyIndicator == nil)
        || (uuid == nil)
        || (issueTime == nil)
        || (dueDate == nil)
        || (invoiceTypeCode == nil)
        || (taxPointDate == nil)
        || (documentCurrencyCode == nil)
        || (taxCurrencyCode == nil)
        || (pricingCurrencyCode == nil)
        || (paymentCurrencyCode == nil)
        || (paymentAlternativeCurrencyCode == nil)
        || (accountingCostCode == nil)
        || (accountingCost == nil)
        || (lineCountNumeric == nil)
        || (buyerReference == nil)
        || (orderReference == nil)
        || (payeeParty == nil)
        || (buyerCustomerParty == nil)
        || (sellerSupplierParty == nil)
        || (taxRepresentativeParty == nil)
        || (deliveryTerms == nil)
        || (taxExchangeRate == nil)
        || (pricingExchangeRate == nil)
        || (paymentExchangeRate == nil)
        || (paymentAlternativeExchangeRate == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .ublVersionID: return "UBL Version Identifier"
            case .customizationID: return "Customization Identifier"
            case .profileID: return "Profile Identifier"
            case .profileExecutionID: return "Profile Execution Identifier"
            case .id: return "Identifier"
            case .copyIndicator: return "Copy Indicator"
            case .uuid: return "UUID"
            case .issueDate: return "Issue Date"
            case .issueTime: return "Issue Time"
            case .dueDate: return "Due Date"
            case .invoiceTypeCode: return "Invoice Type Code"
            case .note: return "Note"
            case .taxPointDate: return "Tax Point Date"
            case .documentCurrencyCode: return "Document Currency Code"
            case .taxCurrencyCode: return "Tax Currency Code"
            case .pricingCurrencyCode: return "Pricing Currency Code"
            case .paymentCurrencyCode: return "Payment Currency Code"
            case .paymentAlternativeCurrencyCode: return "Payment Alternative Currency Code"
            case .accountingCostCode: return "Accounting Cost Code"
            case .accountingCost: return "Accounting Cost"
            case .lineCountNumeric: return "Line Count"
            case .buyerReference: return "Buyer Reference"
            case .invoicePeriod: return "Invoice Period"
            case .orderReference: return "Order Reference"
            case .billingReference: return "Billing Reference"
            case .despatchDocumentReference: return "Despatch Document Reference"
            case .receiptDocumentReference: return "Receipt Document Reference"
            case .statementDocumentReference: return "Statement Document Reference"
            case .originatorDocumentReference: return "Originator Document Reference"
            case .contractDocumentReference: return "Contract Document Reference"
            case .additionalDocumentReference: return "Additional Document Reference"
            case .projectReference: return "Project Reference"
            case .signature: return "Signature"
            case .accountingSupplierParty: return "Accounting Supplier Party"
            case .accountingCustomerParty: return "Accounting Customer Party"
            case .payeeParty: return "Payee Party"
            case .buyerCustomerParty: return "Buyer Customer Party"
            case .sellerSupplierParty: return "Seller Supplier Party"
            case .taxRepresentativeParty: return "Tax Representative Party"
            case .delivery: return "Delivery"
            case .deliveryTerms: return "Delivery Terms"
            case .paymentMeans: return "Payment Means"
            case .paymentTerms: return "Payment Terms"
            case .prepaidPayment: return "Prepaid Payment"
            case .allowanceCharge: return "Allowance Charge"
            case .taxExchangeRate: return "Tax Exchange Rate"
            case .pricingExchangeRate: return "Pricing Exchange Rate"
            case .paymentExchangeRate: return "Payment Exchange Rate"
            case .paymentAlternativeExchangeRate: return "Payment Alternative Exchange Rate"
            case .taxTotal: return "Tax Total"
            case .withholdingTaxTotal: return "Withholding Tax Total"
            case .legalMonetaryTotal: return "Legal Monetary Total"
            case .invoiceLine: return "Invoice Line"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .ublVersionID: return "Identifies the earliest version of the UBL 2 schema for this document type that defines all of the elements that might be encountered in the current instance."
            case .customizationID: return "Identifies a user-defined customization of UBL for a specific use."
            case .profileID: return "Identifies a user-defined profile of the customization of UBL being used."
            case .profileExecutionID: return "Identifies an instance of executing a profile, to associate all transactions in a collaboration."
            case .id: return "An identifier for this document, assigned by the sender."
            case .copyIndicator: return "Indicates whether this document is a copy (true) or not (false)."
            case .uuid: return "A universally unique identifier for an instance of this document."
            case .issueDate: return "The date, assigned by the sender, on which this document was issued."
            case .issueTime: return "The time, assigned by the sender, at which this document was issued."
            case .dueDate: return "The date on which Invoice is due."
            case .invoiceTypeCode: return "A code signifying the type of the Invoice."
            case .note: return "Free-form text pertinent to this document, conveying information that is not contained explicitly in other structures."
            case .taxPointDate: return "The date of the Invoice, used to indicate the point at which tax becomes applicable."
            case .documentCurrencyCode: return "A code signifying the default currency for this document."
            case .taxCurrencyCode: return "A code signifying the currency used for tax amounts in the Invoice."
            case .pricingCurrencyCode: return "A code signifying the currency used for prices in the Invoice."
            case .paymentCurrencyCode: return "A code signifying the currency used for payment in the Invoice."
            case .paymentAlternativeCurrencyCode: return "A code signifying the alternative currency used for payment in the Invoice."
            case .accountingCostCode: return "The buyer's accounting code, applied to the Invoice as a whole."
            case .accountingCost: return "The buyer's accounting code, applied to the Invoice as a whole, expressed as text."
            case .lineCountNumeric: return "The number of lines in the document."
            case .buyerReference: return "A reference provided by the buyer used for internal routing of the document."
            case .invoicePeriod: return "A period to which the Invoice applies."
            case .orderReference: return "A reference to the Order with which this Invoice is associated."
            case .billingReference: return "A reference to a billing document associated with this document."
            case .despatchDocumentReference: return "A reference to a Despatch Advice associated with this document."
            case .receiptDocumentReference: return "A reference to a Receipt Advice associated with this document."
            case .statementDocumentReference: return "A reference to a Statement associated with this document."
            case .originatorDocumentReference: return "A reference to an originator document associated with this document."
            case .contractDocumentReference: return "A reference to a contract associated with this document."
            case .additionalDocumentReference: return "A reference to an additional document associated with this document."
            case .projectReference: return "Information about a project."
            case .signature: return "A signature applied to this document."
            case .accountingSupplierParty: return "The accounting supplier party."
            case .accountingCustomerParty: return "The accounting customer party."
            case .payeeParty: return "The payee."
            case .buyerCustomerParty: return "The buyer."
            case .sellerSupplierParty: return "The seller."
            case .taxRepresentativeParty: return "The tax representative."
            case .delivery: return "A delivery associated with this document."
            case .deliveryTerms: return "A set of delivery terms associated with this document."
            case .paymentMeans: return "Expected means of payment."
            case .paymentTerms: return "A set of payment terms associated with this document."
            case .prepaidPayment: return "A prepaid payment."
            case .allowanceCharge: return "A discount or charge that applies to a price component."
            case .taxExchangeRate: return "The exchange rate between the document currency and the tax currency."
            case .pricingExchangeRate: return "The exchange rate between the document currency and the pricing currency."
            case .paymentExchangeRate: return "The exchange rate between the document currency and the payment currency."
            case .paymentAlternativeExchangeRate: return "The exchange rate between the document currency and the payment alternative currency."
            case .taxTotal: return "The total amount of a specific type of tax."
            case .withholdingTaxTotal: return "The total withholding tax."
            case .legalMonetaryTotal: return "The total amount payable on the Invoice, including Allowances, Charges, and Taxes."
            case .invoiceLine: return "A line describing an invoice item."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .ublVersionID: result = ublVersionID?.search(_search)
                case .customizationID: result = customizationID?.search(_search)
                case .profileID: result = profileID?.search(_search)
                case .profileExecutionID: result = profileExecutionID?.search(_search)
                case .id: result = id.search(_search)
                case .copyIndicator: result = copyIndicator?.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .issueDate: result = issueDate.search(_search)
                case .issueTime: result = issueTime?.search(_search)
                case .dueDate: result = dueDate?.search(_search)
                case .invoiceTypeCode: result = invoiceTypeCode?.search(_search)
                case .note: result = note.compactMap({ $0.search(_search) }).first
                case .taxPointDate: result = taxPointDate?.search(_search)
                case .documentCurrencyCode: result = documentCurrencyCode?.search(_search)
                case .taxCurrencyCode: result = taxCurrencyCode?.search(_search)
                case .pricingCurrencyCode: result = pricingCurrencyCode?.search(_search)
                case .paymentCurrencyCode: result = paymentCurrencyCode?.search(_search)
                case .paymentAlternativeCurrencyCode: result = paymentAlternativeCurrencyCode?.search(_search)
                case .accountingCostCode: result = accountingCostCode?.search(_search)
                case .accountingCost: result = accountingCost?.search(_search)
                case .lineCountNumeric: result = lineCountNumeric?.search(_search)
                case .buyerReference: result = buyerReference?.search(_search)
                case .invoicePeriod: result = invoicePeriod.compactMap({ $0.search(_search) }).first
                case .orderReference: result = orderReference?.search(_search)
                case .billingReference: result = billingReference.compactMap({ $0.search(_search) }).first
                case .despatchDocumentReference: result = despatchDocumentReference.compactMap({ $0.search(_search) }).first
                case .receiptDocumentReference: result = receiptDocumentReference.compactMap({ $0.search(_search) }).first
                case .statementDocumentReference: result = statementDocumentReference.compactMap({ $0.search(_search) }).first
                case .originatorDocumentReference: result = originatorDocumentReference.compactMap({ $0.search(_search) }).first
                case .contractDocumentReference: result = contractDocumentReference.compactMap({ $0.search(_search) }).first
                case .additionalDocumentReference: result = additionalDocumentReference.compactMap({ $0.search(_search) }).first
                case .projectReference: result = projectReference.compactMap({ $0.search(_search) }).first
                case .signature: result = signature.compactMap({ $0.search(_search) }).first
                case .accountingSupplierParty: result = accountingSupplierParty.search(_search)
                case .accountingCustomerParty: result = accountingCustomerParty.search(_search)
                case .payeeParty: result = payeeParty?.search(_search)
                case .buyerCustomerParty: result = buyerCustomerParty?.search(_search)
                case .sellerSupplierParty: result = sellerSupplierParty?.search(_search)
                case .taxRepresentativeParty: result = taxRepresentativeParty?.search(_search)
                case .delivery: result = delivery.compactMap({ $0.search(_search) }).first
                case .deliveryTerms: result = deliveryTerms?.search(_search)
                case .paymentMeans: result = paymentMeans.compactMap({ $0.search(_search) }).first
                case .paymentTerms: result = paymentTerms.compactMap({ $0.search(_search) }).first
                case .prepaidPayment: result = prepaidPayment.compactMap({ $0.search(_search) }).first
                case .allowanceCharge: result = allowanceCharge.compactMap({ $0.search(_search) }).first
                case .taxExchangeRate: result = taxExchangeRate?.search(_search)
                case .pricingExchangeRate: result = pricingExchangeRate?.search(_search)
                case .paymentExchangeRate: result = paymentExchangeRate?.search(_search)
                case .paymentAlternativeExchangeRate: result = paymentAlternativeExchangeRate?.search(_search)
                case .taxTotal: result = taxTotal.compactMap({ $0.search(_search) }).first
                case .withholdingTaxTotal: result = withholdingTaxTotal.compactMap({ $0.search(_search) }).first
                case .legalMonetaryTotal: result = legalMonetaryTotal.search(_search)
                case .invoiceLine: result = invoiceLine.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .ublVersionID: presentableView(for: ublVersionID, title: _title, details: _details, show: show, search: _search)
                case .customizationID: presentableView(for: customizationID, title: _title, details: _details, show: show, search: _search)
                case .profileID: presentableView(for: profileID, title: _title, details: _details, show: show, search: _search)
                case .profileExecutionID: presentableView(for: profileExecutionID, title: _title, details: _details, show: show, search: _search)
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .copyIndicator: presentableView(for: copyIndicator, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .issueDate: anyView(for: issueDate, title: _title, details: _details, show: show, search: _search)
                case .issueTime: anyView(for: issueTime, title: _title, details: _details, show: show, search: _search)
                case .dueDate: anyView(for: dueDate, title: _title, details: _details, show: show, search: _search)
                case .invoiceTypeCode: presentableView(for: invoiceTypeCode, title: _title, details: _details, show: show, search: _search)
                case .note: presentableView(for: note.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .taxPointDate: anyView(for: taxPointDate, title: _title, details: _details, show: show, search: _search)
                case .documentCurrencyCode: presentableView(for: documentCurrencyCode, title: _title, details: _details, show: show, search: _search)
                case .taxCurrencyCode: presentableView(for: taxCurrencyCode, title: _title, details: _details, show: show, search: _search)
                case .pricingCurrencyCode: presentableView(for: pricingCurrencyCode, title: _title, details: _details, show: show, search: _search)
                case .paymentCurrencyCode: presentableView(for: paymentCurrencyCode, title: _title, details: _details, show: show, search: _search)
                case .paymentAlternativeCurrencyCode: presentableView(for: paymentAlternativeCurrencyCode, title: _title, details: _details, show: show, search: _search)
                case .accountingCostCode: presentableView(for: accountingCostCode, title: _title, details: _details, show: show, search: _search)
                case .accountingCost: presentableView(for: accountingCost, title: _title, details: _details, show: show, search: _search)
                case .lineCountNumeric: presentableView(for: lineCountNumeric, title: _title, details: _details, show: show, search: _search)
                case .buyerReference: presentableView(for: buyerReference, title: _title, details: _details, show: show, search: _search)
                case .invoicePeriod: presentableView(for: invoicePeriod.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .orderReference: presentableView(for: orderReference, title: _title, details: _details, show: show, search: _search)
                case .billingReference: presentableView(for: billingReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .despatchDocumentReference: presentableView(for: despatchDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .receiptDocumentReference: presentableView(for: receiptDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .statementDocumentReference: presentableView(for: statementDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .originatorDocumentReference: presentableView(for: originatorDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .contractDocumentReference: presentableView(for: contractDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .additionalDocumentReference: presentableView(for: additionalDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .projectReference: presentableView(for: projectReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .signature: presentableView(for: signature.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .accountingSupplierParty: presentableView(for: accountingSupplierParty, title: _title, details: _details, show: show, search: _search)
                case .accountingCustomerParty: presentableView(for: accountingCustomerParty, title: _title, details: _details, show: show, search: _search)
                case .payeeParty: presentableView(for: payeeParty, title: _title, details: _details, show: show, search: _search)
                case .buyerCustomerParty: presentableView(for: buyerCustomerParty, title: _title, details: _details, show: show, search: _search)
                case .sellerSupplierParty: presentableView(for: sellerSupplierParty, title: _title, details: _details, show: show, search: _search)
                case .taxRepresentativeParty: presentableView(for: taxRepresentativeParty, title: _title, details: _details, show: show, search: _search)
                case .delivery: presentableView(for: delivery.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .deliveryTerms: presentableView(for: deliveryTerms, title: _title, details: _details, show: show, search: _search)
                case .paymentMeans: presentableView(for: paymentMeans.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .paymentTerms: presentableView(for: paymentTerms.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .prepaidPayment: presentableView(for: prepaidPayment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .allowanceCharge: presentableView(for: allowanceCharge.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .taxExchangeRate: presentableView(for: taxExchangeRate, title: _title, details: _details, show: show, search: _search)
                case .pricingExchangeRate: presentableView(for: pricingExchangeRate, title: _title, details: _details, show: show, search: _search)
                case .paymentExchangeRate: presentableView(for: paymentExchangeRate, title: _title, details: _details, show: show, search: _search)
                case .paymentAlternativeExchangeRate: presentableView(for: paymentAlternativeExchangeRate, title: _title, details: _details, show: show, search: _search)
                case .taxTotal: presentableView(for: taxTotal.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .withholdingTaxTotal: presentableView(for: withholdingTaxTotal.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .legalMonetaryTotal: presentableView(for: legalMonetaryTotal, title: _title, details: _details, show: show, search: _search)
                case .invoiceLine: presentableView(for: invoiceLine.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Invoice: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = issueDate.search(string) { return result }
        if let result = issueTime?.search(string) { return result }
        if let result = dueDate?.search(string) { return result }
        if let result = taxPointDate?.search(string) { return result }
        if let result = ublVersionID?.search(string) { return result }
        if let result = customizationID?.search(string) { return result }
        if let result = profileID?.search(string) { return result }
        if let result = profileExecutionID?.search(string) { return result }
        if let result = id.search(string) { return result }
        if let result = copyIndicator?.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = invoiceTypeCode?.search(string) { return result }
        if let result = note.compactMap({ $0.search(string)  }).first { return result }
        if let result = documentCurrencyCode?.search(string) { return result }
        if let result = taxCurrencyCode?.search(string) { return result }
        if let result = pricingCurrencyCode?.search(string) { return result }
        if let result = paymentCurrencyCode?.search(string) { return result }
        if let result = paymentAlternativeCurrencyCode?.search(string) { return result }
        if let result = accountingCostCode?.search(string) { return result }
        if let result = accountingCost?.search(string) { return result }
        if let result = lineCountNumeric?.search(string) { return result }
        if let result = buyerReference?.search(string) { return result }
        if let result = invoicePeriod.compactMap({ $0.search(string)  }).first { return result }
        if let result = orderReference?.search(string) { return result }
        if let result = billingReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = despatchDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = receiptDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = statementDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = originatorDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = contractDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = additionalDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = projectReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = signature.compactMap({ $0.search(string)  }).first { return result }
        if let result = accountingSupplierParty.search(string) { return result }
        if let result = accountingCustomerParty.search(string) { return result }
        if let result = payeeParty?.search(string) { return result }
        if let result = buyerCustomerParty?.search(string) { return result }
        if let result = sellerSupplierParty?.search(string) { return result }
        if let result = taxRepresentativeParty?.search(string) { return result }
        if let result = delivery.compactMap({ $0.search(string)  }).first { return result }
        if let result = deliveryTerms?.search(string) { return result }
        if let result = paymentMeans.compactMap({ $0.search(string)  }).first { return result }
        if let result = paymentTerms.compactMap({ $0.search(string)  }).first { return result }
        if let result = prepaidPayment.compactMap({ $0.search(string)  }).first { return result }
        if let result = allowanceCharge.compactMap({ $0.search(string)  }).first { return result }
        if let result = taxExchangeRate?.search(string) { return result }
        if let result = pricingExchangeRate?.search(string) { return result }
        if let result = paymentExchangeRate?.search(string) { return result }
        if let result = paymentAlternativeExchangeRate?.search(string) { return result }
        if let result = taxTotal.compactMap({ $0.search(string)  }).first { return result }
        if let result = withholdingTaxTotal.compactMap({ $0.search(string)  }).first { return result }
        if let result = legalMonetaryTotal.search(string) { return result }
        if let result = invoiceLine.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
