// 
// InvoiceLine.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.InvoiceLine: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .uuid,
        .note,
        .invoicedQuantity,
        .lineExtensionAmount,
        .taxPointDate,
        .accountingCostCode,
        .accountingCost,
        .paymentPurposeCode,
        .freeOfChargeIndicator,
        .invoicePeriod,
        .orderLineReference,
        .despatchLineReference,
        .receiptLineReference,
        .billingReference,
        .documentReference,
        .pricingReference,
        .originatorParty,
        .delivery,
        .paymentTerms,
        .allowanceCharge,
        .taxTotal,
        .withholdingTaxTotal,
        .item,
        .price,
        .deliveryTerms,
        .subInvoiceLine,
        .itemPriceExtension
]
    public var hasNilValue: Bool {(uuid == nil)
        || (invoicedQuantity == nil)
        || (taxPointDate == nil)
        || (accountingCostCode == nil)
        || (accountingCost == nil)
        || (paymentPurposeCode == nil)
        || (freeOfChargeIndicator == nil)
        || (pricingReference == nil)
        || (originatorParty == nil)
        || (price == nil)
        || (deliveryTerms == nil)
        || (itemPriceExtension == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .uuid: return "UUID"
            case .note: return "Note"
            case .invoicedQuantity: return "Invoiced Quantity"
            case .lineExtensionAmount: return "Line Extension Amount"
            case .taxPointDate: return "Tax Point Date"
            case .accountingCostCode: return "Accounting Cost Code"
            case .accountingCost: return "Accounting Cost"
            case .paymentPurposeCode: return "Payment Purpose Code"
            case .freeOfChargeIndicator: return "Free Of Charge Indicator"
            case .invoicePeriod: return "Invoice Period"
            case .orderLineReference: return "Order Line Reference"
            case .despatchLineReference: return "Despatch Line Reference"
            case .receiptLineReference: return "Receipt Line Reference"
            case .billingReference: return "Billing Reference"
            case .documentReference: return "Document Reference"
            case .pricingReference: return "Pricing Reference"
            case .originatorParty: return "Originator Party"
            case .delivery: return "Delivery"
            case .paymentTerms: return "Payment Terms"
            case .allowanceCharge: return "Allowance Charge"
            case .taxTotal: return "Tax Total"
            case .withholdingTaxTotal: return "Withholding Tax Total"
            case .item: return "Item"
            case .price: return "Price"
            case .deliveryTerms: return "Delivery Terms"
            case .subInvoiceLine: return "Sub Invoice Line"
            case .itemPriceExtension: return "Item Price Extension"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this invoice line."
            case .uuid: return "A universally unique identifier for this invoice line."
            case .note: return "Free-form text conveying information that is not contained explicitly in other structures."
            case .invoicedQuantity: return "The quantity (of items) on this invoice line."
            case .lineExtensionAmount: return "The total amount for this invoice line, including allowance charges but net of taxes."
            case .taxPointDate: return "The date of this invoice line, used to indicate the point at which tax becomes applicable."
            case .accountingCostCode: return "The buyer's accounting cost centre for this invoice line, expressed as a code."
            case .accountingCost: return "The buyer's accounting cost centre for this invoice line, expressed as text."
            case .paymentPurposeCode: return "A code signifying the business purpose for this payment."
            case .freeOfChargeIndicator: return "An indicator that this invoice line is free of charge (true) or not (false). The default is false."
            case .invoicePeriod: return "An invoice period to which this invoice line applies."
            case .orderLineReference: return "A reference to an order line associated with this invoice line."
            case .despatchLineReference: return "A reference to a despatch line associated with this invoice line."
            case .receiptLineReference: return "A reference to a receipt line associated with this invoice line."
            case .billingReference: return "A reference to a billing document associated with this invoice line."
            case .documentReference: return "A reference to a document associated with this invoice line."
            case .pricingReference: return "A reference to pricing and item location information associated with this invoice line."
            case .originatorParty: return "The party who originated the Order to which the Invoice is related."
            case .delivery: return "A delivery associated with this invoice line."
            case .paymentTerms: return "A specification of payment terms associated with this invoice line."
            case .allowanceCharge: return "An allowance or charge associated with this invoice line."
            case .taxTotal: return "A total amount of taxes of a particular kind applicable to this invoice line."
            case .withholdingTaxTotal: return "A reference to a TaxTotal class describing the amount that has been withhold by the authorities, e.g. if the creditor is in dept because of non paid taxes."
            case .item: return "The item associated with this invoice line."
            case .price: return "The price of the item associated with this invoice line."
            case .deliveryTerms: return "Terms and conditions of the delivery associated with this invoice line."
            case .subInvoiceLine: return "An invoice line subsidiary to this invoice line."
            case .itemPriceExtension: return "The price extension, calculated by multiplying the price per unit by the quantity of items on this invoice line."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .note: result = note.compactMap({ $0.search(_search) }).first
                case .invoicedQuantity: result = invoicedQuantity?.search(_search)
                case .lineExtensionAmount: result = lineExtensionAmount.search(_search)
                case .taxPointDate: result = taxPointDate?.search(_search)
                case .accountingCostCode: result = accountingCostCode?.search(_search)
                case .accountingCost: result = accountingCost?.search(_search)
                case .paymentPurposeCode: result = paymentPurposeCode?.search(_search)
                case .freeOfChargeIndicator: result = freeOfChargeIndicator?.search(_search)
                case .invoicePeriod: result = invoicePeriod.compactMap({ $0.search(_search) }).first
                case .orderLineReference: result = orderLineReference.compactMap({ $0.search(_search) }).first
                case .despatchLineReference: result = despatchLineReference.compactMap({ $0.search(_search) }).first
                case .receiptLineReference: result = receiptLineReference.compactMap({ $0.search(_search) }).first
                case .billingReference: result = billingReference.compactMap({ $0.search(_search) }).first
                case .documentReference: result = documentReference.compactMap({ $0.search(_search) }).first
                case .pricingReference: result = pricingReference?.search(_search)
                case .originatorParty: result = originatorParty?.search(_search)
                case .delivery: result = delivery.compactMap({ $0.search(_search) }).first
                case .paymentTerms: result = paymentTerms.compactMap({ $0.search(_search) }).first
                case .allowanceCharge: result = allowanceCharge.compactMap({ $0.search(_search) }).first
                case .taxTotal: result = taxTotal.compactMap({ $0.search(_search) }).first
                case .withholdingTaxTotal: result = withholdingTaxTotal.compactMap({ $0.search(_search) }).first
                case .item: result = item.search(_search)
                case .price: result = price?.search(_search)
                case .deliveryTerms: result = deliveryTerms?.search(_search)
                case .subInvoiceLine: result = subInvoiceLine.compactMap({ $0.search(_search) }).first
                case .itemPriceExtension: result = itemPriceExtension?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .note: presentableView(for: note.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .invoicedQuantity: presentableView(for: invoicedQuantity, title: _title, details: _details, show: show, search: _search)
                case .lineExtensionAmount: presentableView(for: lineExtensionAmount, title: _title, details: _details, show: show, search: _search)
                case .taxPointDate: anyView(for: taxPointDate, title: _title, details: _details, show: show, search: _search)
                case .accountingCostCode: presentableView(for: accountingCostCode, title: _title, details: _details, show: show, search: _search)
                case .accountingCost: presentableView(for: accountingCost, title: _title, details: _details, show: show, search: _search)
                case .paymentPurposeCode: presentableView(for: paymentPurposeCode, title: _title, details: _details, show: show, search: _search)
                case .freeOfChargeIndicator: presentableView(for: freeOfChargeIndicator, title: _title, details: _details, show: show, search: _search)
                case .invoicePeriod: presentableView(for: invoicePeriod.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .orderLineReference: presentableView(for: orderLineReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .despatchLineReference: presentableView(for: despatchLineReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .receiptLineReference: presentableView(for: receiptLineReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .billingReference: presentableView(for: billingReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .documentReference: presentableView(for: documentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .pricingReference: presentableView(for: pricingReference, title: _title, details: _details, show: show, search: _search)
                case .originatorParty: presentableView(for: originatorParty, title: _title, details: _details, show: show, search: _search)
                case .delivery: presentableView(for: delivery.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .paymentTerms: presentableView(for: paymentTerms.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .allowanceCharge: presentableView(for: allowanceCharge.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .taxTotal: presentableView(for: taxTotal.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .withholdingTaxTotal: presentableView(for: withholdingTaxTotal.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .item: presentableView(for: item, title: _title, details: _details, show: show, search: _search)
                case .price: presentableView(for: price, title: _title, details: _details, show: show, search: _search)
                case .deliveryTerms: presentableView(for: deliveryTerms, title: _title, details: _details, show: show, search: _search)
                case .subInvoiceLine: presentableView(for: subInvoiceLine.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .itemPriceExtension: presentableView(for: itemPriceExtension, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.InvoiceLine: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = taxPointDate?.search(string) { return result }
        if let result = id.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = note.compactMap({ $0.search(string)  }).first { return result }
        if let result = invoicedQuantity?.search(string) { return result }
        if let result = lineExtensionAmount.search(string) { return result }
        if let result = accountingCostCode?.search(string) { return result }
        if let result = accountingCost?.search(string) { return result }
        if let result = paymentPurposeCode?.search(string) { return result }
        if let result = freeOfChargeIndicator?.search(string) { return result }
        if let result = invoicePeriod.compactMap({ $0.search(string)  }).first { return result }
        if let result = orderLineReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = despatchLineReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = receiptLineReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = billingReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = documentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = pricingReference?.search(string) { return result }
        if let result = originatorParty?.search(string) { return result }
        if let result = delivery.compactMap({ $0.search(string)  }).first { return result }
        if let result = paymentTerms.compactMap({ $0.search(string)  }).first { return result }
        if let result = allowanceCharge.compactMap({ $0.search(string)  }).first { return result }
        if let result = taxTotal.compactMap({ $0.search(string)  }).first { return result }
        if let result = withholdingTaxTotal.compactMap({ $0.search(string)  }).first { return result }
        if let result = item.search(string) { return result }
        if let result = price?.search(string) { return result }
        if let result = deliveryTerms?.search(string) { return result }
        if let result = subInvoiceLine.compactMap({ $0.search(string)  }).first { return result }
        if let result = itemPriceExtension?.search(string) { return result }
        return nil
    }
}
