// 
// Item.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Item: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .description,
        .packQuantity,
        .packSizeNumeric,
        .catalogueIndicator,
        .name,
        .hazardousRiskIndicator,
        .additionalInformation,
        .keyword,
        .brandName,
        .modelName,
        .buyersItemIdentification,
        .sellersItemIdentification,
        .manufacturersItemIdentification,
        .standardItemIdentification,
        .catalogueItemIdentification,
        .additionalItemIdentification,
        .catalogueDocumentReference,
        .itemSpecificationDocumentReference,
        .originCountry,
        .commodityClassification,
        .transactionConditions,
        .hazardousItem,
        .classifiedTaxCategory,
        .additionalItemProperty,
        .manufacturerParty,
        .informationContentProviderParty,
        .originAddress,
        .itemInstance,
        .certificate,
        .dimension
]
    public var hasNilValue: Bool {(packQuantity == nil)
        || (packSizeNumeric == nil)
        || (catalogueIndicator == nil)
        || (name == nil)
        || (hazardousRiskIndicator == nil)
        || (buyersItemIdentification == nil)
        || (sellersItemIdentification == nil)
        || (standardItemIdentification == nil)
        || (catalogueItemIdentification == nil)
        || (catalogueDocumentReference == nil)
        || (originCountry == nil)
        || (informationContentProviderParty == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .description: return "Description"
            case .packQuantity: return "Pack Quantity"
            case .packSizeNumeric: return "Pack Size"
            case .catalogueIndicator: return "Catalogue Indicator"
            case .name: return "Name"
            case .hazardousRiskIndicator: return "Hazardous Risk Indicator"
            case .additionalInformation: return "Additional Information"
            case .keyword: return "Keyword"
            case .brandName: return "Brand Name"
            case .modelName: return "Model Name"
            case .buyersItemIdentification: return "Buyers Item Identification"
            case .sellersItemIdentification: return "Sellers Item Identification"
            case .manufacturersItemIdentification: return "Manufacturers Item Identification"
            case .standardItemIdentification: return "Standard Item Identification"
            case .catalogueItemIdentification: return "Catalogue Item Identification"
            case .additionalItemIdentification: return "Additional Item Identification"
            case .catalogueDocumentReference: return "Catalogue Document Reference"
            case .itemSpecificationDocumentReference: return "Item Specification Document Reference"
            case .originCountry: return "Origin Country"
            case .commodityClassification: return "Commodity Classification"
            case .transactionConditions: return "Transaction Conditions"
            case .hazardousItem: return "Hazardous Item"
            case .classifiedTaxCategory: return "Classified Tax Category"
            case .additionalItemProperty: return "Additional Item Property"
            case .manufacturerParty: return "Manufacturer Party"
            case .informationContentProviderParty: return "Information Content Provider Party"
            case .originAddress: return "Origin Address"
            case .itemInstance: return "Item Instance"
            case .certificate: return "Certificate"
            case .dimension: return "Dimension"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .description: return "Text describing this item."
            case .packQuantity: return "The unit packaging quantity; the number of subunits making up this item."
            case .packSizeNumeric: return "The number of items in a pack of this item."
            case .catalogueIndicator: return "An indicator that this item was ordered from a catalogue (true) or not (false)."
            case .name: return "A short name optionally given to this item, such as a name from a catalogue, as distinct from a description."
            case .hazardousRiskIndicator: return "An indication that the transported item, as delivered, is subject to an international regulation concerning the carriage of dangerous goods (true) or not (false)."
            case .additionalInformation: return "Further details regarding this item (e.g., the URL of a relevant web page)."
            case .keyword: return "A keyword (search string) for this item, assigned by the seller party. Can also be a synonym for the name of the item."
            case .brandName: return "A brand name of this item."
            case .modelName: return "A model name of this item."
            case .buyersItemIdentification: return "Identifying information for this item, assigned by the buyer."
            case .sellersItemIdentification: return "Identifying information for this item, assigned by the seller."
            case .manufacturersItemIdentification: return "Identifying information for this item, assigned by the manufacturer."
            case .standardItemIdentification: return "Identifying information for this item, assigned according to a standard system."
            case .catalogueItemIdentification: return "Identifying information for this item, assigned according to a cataloguing system."
            case .additionalItemIdentification: return "An additional identifier for this item."
            case .catalogueDocumentReference: return "A reference to the catalogue in which this item appears."
            case .itemSpecificationDocumentReference: return "A reference to a specification document for this item."
            case .originCountry: return "The country of origin of this item."
            case .commodityClassification: return "A classification of this item according to a specific system for classifying commodities."
            case .transactionConditions: return "A set of sales conditions applying to this item."
            case .hazardousItem: return "Information pertaining to this item as a hazardous item."
            case .classifiedTaxCategory: return "A tax category applicable to this item."
            case .additionalItemProperty: return "An additional property of this item."
            case .manufacturerParty: return "The manufacturer of this item."
            case .informationContentProviderParty: return "The party responsible for specification of this item."
            case .originAddress: return "A region (not country) of origin of this item."
            case .itemInstance: return "A trackable, unique instantiation of this item."
            case .certificate: return "A certificate associated with this item."
            case .dimension: return "One of the measurable dimensions (length, mass, weight, or volume) of this item."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .description: result = description.compactMap({ $0.search(_search) }).first
                case .packQuantity: result = packQuantity?.search(_search)
                case .packSizeNumeric: result = packSizeNumeric?.search(_search)
                case .catalogueIndicator: result = catalogueIndicator?.search(_search)
                case .name: result = name?.search(_search)
                case .hazardousRiskIndicator: result = hazardousRiskIndicator?.search(_search)
                case .additionalInformation: result = additionalInformation.compactMap({ $0.search(_search) }).first
                case .keyword: result = keyword.compactMap({ $0.search(_search) }).first
                case .brandName: result = brandName.compactMap({ $0.search(_search) }).first
                case .modelName: result = modelName.compactMap({ $0.search(_search) }).first
                case .buyersItemIdentification: result = buyersItemIdentification?.search(_search)
                case .sellersItemIdentification: result = sellersItemIdentification?.search(_search)
                case .manufacturersItemIdentification: result = manufacturersItemIdentification.compactMap({ $0.search(_search) }).first
                case .standardItemIdentification: result = standardItemIdentification?.search(_search)
                case .catalogueItemIdentification: result = catalogueItemIdentification?.search(_search)
                case .additionalItemIdentification: result = additionalItemIdentification.compactMap({ $0.search(_search) }).first
                case .catalogueDocumentReference: result = catalogueDocumentReference?.search(_search)
                case .itemSpecificationDocumentReference: result = itemSpecificationDocumentReference.compactMap({ $0.search(_search) }).first
                case .originCountry: result = originCountry?.search(_search)
                case .commodityClassification: result = commodityClassification.compactMap({ $0.search(_search) }).first
                case .transactionConditions: result = transactionConditions.compactMap({ $0.search(_search) }).first
                case .hazardousItem: result = hazardousItem.compactMap({ $0.search(_search) }).first
                case .classifiedTaxCategory: result = classifiedTaxCategory.compactMap({ $0.search(_search) }).first
                case .additionalItemProperty: result = additionalItemProperty.compactMap({ $0.search(_search) }).first
                case .manufacturerParty: result = manufacturerParty.compactMap({ $0.search(_search) }).first
                case .informationContentProviderParty: result = informationContentProviderParty?.search(_search)
                case .originAddress: result = originAddress.compactMap({ $0.search(_search) }).first
                case .itemInstance: result = itemInstance.compactMap({ $0.search(_search) }).first
                case .certificate: result = certificate.compactMap({ $0.search(_search) }).first
                case .dimension: result = dimension.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .packQuantity: presentableView(for: packQuantity, title: _title, details: _details, show: show, search: _search)
                case .packSizeNumeric: presentableView(for: packSizeNumeric, title: _title, details: _details, show: show, search: _search)
                case .catalogueIndicator: presentableView(for: catalogueIndicator, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .hazardousRiskIndicator: presentableView(for: hazardousRiskIndicator, title: _title, details: _details, show: show, search: _search)
                case .additionalInformation: presentableView(for: additionalInformation.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .keyword: presentableView(for: keyword.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .brandName: presentableView(for: brandName.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .modelName: presentableView(for: modelName.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .buyersItemIdentification: presentableView(for: buyersItemIdentification, title: _title, details: _details, show: show, search: _search)
                case .sellersItemIdentification: presentableView(for: sellersItemIdentification, title: _title, details: _details, show: show, search: _search)
                case .manufacturersItemIdentification: presentableView(for: manufacturersItemIdentification.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .standardItemIdentification: presentableView(for: standardItemIdentification, title: _title, details: _details, show: show, search: _search)
                case .catalogueItemIdentification: presentableView(for: catalogueItemIdentification, title: _title, details: _details, show: show, search: _search)
                case .additionalItemIdentification: presentableView(for: additionalItemIdentification.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .catalogueDocumentReference: presentableView(for: catalogueDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .itemSpecificationDocumentReference: presentableView(for: itemSpecificationDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .originCountry: presentableView(for: originCountry, title: _title, details: _details, show: show, search: _search)
                case .commodityClassification: presentableView(for: commodityClassification.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .transactionConditions: presentableView(for: transactionConditions.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .hazardousItem: presentableView(for: hazardousItem.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .classifiedTaxCategory: presentableView(for: classifiedTaxCategory.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .additionalItemProperty: presentableView(for: additionalItemProperty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .manufacturerParty: presentableView(for: manufacturerParty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .informationContentProviderParty: presentableView(for: informationContentProviderParty, title: _title, details: _details, show: show, search: _search)
                case .originAddress: presentableView(for: originAddress.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .itemInstance: presentableView(for: itemInstance.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .certificate: presentableView(for: certificate.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .dimension: presentableView(for: dimension.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Item: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        if let result = packQuantity?.search(string) { return result }
        if let result = packSizeNumeric?.search(string) { return result }
        if let result = catalogueIndicator?.search(string) { return result }
        if let result = name?.search(string) { return result }
        if let result = hazardousRiskIndicator?.search(string) { return result }
        if let result = additionalInformation.compactMap({ $0.search(string)  }).first { return result }
        if let result = keyword.compactMap({ $0.search(string)  }).first { return result }
        if let result = brandName.compactMap({ $0.search(string)  }).first { return result }
        if let result = modelName.compactMap({ $0.search(string)  }).first { return result }
        if let result = buyersItemIdentification?.search(string) { return result }
        if let result = sellersItemIdentification?.search(string) { return result }
        if let result = manufacturersItemIdentification.compactMap({ $0.search(string)  }).first { return result }
        if let result = standardItemIdentification?.search(string) { return result }
        if let result = catalogueItemIdentification?.search(string) { return result }
        if let result = additionalItemIdentification.compactMap({ $0.search(string)  }).first { return result }
        if let result = catalogueDocumentReference?.search(string) { return result }
        if let result = itemSpecificationDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = originCountry?.search(string) { return result }
        if let result = commodityClassification.compactMap({ $0.search(string)  }).first { return result }
        if let result = transactionConditions.compactMap({ $0.search(string)  }).first { return result }
        if let result = hazardousItem.compactMap({ $0.search(string)  }).first { return result }
        if let result = classifiedTaxCategory.compactMap({ $0.search(string)  }).first { return result }
        if let result = additionalItemProperty.compactMap({ $0.search(string)  }).first { return result }
        if let result = manufacturerParty.compactMap({ $0.search(string)  }).first { return result }
        if let result = informationContentProviderParty?.search(string) { return result }
        if let result = originAddress.compactMap({ $0.search(string)  }).first { return result }
        if let result = itemInstance.compactMap({ $0.search(string)  }).first { return result }
        if let result = certificate.compactMap({ $0.search(string)  }).first { return result }
        if let result = dimension.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
