// 
// ItemIdentification.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ItemIdentification: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .extendedID,
        .barcodeSymbologyID,
        .physicalAttribute,
        .measurementDimension,
        .issuerParty
]
    public var hasNilValue: Bool {(extendedID == nil)
        || (barcodeSymbologyID == nil)
        || (issuerParty == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .extendedID: return "Extended Identifier"
            case .barcodeSymbologyID: return "Barcode Symbology Identifier"
            case .physicalAttribute: return "Physical Attribute"
            case .measurementDimension: return "Measurement Dimension"
            case .issuerParty: return "Issuer Party"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for the item."
            case .extendedID: return "An extended identifier for the item that identifies the item with specific properties, e.g., Item 123 = Chair / Item 123 Ext 45 = brown chair. Two chairs can have the same item number, but one is brown. The other is white."
            case .barcodeSymbologyID: return "An identifier for a system of barcodes."
            case .physicalAttribute: return "A physical attribute of the item."
            case .measurementDimension: return "A measurable dimension (length, mass, weight, or volume) of the item."
            case .issuerParty: return "The party that issued this item identification."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .extendedID: result = extendedID?.search(_search)
                case .barcodeSymbologyID: result = barcodeSymbologyID?.search(_search)
                case .physicalAttribute: result = physicalAttribute.compactMap({ $0.search(_search) }).first
                case .measurementDimension: result = measurementDimension.compactMap({ $0.search(_search) }).first
                case .issuerParty: result = issuerParty?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .extendedID: presentableView(for: extendedID, title: _title, details: _details, show: show, search: _search)
                case .barcodeSymbologyID: presentableView(for: barcodeSymbologyID, title: _title, details: _details, show: show, search: _search)
                case .physicalAttribute: presentableView(for: physicalAttribute.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .measurementDimension: presentableView(for: measurementDimension.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .issuerParty: presentableView(for: issuerParty, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ItemIdentification: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id.search(string) { return result }
        if let result = extendedID?.search(string) { return result }
        if let result = barcodeSymbologyID?.search(string) { return result }
        if let result = physicalAttribute.compactMap({ $0.search(string)  }).first { return result }
        if let result = measurementDimension.compactMap({ $0.search(string)  }).first { return result }
        if let result = issuerParty?.search(string) { return result }
        return nil
    }
}
