// 
// ItemInstance.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ItemInstance: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .productTraceID,
        .manufactureDate,
        .manufactureTime,
        .bestBeforeDate,
        .registrationID,
        .serialID,
        .additionalItemProperty,
        .lotIdentification
]
    public var hasNilValue: Bool {(productTraceID == nil)
        || (manufactureDate == nil)
        || (manufactureTime == nil)
        || (bestBeforeDate == nil)
        || (registrationID == nil)
        || (serialID == nil)
        || (lotIdentification == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .productTraceID: return "Product Trace Identifier"
            case .manufactureDate: return "Manufacture Date"
            case .manufactureTime: return "Manufacture Time"
            case .bestBeforeDate: return "Best Before Date"
            case .registrationID: return "Registration Identifier"
            case .serialID: return "Serial Identifier"
            case .additionalItemProperty: return "Additional Item Property"
            case .lotIdentification: return "Lot Identification"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .productTraceID: return "An identifier used for tracing this item instance, such as the EPC number used in RFID."
            case .manufactureDate: return "The date on which this item instance was manufactured."
            case .manufactureTime: return "The time at which this item instance was manufactured."
            case .bestBeforeDate: return "The date before which it is best to use this item instance."
            case .registrationID: return "The registration identifier of this item instance."
            case .serialID: return "The serial number of this item instance."
            case .additionalItemProperty: return "An additional property of this item instance."
            case .lotIdentification: return "The lot identifier of this item instance (the identifier that allows recall of the item if necessary)."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .productTraceID: result = productTraceID?.search(_search)
                case .manufactureDate: result = manufactureDate?.search(_search)
                case .manufactureTime: result = manufactureTime?.search(_search)
                case .bestBeforeDate: result = bestBeforeDate?.search(_search)
                case .registrationID: result = registrationID?.search(_search)
                case .serialID: result = serialID?.search(_search)
                case .additionalItemProperty: result = additionalItemProperty.compactMap({ $0.search(_search) }).first
                case .lotIdentification: result = lotIdentification?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .productTraceID: presentableView(for: productTraceID, title: _title, details: _details, show: show, search: _search)
                case .manufactureDate: anyView(for: manufactureDate, title: _title, details: _details, show: show, search: _search)
                case .manufactureTime: anyView(for: manufactureTime, title: _title, details: _details, show: show, search: _search)
                case .bestBeforeDate: anyView(for: bestBeforeDate, title: _title, details: _details, show: show, search: _search)
                case .registrationID: presentableView(for: registrationID, title: _title, details: _details, show: show, search: _search)
                case .serialID: presentableView(for: serialID, title: _title, details: _details, show: show, search: _search)
                case .additionalItemProperty: presentableView(for: additionalItemProperty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .lotIdentification: presentableView(for: lotIdentification, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ItemInstance: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = manufactureDate?.search(string) { return result }
        if let result = manufactureTime?.search(string) { return result }
        if let result = bestBeforeDate?.search(string) { return result }
        if let result = productTraceID?.search(string) { return result }
        if let result = registrationID?.search(string) { return result }
        if let result = serialID?.search(string) { return result }
        if let result = additionalItemProperty.compactMap({ $0.search(string)  }).first { return result }
        if let result = lotIdentification?.search(string) { return result }
        return nil
    }
}
