// 
// ItemLocationQuantity.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ItemLocationQuantity: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .leadTimeMeasure,
        .minimumQuantity,
        .maximumQuantity,
        .hazardousRiskIndicator,
        .tradingRestrictions,
        .applicableTerritoryAddress,
        .price,
        .deliveryUnit,
        .applicableTaxCategory,
        .package,
        .allowanceCharge,
        .dependentPriceReference
]
    public var hasNilValue: Bool {(leadTimeMeasure == nil)
        || (minimumQuantity == nil)
        || (maximumQuantity == nil)
        || (hazardousRiskIndicator == nil)
        || (price == nil)
        || (package == nil)
        || (dependentPriceReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .leadTimeMeasure: return "Lead Time"
            case .minimumQuantity: return "Minimum Quantity"
            case .maximumQuantity: return "Maximum Quantity"
            case .hazardousRiskIndicator: return "Hazardous Risk Indicator"
            case .tradingRestrictions: return "Trading Restrictions"
            case .applicableTerritoryAddress: return "Applicable Territory Address"
            case .price: return "Price"
            case .deliveryUnit: return "Delivery Unit"
            case .applicableTaxCategory: return "Applicable Tax Category"
            case .package: return "Package"
            case .allowanceCharge: return "Allowance Charge"
            case .dependentPriceReference: return "Dependent Price Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .leadTimeMeasure: return "The lead time, i.e., the time taken from the time at which an item is ordered to the time of its delivery."
            case .minimumQuantity: return "The minimum quantity that can be ordered to qualify for a specific price."
            case .maximumQuantity: return "The maximum quantity that can be ordered to qualify for a specific price."
            case .hazardousRiskIndicator: return "An indication that the transported item, as delivered, in the stated quantity to the stated location, is subject to an international regulation concerning the carriage of dangerous goods (true) or not (false)."
            case .tradingRestrictions: return "Text describing trade restrictions on the quantity of this item or on the item itself."
            case .applicableTerritoryAddress: return "The applicable sales territory."
            case .price: return "The price associated with the given location."
            case .deliveryUnit: return "A delivery unit in which the item is located."
            case .applicableTaxCategory: return "A tax category applicable to this item location quantity."
            case .package: return "The package to which this price applies."
            case .allowanceCharge: return "An allowance or charge associated with this item location quantity."
            case .dependentPriceReference: return "The price of the item as a percentage of the price of some other item."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .leadTimeMeasure: result = leadTimeMeasure?.search(_search)
                case .minimumQuantity: result = minimumQuantity?.search(_search)
                case .maximumQuantity: result = maximumQuantity?.search(_search)
                case .hazardousRiskIndicator: result = hazardousRiskIndicator?.search(_search)
                case .tradingRestrictions: result = tradingRestrictions.compactMap({ $0.search(_search) }).first
                case .applicableTerritoryAddress: result = applicableTerritoryAddress.compactMap({ $0.search(_search) }).first
                case .price: result = price?.search(_search)
                case .deliveryUnit: result = deliveryUnit.compactMap({ $0.search(_search) }).first
                case .applicableTaxCategory: result = applicableTaxCategory.compactMap({ $0.search(_search) }).first
                case .package: result = package?.search(_search)
                case .allowanceCharge: result = allowanceCharge.compactMap({ $0.search(_search) }).first
                case .dependentPriceReference: result = dependentPriceReference?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .leadTimeMeasure: presentableView(for: leadTimeMeasure, title: _title, details: _details, show: show, search: _search)
                case .minimumQuantity: presentableView(for: minimumQuantity, title: _title, details: _details, show: show, search: _search)
                case .maximumQuantity: presentableView(for: maximumQuantity, title: _title, details: _details, show: show, search: _search)
                case .hazardousRiskIndicator: presentableView(for: hazardousRiskIndicator, title: _title, details: _details, show: show, search: _search)
                case .tradingRestrictions: presentableView(for: tradingRestrictions.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .applicableTerritoryAddress: presentableView(for: applicableTerritoryAddress.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .price: presentableView(for: price, title: _title, details: _details, show: show, search: _search)
                case .deliveryUnit: presentableView(for: deliveryUnit.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .applicableTaxCategory: presentableView(for: applicableTaxCategory.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .package: presentableView(for: package, title: _title, details: _details, show: show, search: _search)
                case .allowanceCharge: presentableView(for: allowanceCharge.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .dependentPriceReference: presentableView(for: dependentPriceReference, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ItemLocationQuantity: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = leadTimeMeasure?.search(string) { return result }
        if let result = minimumQuantity?.search(string) { return result }
        if let result = maximumQuantity?.search(string) { return result }
        if let result = hazardousRiskIndicator?.search(string) { return result }
        if let result = tradingRestrictions.compactMap({ $0.search(string)  }).first { return result }
        if let result = applicableTerritoryAddress.compactMap({ $0.search(string)  }).first { return result }
        if let result = price?.search(string) { return result }
        if let result = deliveryUnit.compactMap({ $0.search(string)  }).first { return result }
        if let result = applicableTaxCategory.compactMap({ $0.search(string)  }).first { return result }
        if let result = package?.search(string) { return result }
        if let result = allowanceCharge.compactMap({ $0.search(string)  }).first { return result }
        if let result = dependentPriceReference?.search(string) { return result }
        return nil
    }
}
