// 
// ItemProperty.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ItemProperty: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .name,
        .nameCode,
        .testMethod,
        .value,
        .valueQuantity,
        .valueQualifier,
        .importanceCode,
        .listValue,
        .usabilityPeriod,
        .itemPropertyGroup,
        .rangeDimension,
        .itemPropertyRange
]
    public var hasNilValue: Bool {(id == nil)
        || (nameCode == nil)
        || (testMethod == nil)
        || (value == nil)
        || (valueQuantity == nil)
        || (importanceCode == nil)
        || (usabilityPeriod == nil)
        || (rangeDimension == nil)
        || (itemPropertyRange == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .name: return "Name"
            case .nameCode: return "Name Code"
            case .testMethod: return "Test Method"
            case .value: return "Value"
            case .valueQuantity: return "Value Quantity"
            case .valueQualifier: return "Value Qualifier"
            case .importanceCode: return "Importance Code"
            case .listValue: return "List Value"
            case .usabilityPeriod: return "Usability Period"
            case .itemPropertyGroup: return "Item Property Group"
            case .rangeDimension: return "Range Dimension"
            case .itemPropertyRange: return "Item Property Range"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this property of an item."
            case .name: return "The name of this item property."
            case .nameCode: return "The name of this item property, expressed as a code."
            case .testMethod: return "The method of testing the value of this item property."
            case .value: return "The value of this item property, expressed as text."
            case .valueQuantity: return "The value of this item property, expressed as a quantity."
            case .valueQualifier: return "Text qualifying the value of the property."
            case .importanceCode: return "A code signifying the importance of this property in using it to describe a related Item."
            case .listValue: return "The value expressed as a text in case the property is a value in a list. For example, a colour."
            case .usabilityPeriod: return "The period during which this item property is valid."
            case .itemPropertyGroup: return "A description of the property group to which this item property belongs."
            case .rangeDimension: return "The range of values for the dimensions of this property."
            case .itemPropertyRange: return "A range of values for this item property."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .name: result = name.search(_search)
                case .nameCode: result = nameCode?.search(_search)
                case .testMethod: result = testMethod?.search(_search)
                case .value: result = value?.search(_search)
                case .valueQuantity: result = valueQuantity?.search(_search)
                case .valueQualifier: result = valueQualifier.compactMap({ $0.search(_search) }).first
                case .importanceCode: result = importanceCode?.search(_search)
                case .listValue: result = listValue.compactMap({ $0.search(_search) }).first
                case .usabilityPeriod: result = usabilityPeriod?.search(_search)
                case .itemPropertyGroup: result = itemPropertyGroup.compactMap({ $0.search(_search) }).first
                case .rangeDimension: result = rangeDimension?.search(_search)
                case .itemPropertyRange: result = itemPropertyRange?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .nameCode: presentableView(for: nameCode, title: _title, details: _details, show: show, search: _search)
                case .testMethod: presentableView(for: testMethod, title: _title, details: _details, show: show, search: _search)
                case .value: presentableView(for: value, title: _title, details: _details, show: show, search: _search)
                case .valueQuantity: presentableView(for: valueQuantity, title: _title, details: _details, show: show, search: _search)
                case .valueQualifier: presentableView(for: valueQualifier.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .importanceCode: presentableView(for: importanceCode, title: _title, details: _details, show: show, search: _search)
                case .listValue: presentableView(for: listValue.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .usabilityPeriod: presentableView(for: usabilityPeriod, title: _title, details: _details, show: show, search: _search)
                case .itemPropertyGroup: presentableView(for: itemPropertyGroup.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .rangeDimension: presentableView(for: rangeDimension, title: _title, details: _details, show: show, search: _search)
                case .itemPropertyRange: presentableView(for: itemPropertyRange, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ItemProperty: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = name.search(string) { return result }
        if let result = nameCode?.search(string) { return result }
        if let result = testMethod?.search(string) { return result }
        if let result = value?.search(string) { return result }
        if let result = valueQuantity?.search(string) { return result }
        if let result = valueQualifier.compactMap({ $0.search(string)  }).first { return result }
        if let result = importanceCode?.search(string) { return result }
        if let result = listValue.compactMap({ $0.search(string)  }).first { return result }
        if let result = usabilityPeriod?.search(string) { return result }
        if let result = itemPropertyGroup.compactMap({ $0.search(string)  }).first { return result }
        if let result = rangeDimension?.search(string) { return result }
        if let result = itemPropertyRange?.search(string) { return result }
        return nil
    }
}
