// 
// ItemPropertyRange.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ItemPropertyRange: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .minimumValue,
        .maximumValue
]
    public var hasNilValue: Bool {(minimumValue == nil)
        || (maximumValue == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .minimumValue: return "Minimum Value"
            case .maximumValue: return "Maximum Value"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .minimumValue: return "The minimum value in this range of values."
            case .maximumValue: return "The maximum value in this range of values."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .minimumValue: result = minimumValue?.search(_search)
                case .maximumValue: result = maximumValue?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .minimumValue: presentableView(for: minimumValue, title: _title, details: _details, show: show, search: _search)
                case .maximumValue: presentableView(for: maximumValue, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ItemPropertyRange: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = minimumValue?.search(string) { return result }
        if let result = maximumValue?.search(string) { return result }
        return nil
    }
}
