// 
// Language.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Language: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .name,
        .localeCode
]
    public var hasNilValue: Bool {(id == nil)
        || (name == nil)
        || (localeCode == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .name: return "Name"
            case .localeCode: return "Locale Code"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this language."
            case .name: return "The name of this language."
            case .localeCode: return "A code signifying the locale in which this language is used."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .name: result = name?.search(_search)
                case .localeCode: result = localeCode?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .localeCode: presentableView(for: localeCode, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Language: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = name?.search(string) { return result }
        if let result = localeCode?.search(string) { return result }
        return nil
    }
}
