// 
// LineReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.LineReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .lineID,
        .uuid,
        .lineStatusCode,
        .documentReference
]
    public var hasNilValue: Bool {(uuid == nil)
        || (lineStatusCode == nil)
        || (documentReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .lineID: return "Line Identifier"
            case .uuid: return "UUID"
            case .lineStatusCode: return "Line Status Code"
            case .documentReference: return "Document Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .lineID: return "Identifies the referenced line in the document."
            case .uuid: return "A universally unique identifier for this line reference."
            case .lineStatusCode: return "A code signifying the status of the referenced line with respect to its original state."
            case .documentReference: return "A reference to the document containing the referenced line."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .lineID: result = lineID.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .lineStatusCode: result = lineStatusCode?.search(_search)
                case .documentReference: result = documentReference?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .lineID: presentableView(for: lineID, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .lineStatusCode: presentableView(for: lineStatusCode, title: _title, details: _details, show: show, search: _search)
                case .documentReference: presentableView(for: documentReference, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.LineReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = lineID.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = lineStatusCode?.search(string) { return result }
        if let result = documentReference?.search(string) { return result }
        return nil
    }
}
