// 
// Location.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Location: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .description,
        .conditions,
        .countrySubentity,
        .countrySubentityCode,
        .locationTypeCode,
        .informationURI,
        .name,
        .validityPeriod,
        .address,
        .subsidiaryLocation,
        .locationCoordinate,
        .value,
        .languageID,
        .languageLocaleID
]
    public var hasNilValue: Bool {(id == nil)
        || (countrySubentity == nil)
        || (countrySubentityCode == nil)
        || (locationTypeCode == nil)
        || (informationURI == nil)
        || (name == nil)
        || (address == nil)
        || (languageID == nil)
        || (languageLocaleID == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .description: return "Description"
            case .conditions: return "Conditions"
            case .countrySubentity: return "Country Subentity"
            case .countrySubentityCode: return "Country Subentity Code"
            case .locationTypeCode: return "Location Type Code"
            case .informationURI: return "Information URI"
            case .name: return "Name"
            case .validityPeriod: return "Validity Period"
            case .address: return "Address"
            case .subsidiaryLocation: return "Subsidiary Location"
            case .locationCoordinate: return "Location Coordinate"
            case .value: return "Value"
            case .languageID: return "Language Id"
            case .languageLocaleID: return "Language Locale Id"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this location, e.g., the EAN Location Number, GLN."
            case .description: return "Text describing this location."
            case .conditions: return "Free-form text describing the physical conditions of the location."
            case .countrySubentity: return "A territorial division of a country, such as a county or state, expressed as text."
            case .countrySubentityCode: return "A territorial division of a country, such as a county or state, expressed as a code."
            case .locationTypeCode: return "A code signifying the type of location."
            case .informationURI: return "The Uniform Resource Identifier (URI) of a document providing information about this location."
            case .name: return "The name of this location."
            case .validityPeriod: return "A period during which this location can be used (e.g., for delivery)."
            case .address: return "The address of this location."
            case .subsidiaryLocation: return "A location subsidiary to this location."
            case .locationCoordinate: return "The geographical coordinates of this location."
            case .value: return nil
            case .languageID: return nil
            case .languageLocaleID: return nil
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
                case .conditions: result = conditions.compactMap({ $0.search(_search) }).first
                case .countrySubentity: result = countrySubentity?.search(_search)
                case .countrySubentityCode: result = countrySubentityCode?.search(_search)
                case .locationTypeCode: result = locationTypeCode?.search(_search)
                case .informationURI: result = informationURI?.search(_search)
                case .name: result = name?.search(_search)
                case .validityPeriod: result = validityPeriod.compactMap({ $0.search(_search) }).first
                case .address: result = address?.search(_search)
                case .subsidiaryLocation: result = subsidiaryLocation.compactMap({ $0.search(_search) }).first
                case .locationCoordinate: result = locationCoordinate.compactMap({ $0.search(_search) }).first
                case .value: result = value.search(_search)
                case .languageID: result = languageID?.search(_search)
                case .languageLocaleID: result = languageLocaleID?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .conditions: presentableView(for: conditions.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .countrySubentity: presentableView(for: countrySubentity, title: _title, details: _details, show: show, search: _search)
                case .countrySubentityCode: presentableView(for: countrySubentityCode, title: _title, details: _details, show: show, search: _search)
                case .locationTypeCode: presentableView(for: locationTypeCode, title: _title, details: _details, show: show, search: _search)
                case .informationURI: presentableView(for: informationURI, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .validityPeriod: presentableView(for: validityPeriod.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .address: presentableView(for: address, title: _title, details: _details, show: show, search: _search)
                case .subsidiaryLocation: presentableView(for: subsidiaryLocation.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .locationCoordinate: presentableView(for: locationCoordinate.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .value: anyView(for: value, title: _title, details: _details, show: show, search: _search)
                case .languageID: anyView(for: languageID, title: _title, details: _details, show: show, search: _search)
                case .languageLocaleID: anyView(for: languageLocaleID, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Location: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = value.search(string) { return result }
        if let result = languageID?.search(string) { return result }
        if let result = languageLocaleID?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        if let result = conditions.compactMap({ $0.search(string)  }).first { return result }
        if let result = countrySubentity?.search(string) { return result }
        if let result = countrySubentityCode?.search(string) { return result }
        if let result = locationTypeCode?.search(string) { return result }
        if let result = informationURI?.search(string) { return result }
        if let result = name?.search(string) { return result }
        if let result = validityPeriod.compactMap({ $0.search(string)  }).first { return result }
        if let result = address?.search(string) { return result }
        if let result = subsidiaryLocation.compactMap({ $0.search(string)  }).first { return result }
        if let result = locationCoordinate.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
