// 
// LocationCoordinate.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.LocationCoordinate: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .coordinateSystemCode,
        .latitudeDegreesMeasure,
        .latitudeMinutesMeasure,
        .latitudeDirectionCode,
        .longitudeDegreesMeasure,
        .longitudeMinutesMeasure,
        .longitudeDirectionCode,
        .altitudeMeasure
]
    public var hasNilValue: Bool {(coordinateSystemCode == nil)
        || (latitudeDegreesMeasure == nil)
        || (latitudeMinutesMeasure == nil)
        || (latitudeDirectionCode == nil)
        || (longitudeDegreesMeasure == nil)
        || (longitudeMinutesMeasure == nil)
        || (longitudeDirectionCode == nil)
        || (altitudeMeasure == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .coordinateSystemCode: return "Coordinate System Code"
            case .latitudeDegreesMeasure: return "Latitude Degrees"
            case .latitudeMinutesMeasure: return "Latitude Minutes"
            case .latitudeDirectionCode: return "Latitude Direction Code"
            case .longitudeDegreesMeasure: return "Longitude Degrees"
            case .longitudeMinutesMeasure: return "Longitude Minutes"
            case .longitudeDirectionCode: return "Longitude Direction Code"
            case .altitudeMeasure: return "Altitude"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .coordinateSystemCode: return "A code signifying the location system used."
            case .latitudeDegreesMeasure: return "The degree component of a latitude measured in degrees and minutes."
            case .latitudeMinutesMeasure: return "The minutes component of a latitude measured in degrees and minutes (modulo 60)."
            case .latitudeDirectionCode: return "A code signifying the direction of latitude measurement from the equator (north or south)."
            case .longitudeDegreesMeasure: return "The degree component of a longitude measured in degrees and minutes."
            case .longitudeMinutesMeasure: return "The minutes component of a longitude measured in degrees and minutes (modulo 60)."
            case .longitudeDirectionCode: return "A code signifying the direction of longitude measurement from the prime meridian (east or west)."
            case .altitudeMeasure: return "The altitude of the location."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .coordinateSystemCode: result = coordinateSystemCode?.search(_search)
                case .latitudeDegreesMeasure: result = latitudeDegreesMeasure?.search(_search)
                case .latitudeMinutesMeasure: result = latitudeMinutesMeasure?.search(_search)
                case .latitudeDirectionCode: result = latitudeDirectionCode?.search(_search)
                case .longitudeDegreesMeasure: result = longitudeDegreesMeasure?.search(_search)
                case .longitudeMinutesMeasure: result = longitudeMinutesMeasure?.search(_search)
                case .longitudeDirectionCode: result = longitudeDirectionCode?.search(_search)
                case .altitudeMeasure: result = altitudeMeasure?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .coordinateSystemCode: presentableView(for: coordinateSystemCode, title: _title, details: _details, show: show, search: _search)
                case .latitudeDegreesMeasure: presentableView(for: latitudeDegreesMeasure, title: _title, details: _details, show: show, search: _search)
                case .latitudeMinutesMeasure: presentableView(for: latitudeMinutesMeasure, title: _title, details: _details, show: show, search: _search)
                case .latitudeDirectionCode: presentableView(for: latitudeDirectionCode, title: _title, details: _details, show: show, search: _search)
                case .longitudeDegreesMeasure: presentableView(for: longitudeDegreesMeasure, title: _title, details: _details, show: show, search: _search)
                case .longitudeMinutesMeasure: presentableView(for: longitudeMinutesMeasure, title: _title, details: _details, show: show, search: _search)
                case .longitudeDirectionCode: presentableView(for: longitudeDirectionCode, title: _title, details: _details, show: show, search: _search)
                case .altitudeMeasure: presentableView(for: altitudeMeasure, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.LocationCoordinate: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = coordinateSystemCode?.search(string) { return result }
        if let result = latitudeDegreesMeasure?.search(string) { return result }
        if let result = latitudeMinutesMeasure?.search(string) { return result }
        if let result = latitudeDirectionCode?.search(string) { return result }
        if let result = longitudeDegreesMeasure?.search(string) { return result }
        if let result = longitudeMinutesMeasure?.search(string) { return result }
        if let result = longitudeDirectionCode?.search(string) { return result }
        if let result = altitudeMeasure?.search(string) { return result }
        return nil
    }
}
