// 
// LotIdentification.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.LotIdentification: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .lotNumberID,
        .expiryDate,
        .additionalItemProperty
]
    public var hasNilValue: Bool {(lotNumberID == nil)
        || (expiryDate == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .lotNumberID: return "Lot Number"
            case .expiryDate: return "Expiry Date"
            case .additionalItemProperty: return "Additional Item Property"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .lotNumberID: return "An identifier for the lot."
            case .expiryDate: return "The expiry date of the lot."
            case .additionalItemProperty: return "An additional property of the lot."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .lotNumberID: result = lotNumberID?.search(_search)
                case .expiryDate: result = expiryDate?.search(_search)
                case .additionalItemProperty: result = additionalItemProperty.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .lotNumberID: presentableView(for: lotNumberID, title: _title, details: _details, show: show, search: _search)
                case .expiryDate: anyView(for: expiryDate, title: _title, details: _details, show: show, search: _search)
                case .additionalItemProperty: presentableView(for: additionalItemProperty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.LotIdentification: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = expiryDate?.search(string) { return result }
        if let result = lotNumberID?.search(string) { return result }
        if let result = additionalItemProperty.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
