// 
// MaritimeTransport.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.MaritimeTransport: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .vesselID,
        .vesselName,
        .radioCallSignID,
        .shipsRequirements,
        .grossTonnageMeasure,
        .netTonnageMeasure,
        .registryCertificateDocumentReference,
        .registryPortLocation
]
    public var hasNilValue: Bool {(vesselID == nil)
        || (vesselName == nil)
        || (radioCallSignID == nil)
        || (grossTonnageMeasure == nil)
        || (netTonnageMeasure == nil)
        || (registryCertificateDocumentReference == nil)
        || (registryPortLocation == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .vesselID: return "Vessel Identifier"
            case .vesselName: return "Vessel Name"
            case .radioCallSignID: return "Radio Call Sign Identifier"
            case .shipsRequirements: return "Ships Requirements"
            case .grossTonnageMeasure: return "Gross Tonnage"
            case .netTonnageMeasure: return "Net Tonnage"
            case .registryCertificateDocumentReference: return "Registry Certificate Document Reference"
            case .registryPortLocation: return "Registry Port Location"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .vesselID: return "An identifier for a specific vessel."
            case .vesselName: return "The name of the vessel."
            case .radioCallSignID: return "The radio call sign of the vessel."
            case .shipsRequirements: return "Information about what services a vessel will require when it arrives at a port, such as refueling, maintenance, waste disposal etc."
            case .grossTonnageMeasure: return "Gross tonnage is calculated by measuring a ship's volume (from keel to funnel, to the outside of the hull framing) and applying a mathematical formula and is used to determine things such as a ship's manning regulations, safety rules, registration fees and port dues."
            case .netTonnageMeasure: return "Net tonnage is calculated by measuring a ship's internal volume and applying a mathematical formula and is used to calculate the port duties."
            case .registryCertificateDocumentReference: return "The certificate issued to the ship by the ships registry in a given flag state."
            case .registryPortLocation: return "The port in which a vessel is registered or permanently based."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .vesselID: result = vesselID?.search(_search)
                case .vesselName: result = vesselName?.search(_search)
                case .radioCallSignID: result = radioCallSignID?.search(_search)
                case .shipsRequirements: result = shipsRequirements.compactMap({ $0.search(_search) }).first
                case .grossTonnageMeasure: result = grossTonnageMeasure?.search(_search)
                case .netTonnageMeasure: result = netTonnageMeasure?.search(_search)
                case .registryCertificateDocumentReference: result = registryCertificateDocumentReference?.search(_search)
                case .registryPortLocation: result = registryPortLocation?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .vesselID: presentableView(for: vesselID, title: _title, details: _details, show: show, search: _search)
                case .vesselName: presentableView(for: vesselName, title: _title, details: _details, show: show, search: _search)
                case .radioCallSignID: presentableView(for: radioCallSignID, title: _title, details: _details, show: show, search: _search)
                case .shipsRequirements: presentableView(for: shipsRequirements.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .grossTonnageMeasure: presentableView(for: grossTonnageMeasure, title: _title, details: _details, show: show, search: _search)
                case .netTonnageMeasure: presentableView(for: netTonnageMeasure, title: _title, details: _details, show: show, search: _search)
                case .registryCertificateDocumentReference: presentableView(for: registryCertificateDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .registryPortLocation: presentableView(for: registryPortLocation, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.MaritimeTransport: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = vesselID?.search(string) { return result }
        if let result = vesselName?.search(string) { return result }
        if let result = radioCallSignID?.search(string) { return result }
        if let result = shipsRequirements.compactMap({ $0.search(string)  }).first { return result }
        if let result = grossTonnageMeasure?.search(string) { return result }
        if let result = netTonnageMeasure?.search(string) { return result }
        if let result = registryCertificateDocumentReference?.search(string) { return result }
        if let result = registryPortLocation?.search(string) { return result }
        return nil
    }
}
