// 
// MonetaryTotal.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.MonetaryTotal: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .lineExtensionAmount,
        .taxExclusiveAmount,
        .taxInclusiveAmount,
        .allowanceTotalAmount,
        .chargeTotalAmount,
        .prepaidAmount,
        .payableRoundingAmount,
        .payableAmount,
        .payableAlternativeAmount
]
    public var hasNilValue: Bool {(lineExtensionAmount == nil)
        || (taxExclusiveAmount == nil)
        || (taxInclusiveAmount == nil)
        || (allowanceTotalAmount == nil)
        || (chargeTotalAmount == nil)
        || (prepaidAmount == nil)
        || (payableRoundingAmount == nil)
        || (payableAlternativeAmount == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .lineExtensionAmount: return "Line Extension Amount"
            case .taxExclusiveAmount: return "Tax Exclusive Amount"
            case .taxInclusiveAmount: return "Tax Inclusive Amount"
            case .allowanceTotalAmount: return "Allowance Total Amount"
            case .chargeTotalAmount: return "Charge Total Amount"
            case .prepaidAmount: return "Prepaid Amount"
            case .payableRoundingAmount: return "Payable Rounding Amount"
            case .payableAmount: return "Payable Amount"
            case .payableAlternativeAmount: return "Payable Alternative Amount"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .lineExtensionAmount: return "The monetary amount of an extended transaction line, net of tax and settlement discounts, but inclusive of any applicable rounding amount."
            case .taxExclusiveAmount: return "The monetary amount of an extended transaction line, exclusive of taxes."
            case .taxInclusiveAmount: return "The monetary amount including taxes; the sum of payable amount and prepaid amount."
            case .allowanceTotalAmount: return "The total monetary amount of all allowances."
            case .chargeTotalAmount: return "The total monetary amount of all charges."
            case .prepaidAmount: return "The total prepaid monetary amount."
            case .payableRoundingAmount: return "The rounding amount (positive or negative) added to produce the line extension amount."
            case .payableAmount: return "The amount of the monetary total to be paid."
            case .payableAlternativeAmount: return "The amount of the monetary total to be paid, expressed in an alternative currency."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .lineExtensionAmount: result = lineExtensionAmount?.search(_search)
                case .taxExclusiveAmount: result = taxExclusiveAmount?.search(_search)
                case .taxInclusiveAmount: result = taxInclusiveAmount?.search(_search)
                case .allowanceTotalAmount: result = allowanceTotalAmount?.search(_search)
                case .chargeTotalAmount: result = chargeTotalAmount?.search(_search)
                case .prepaidAmount: result = prepaidAmount?.search(_search)
                case .payableRoundingAmount: result = payableRoundingAmount?.search(_search)
                case .payableAmount: result = payableAmount.search(_search)
                case .payableAlternativeAmount: result = payableAlternativeAmount?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .lineExtensionAmount: presentableView(for: lineExtensionAmount, title: _title, details: _details, show: show, search: _search)
                case .taxExclusiveAmount: presentableView(for: taxExclusiveAmount, title: _title, details: _details, show: show, search: _search)
                case .taxInclusiveAmount: presentableView(for: taxInclusiveAmount, title: _title, details: _details, show: show, search: _search)
                case .allowanceTotalAmount: presentableView(for: allowanceTotalAmount, title: _title, details: _details, show: show, search: _search)
                case .chargeTotalAmount: presentableView(for: chargeTotalAmount, title: _title, details: _details, show: show, search: _search)
                case .prepaidAmount: presentableView(for: prepaidAmount, title: _title, details: _details, show: show, search: _search)
                case .payableRoundingAmount: presentableView(for: payableRoundingAmount, title: _title, details: _details, show: show, search: _search)
                case .payableAmount: presentableView(for: payableAmount, title: _title, details: _details, show: show, search: _search)
                case .payableAlternativeAmount: presentableView(for: payableAlternativeAmount, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.MonetaryTotal: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = lineExtensionAmount?.search(string) { return result }
        if let result = taxExclusiveAmount?.search(string) { return result }
        if let result = taxInclusiveAmount?.search(string) { return result }
        if let result = allowanceTotalAmount?.search(string) { return result }
        if let result = chargeTotalAmount?.search(string) { return result }
        if let result = prepaidAmount?.search(string) { return result }
        if let result = payableRoundingAmount?.search(string) { return result }
        if let result = payableAmount.search(string) { return result }
        if let result = payableAlternativeAmount?.search(string) { return result }
        return nil
    }
}
