// 
// OrderLineReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.OrderLineReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .lineID,
        .salesOrderLineID,
        .uuid,
        .lineStatusCode,
        .orderReference
]
    public var hasNilValue: Bool {(salesOrderLineID == nil)
        || (uuid == nil)
        || (lineStatusCode == nil)
        || (orderReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .lineID: return "Line Identifier"
            case .salesOrderLineID: return "Sales Order Line Identifier"
            case .uuid: return "UUID"
            case .lineStatusCode: return "Line Status Code"
            case .orderReference: return "Order Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .lineID: return "An identifier for the referenced order line, assigned by the buyer."
            case .salesOrderLineID: return "An identifier for the referenced order line, assigned by the seller."
            case .uuid: return "A universally unique identifier for this order line reference."
            case .lineStatusCode: return "A code signifying the status of the referenced order line with respect to its original state."
            case .orderReference: return "A reference to the Order containing the referenced order line."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .lineID: result = lineID.search(_search)
                case .salesOrderLineID: result = salesOrderLineID?.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .lineStatusCode: result = lineStatusCode?.search(_search)
                case .orderReference: result = orderReference?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .lineID: presentableView(for: lineID, title: _title, details: _details, show: show, search: _search)
                case .salesOrderLineID: presentableView(for: salesOrderLineID, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .lineStatusCode: presentableView(for: lineStatusCode, title: _title, details: _details, show: show, search: _search)
                case .orderReference: presentableView(for: orderReference, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.OrderLineReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = lineID.search(string) { return result }
        if let result = salesOrderLineID?.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = lineStatusCode?.search(string) { return result }
        if let result = orderReference?.search(string) { return result }
        return nil
    }
}
