// 
// OrderReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.OrderReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .salesOrderID,
        .copyIndicator,
        .uuid,
        .issueDate,
        .issueTime,
        .customerReference,
        .orderTypeCode,
        .documentReference
]
    public var hasNilValue: Bool {(salesOrderID == nil)
        || (copyIndicator == nil)
        || (uuid == nil)
        || (issueDate == nil)
        || (issueTime == nil)
        || (customerReference == nil)
        || (orderTypeCode == nil)
        || (documentReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .salesOrderID: return "Sales Order Identifier"
            case .copyIndicator: return "Copy Indicator"
            case .uuid: return "UUID"
            case .issueDate: return "Issue Date"
            case .issueTime: return "Issue Time"
            case .customerReference: return "Customer Reference"
            case .orderTypeCode: return "Order Type Code"
            case .documentReference: return "Document Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this order reference, assigned by the buyer."
            case .salesOrderID: return "An identifier for this order reference, assigned by the seller."
            case .copyIndicator: return "Indicates whether the referenced Order is a copy (true) or the original (false)."
            case .uuid: return "A universally unique identifier for this order reference."
            case .issueDate: return "The date on which the referenced Order was issued."
            case .issueTime: return "The time at which the referenced Order was issued."
            case .customerReference: return "Text used for tagging purchasing card transactions."
            case .orderTypeCode: return "A code signifying the type of the referenced Order."
            case .documentReference: return "A document associated with this reference to an Order."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .salesOrderID: result = salesOrderID?.search(_search)
                case .copyIndicator: result = copyIndicator?.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .issueDate: result = issueDate?.search(_search)
                case .issueTime: result = issueTime?.search(_search)
                case .customerReference: result = customerReference?.search(_search)
                case .orderTypeCode: result = orderTypeCode?.search(_search)
                case .documentReference: result = documentReference?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .salesOrderID: presentableView(for: salesOrderID, title: _title, details: _details, show: show, search: _search)
                case .copyIndicator: presentableView(for: copyIndicator, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .issueDate: anyView(for: issueDate, title: _title, details: _details, show: show, search: _search)
                case .issueTime: anyView(for: issueTime, title: _title, details: _details, show: show, search: _search)
                case .customerReference: presentableView(for: customerReference, title: _title, details: _details, show: show, search: _search)
                case .orderTypeCode: presentableView(for: orderTypeCode, title: _title, details: _details, show: show, search: _search)
                case .documentReference: presentableView(for: documentReference, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.OrderReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = issueDate?.search(string) { return result }
        if let result = issueTime?.search(string) { return result }
        if let result = id.search(string) { return result }
        if let result = salesOrderID?.search(string) { return result }
        if let result = copyIndicator?.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = customerReference?.search(string) { return result }
        if let result = orderTypeCode?.search(string) { return result }
        if let result = documentReference?.search(string) { return result }
        return nil
    }
}
