// 
// PackageType.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PackageType: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .quantity,
        .returnableMaterialIndicator,
        .packageLevelCode,
        .packagingTypeCode,
        .packingMaterial,
        .traceID,
        .containedPackage,
        .containingTransportEquipment,
        .goodsItem,
        .measurementDimension,
        .deliveryUnit,
        .delivery,
        .pickup,
        .despatch
]
    public var hasNilValue: Bool {(id == nil)
        || (quantity == nil)
        || (returnableMaterialIndicator == nil)
        || (packageLevelCode == nil)
        || (packagingTypeCode == nil)
        || (traceID == nil)
        || (containingTransportEquipment == nil)
        || (delivery == nil)
        || (pickup == nil)
        || (despatch == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .quantity: return "Quantity"
            case .returnableMaterialIndicator: return "Returnable Material Indicator"
            case .packageLevelCode: return "Package Level Code"
            case .packagingTypeCode: return "Packaging Type Code"
            case .packingMaterial: return "Packing Material"
            case .traceID: return "Trace Identifier"
            case .containedPackage: return "Contained Package"
            case .containingTransportEquipment: return "Containing Transport Equipment"
            case .goodsItem: return "Goods Item"
            case .measurementDimension: return "Measurement Dimension"
            case .deliveryUnit: return "Delivery Unit"
            case .delivery: return "Delivery"
            case .pickup: return "Pickup"
            case .despatch: return "Despatch"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this package."
            case .quantity: return "The quantity of items contained in this package."
            case .returnableMaterialIndicator: return "An indicator that the packaging material is returnable (true) or not (false)."
            case .packageLevelCode: return "A code signifying a level of packaging."
            case .packagingTypeCode: return "A code signifying a type of packaging."
            case .packingMaterial: return "Text describing the packaging material."
            case .traceID: return "An identifier for use in tracing this package, such as the EPC number used in RFID."
            case .containedPackage: return "A package contained within this package."
            case .containingTransportEquipment: return "The piece of transport equipment containing this package."
            case .goodsItem: return "A goods item included in this package."
            case .measurementDimension: return "A measurable dimension (length, mass, weight, or volume) of this package."
            case .deliveryUnit: return "A delivery unit within this package."
            case .delivery: return "The delivery of this package."
            case .pickup: return "The pickup of this package."
            case .despatch: return "The despatch of this package."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .quantity: result = quantity?.search(_search)
                case .returnableMaterialIndicator: result = returnableMaterialIndicator?.search(_search)
                case .packageLevelCode: result = packageLevelCode?.search(_search)
                case .packagingTypeCode: result = packagingTypeCode?.search(_search)
                case .packingMaterial: result = packingMaterial.compactMap({ $0.search(_search) }).first
                case .traceID: result = traceID?.search(_search)
                case .containedPackage: result = containedPackage.compactMap({ $0.search(_search) }).first
                case .containingTransportEquipment: result = containingTransportEquipment?.search(_search)
                case .goodsItem: result = goodsItem.compactMap({ $0.search(_search) }).first
                case .measurementDimension: result = measurementDimension.compactMap({ $0.search(_search) }).first
                case .deliveryUnit: result = deliveryUnit.compactMap({ $0.search(_search) }).first
                case .delivery: result = delivery?.search(_search)
                case .pickup: result = pickup?.search(_search)
                case .despatch: result = despatch?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .quantity: presentableView(for: quantity, title: _title, details: _details, show: show, search: _search)
                case .returnableMaterialIndicator: presentableView(for: returnableMaterialIndicator, title: _title, details: _details, show: show, search: _search)
                case .packageLevelCode: presentableView(for: packageLevelCode, title: _title, details: _details, show: show, search: _search)
                case .packagingTypeCode: presentableView(for: packagingTypeCode, title: _title, details: _details, show: show, search: _search)
                case .packingMaterial: presentableView(for: packingMaterial.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .traceID: presentableView(for: traceID, title: _title, details: _details, show: show, search: _search)
                case .containedPackage: presentableView(for: containedPackage.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .containingTransportEquipment: presentableView(for: containingTransportEquipment, title: _title, details: _details, show: show, search: _search)
                case .goodsItem: presentableView(for: goodsItem.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .measurementDimension: presentableView(for: measurementDimension.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .deliveryUnit: presentableView(for: deliveryUnit.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .delivery: presentableView(for: delivery, title: _title, details: _details, show: show, search: _search)
                case .pickup: presentableView(for: pickup, title: _title, details: _details, show: show, search: _search)
                case .despatch: presentableView(for: despatch, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PackageType: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = quantity?.search(string) { return result }
        if let result = returnableMaterialIndicator?.search(string) { return result }
        if let result = packageLevelCode?.search(string) { return result }
        if let result = packagingTypeCode?.search(string) { return result }
        if let result = packingMaterial.compactMap({ $0.search(string)  }).first { return result }
        if let result = traceID?.search(string) { return result }
        if let result = containedPackage.compactMap({ $0.search(string)  }).first { return result }
        if let result = containingTransportEquipment?.search(string) { return result }
        if let result = goodsItem.compactMap({ $0.search(string)  }).first { return result }
        if let result = measurementDimension.compactMap({ $0.search(string)  }).first { return result }
        if let result = deliveryUnit.compactMap({ $0.search(string)  }).first { return result }
        if let result = delivery?.search(string) { return result }
        if let result = pickup?.search(string) { return result }
        if let result = despatch?.search(string) { return result }
        return nil
    }
}
