// 
// Party.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Party: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .markCareIndicator,
        .markAttentionIndicator,
        .websiteURI,
        .logoReferenceID,
        .endpointID,
        .industryClassificationCode,
        .partyIdentification,
        .partyName,
        .language,
        .postalAddress,
        .physicalLocation,
        .partyTaxScheme,
        .partyLegalEntity,
        .contact,
        .person,
        .agentParty,
        .serviceProviderParty,
        .powerOfAttorney,
        .financialAccount
]
    public var hasNilValue: Bool {(markCareIndicator == nil)
        || (markAttentionIndicator == nil)
        || (websiteURI == nil)
        || (logoReferenceID == nil)
        || (endpointID == nil)
        || (industryClassificationCode == nil)
        || (language == nil)
        || (postalAddress == nil)
        || (physicalLocation == nil)
        || (contact == nil)
        || (agentParty == nil)
        || (financialAccount == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .markCareIndicator: return "Mark Care Indicator"
            case .markAttentionIndicator: return "Mark Attention Indicator"
            case .websiteURI: return "Website URI"
            case .logoReferenceID: return "Logo Reference"
            case .endpointID: return "Endpoint Identifier"
            case .industryClassificationCode: return "Industry Classification Code"
            case .partyIdentification: return "Party Identification"
            case .partyName: return "Party Name"
            case .language: return "Language"
            case .postalAddress: return "Postal Address"
            case .physicalLocation: return "Physical Location"
            case .partyTaxScheme: return "Party Tax Scheme"
            case .partyLegalEntity: return "Party Legal Entity"
            case .contact: return "Contact"
            case .person: return "Person"
            case .agentParty: return "Agent Party"
            case .serviceProviderParty: return "Service Provider Party"
            case .powerOfAttorney: return "Power Of Attorney"
            case .financialAccount: return "Financial Account"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .markCareIndicator: return "An indicator that this party is \"care of\" (c/o) (true) or not (false)."
            case .markAttentionIndicator: return "An indicator that this party is \"for the attention of\" (FAO) (true) or not (false)."
            case .websiteURI: return "The Uniform Resource Identifier (URI) that identifies this party's web site; i.e., the web site's Uniform Resource Locator (URL)."
            case .logoReferenceID: return "An identifier for this party's logo."
            case .endpointID: return "An identifier for the end point of the routing service (e.g., EAN Location Number, GLN)."
            case .industryClassificationCode: return "This party's Industry Classification Code."
            case .partyIdentification: return "An identifier for this party."
            case .partyName: return "A name for this party."
            case .language: return "The language associated with this party."
            case .postalAddress: return "The party's postal address."
            case .physicalLocation: return "The physical location of this party."
            case .partyTaxScheme: return "A tax scheme applying to this party."
            case .partyLegalEntity: return "A description of this party as a legal entity."
            case .contact: return "The primary contact for this party."
            case .person: return "A person associated with this party."
            case .agentParty: return "A party who acts as an agent for this party."
            case .serviceProviderParty: return "A party providing a service to this party."
            case .powerOfAttorney: return "A power of attorney associated with this party."
            case .financialAccount: return "The financial account associated with this party."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .markCareIndicator: result = markCareIndicator?.search(_search)
                case .markAttentionIndicator: result = markAttentionIndicator?.search(_search)
                case .websiteURI: result = websiteURI?.search(_search)
                case .logoReferenceID: result = logoReferenceID?.search(_search)
                case .endpointID: result = endpointID?.search(_search)
                case .industryClassificationCode: result = industryClassificationCode?.search(_search)
                case .partyIdentification: result = partyIdentification.compactMap({ $0.search(_search) }).first
                case .partyName: result = partyName.compactMap({ $0.search(_search) }).first
                case .language: result = language?.search(_search)
                case .postalAddress: result = postalAddress?.search(_search)
                case .physicalLocation: result = physicalLocation?.search(_search)
                case .partyTaxScheme: result = partyTaxScheme.compactMap({ $0.search(_search) }).first
                case .partyLegalEntity: result = partyLegalEntity.compactMap({ $0.search(_search) }).first
                case .contact: result = contact?.search(_search)
                case .person: result = person.compactMap({ $0.search(_search) }).first
                case .agentParty: result = agentParty?.search(_search)
                case .serviceProviderParty: result = serviceProviderParty.compactMap({ $0.search(_search) }).first
                case .powerOfAttorney: result = powerOfAttorney.compactMap({ $0.search(_search) }).first
                case .financialAccount: result = financialAccount?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .markCareIndicator: presentableView(for: markCareIndicator, title: _title, details: _details, show: show, search: _search)
                case .markAttentionIndicator: presentableView(for: markAttentionIndicator, title: _title, details: _details, show: show, search: _search)
                case .websiteURI: presentableView(for: websiteURI, title: _title, details: _details, show: show, search: _search)
                case .logoReferenceID: presentableView(for: logoReferenceID, title: _title, details: _details, show: show, search: _search)
                case .endpointID: presentableView(for: endpointID, title: _title, details: _details, show: show, search: _search)
                case .industryClassificationCode: presentableView(for: industryClassificationCode, title: _title, details: _details, show: show, search: _search)
                case .partyIdentification: presentableView(for: partyIdentification.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .partyName: presentableView(for: partyName.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .language: presentableView(for: language, title: _title, details: _details, show: show, search: _search)
                case .postalAddress: presentableView(for: postalAddress, title: _title, details: _details, show: show, search: _search)
                case .physicalLocation: presentableView(for: physicalLocation, title: _title, details: _details, show: show, search: _search)
                case .partyTaxScheme: presentableView(for: partyTaxScheme.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .partyLegalEntity: presentableView(for: partyLegalEntity.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .contact: presentableView(for: contact, title: _title, details: _details, show: show, search: _search)
                case .person: presentableView(for: person.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .agentParty: presentableView(for: agentParty, title: _title, details: _details, show: show, search: _search)
                case .serviceProviderParty: presentableView(for: serviceProviderParty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .powerOfAttorney: presentableView(for: powerOfAttorney.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .financialAccount: presentableView(for: financialAccount, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Party: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = markCareIndicator?.search(string) { return result }
        if let result = markAttentionIndicator?.search(string) { return result }
        if let result = websiteURI?.search(string) { return result }
        if let result = logoReferenceID?.search(string) { return result }
        if let result = endpointID?.search(string) { return result }
        if let result = industryClassificationCode?.search(string) { return result }
        if let result = partyIdentification.compactMap({ $0.search(string)  }).first { return result }
        if let result = partyName.compactMap({ $0.search(string)  }).first { return result }
        if let result = language?.search(string) { return result }
        if let result = postalAddress?.search(string) { return result }
        if let result = physicalLocation?.search(string) { return result }
        if let result = partyTaxScheme.compactMap({ $0.search(string)  }).first { return result }
        if let result = partyLegalEntity.compactMap({ $0.search(string)  }).first { return result }
        if let result = contact?.search(string) { return result }
        if let result = person.compactMap({ $0.search(string)  }).first { return result }
        if let result = agentParty?.search(string) { return result }
        if let result = serviceProviderParty.compactMap({ $0.search(string)  }).first { return result }
        if let result = powerOfAttorney.compactMap({ $0.search(string)  }).first { return result }
        if let result = financialAccount?.search(string) { return result }
        return nil
    }
}
