// 
// PartyLegalEntity.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PartyLegalEntity: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .registrationName,
        .companyID,
        .registrationDate,
        .registrationExpirationDate,
        .companyLegalFormCode,
        .companyLegalForm,
        .soleProprietorshipIndicator,
        .companyLiquidationStatusCode,
        .corporateStockAmount,
        .fullyPaidSharesIndicator,
        .registrationAddress,
        .corporateRegistrationScheme,
        .headOfficeParty,
        .shareholderParty
]
    public var hasNilValue: Bool {(registrationName == nil)
        || (companyID == nil)
        || (registrationDate == nil)
        || (registrationExpirationDate == nil)
        || (companyLegalFormCode == nil)
        || (companyLegalForm == nil)
        || (soleProprietorshipIndicator == nil)
        || (companyLiquidationStatusCode == nil)
        || (corporateStockAmount == nil)
        || (fullyPaidSharesIndicator == nil)
        || (registrationAddress == nil)
        || (corporateRegistrationScheme == nil)
        || (headOfficeParty == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .registrationName: return "Registration Name"
            case .companyID: return "Company Identifier"
            case .registrationDate: return "Registration Date"
            case .registrationExpirationDate: return "Registration Expiration Date"
            case .companyLegalFormCode: return "Company Legal Form Code"
            case .companyLegalForm: return "Company Legal Form"
            case .soleProprietorshipIndicator: return "Sole Proprietorship Indicator"
            case .companyLiquidationStatusCode: return "Company Liquidation Status Code"
            case .corporateStockAmount: return "Corporate Stock Amount"
            case .fullyPaidSharesIndicator: return "Fully Paid Shares Indicator"
            case .registrationAddress: return "Registration Address"
            case .corporateRegistrationScheme: return "Corporate Registration Scheme"
            case .headOfficeParty: return "Head Office Party"
            case .shareholderParty: return "Shareholder Party"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .registrationName: return "The name of the party as registered with the relevant legal authority."
            case .companyID: return "An identifier for the party as registered within a company registration scheme."
            case .registrationDate: return "The registration date of the CompanyID."
            case .registrationExpirationDate: return "The date upon which a registration expires (e.g., registration for an import/export license)."
            case .companyLegalFormCode: return "A code signifying the party's legal status."
            case .companyLegalForm: return "The company legal status, expressed as a text."
            case .soleProprietorshipIndicator: return "An indicator that the company is owned and controlled by one person (true) or not (false)."
            case .companyLiquidationStatusCode: return "A code signifying the party's liquidation status."
            case .corporateStockAmount: return "The number of shares in the capital stock of a corporation."
            case .fullyPaidSharesIndicator: return "An indicator that all shares of corporate stock have been paid by shareholders (true) or not (false)."
            case .registrationAddress: return "The registered address of the party within a corporate registration scheme."
            case .corporateRegistrationScheme: return "The corporate registration scheme used to register the party."
            case .headOfficeParty: return "The head office of the legal entity"
            case .shareholderParty: return "A party owning shares in this legal entity."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .registrationName: result = registrationName?.search(_search)
                case .companyID: result = companyID?.search(_search)
                case .registrationDate: result = registrationDate?.search(_search)
                case .registrationExpirationDate: result = registrationExpirationDate?.search(_search)
                case .companyLegalFormCode: result = companyLegalFormCode?.search(_search)
                case .companyLegalForm: result = companyLegalForm?.search(_search)
                case .soleProprietorshipIndicator: result = soleProprietorshipIndicator?.search(_search)
                case .companyLiquidationStatusCode: result = companyLiquidationStatusCode?.search(_search)
                case .corporateStockAmount: result = corporateStockAmount?.search(_search)
                case .fullyPaidSharesIndicator: result = fullyPaidSharesIndicator?.search(_search)
                case .registrationAddress: result = registrationAddress?.search(_search)
                case .corporateRegistrationScheme: result = corporateRegistrationScheme?.search(_search)
                case .headOfficeParty: result = headOfficeParty?.search(_search)
                case .shareholderParty: result = shareholderParty.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .registrationName: presentableView(for: registrationName, title: _title, details: _details, show: show, search: _search)
                case .companyID: presentableView(for: companyID, title: _title, details: _details, show: show, search: _search)
                case .registrationDate: anyView(for: registrationDate, title: _title, details: _details, show: show, search: _search)
                case .registrationExpirationDate: anyView(for: registrationExpirationDate, title: _title, details: _details, show: show, search: _search)
                case .companyLegalFormCode: presentableView(for: companyLegalFormCode, title: _title, details: _details, show: show, search: _search)
                case .companyLegalForm: presentableView(for: companyLegalForm, title: _title, details: _details, show: show, search: _search)
                case .soleProprietorshipIndicator: presentableView(for: soleProprietorshipIndicator, title: _title, details: _details, show: show, search: _search)
                case .companyLiquidationStatusCode: presentableView(for: companyLiquidationStatusCode, title: _title, details: _details, show: show, search: _search)
                case .corporateStockAmount: presentableView(for: corporateStockAmount, title: _title, details: _details, show: show, search: _search)
                case .fullyPaidSharesIndicator: presentableView(for: fullyPaidSharesIndicator, title: _title, details: _details, show: show, search: _search)
                case .registrationAddress: presentableView(for: registrationAddress, title: _title, details: _details, show: show, search: _search)
                case .corporateRegistrationScheme: presentableView(for: corporateRegistrationScheme, title: _title, details: _details, show: show, search: _search)
                case .headOfficeParty: presentableView(for: headOfficeParty, title: _title, details: _details, show: show, search: _search)
                case .shareholderParty: presentableView(for: shareholderParty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PartyLegalEntity: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = registrationDate?.search(string) { return result }
        if let result = registrationExpirationDate?.search(string) { return result }
        if let result = registrationName?.search(string) { return result }
        if let result = companyID?.search(string) { return result }
        if let result = companyLegalFormCode?.search(string) { return result }
        if let result = companyLegalForm?.search(string) { return result }
        if let result = soleProprietorshipIndicator?.search(string) { return result }
        if let result = companyLiquidationStatusCode?.search(string) { return result }
        if let result = corporateStockAmount?.search(string) { return result }
        if let result = fullyPaidSharesIndicator?.search(string) { return result }
        if let result = registrationAddress?.search(string) { return result }
        if let result = corporateRegistrationScheme?.search(string) { return result }
        if let result = headOfficeParty?.search(string) { return result }
        if let result = shareholderParty.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
