// 
// PartyTaxScheme.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PartyTaxScheme: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .registrationName,
        .companyID,
        .taxLevelCode,
        .exemptionReasonCode,
        .exemptionReason,
        .registrationAddress,
        .taxScheme
]
    public var hasNilValue: Bool {(registrationName == nil)
        || (companyID == nil)
        || (taxLevelCode == nil)
        || (exemptionReasonCode == nil)
        || (registrationAddress == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .registrationName: return "Registration Name"
            case .companyID: return "Company Identifier"
            case .taxLevelCode: return "Tax Level Code"
            case .exemptionReasonCode: return "Exemption Reason Code"
            case .exemptionReason: return "Exemption Reason"
            case .registrationAddress: return "Registration Address"
            case .taxScheme: return "Tax Scheme"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .registrationName: return "The name of the party as registered with the relevant fiscal authority."
            case .companyID: return "An identifier for the party assigned for tax purposes by the taxation authority."
            case .taxLevelCode: return "A code signifying the tax level applicable to the party within this taxation scheme."
            case .exemptionReasonCode: return "A reason for the party's exemption from tax, expressed as a code."
            case .exemptionReason: return "A reason for the party's exemption from tax, expressed as text."
            case .registrationAddress: return "The address of the party as registered for tax purposes."
            case .taxScheme: return "The taxation scheme applicable to the party."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .registrationName: result = registrationName?.search(_search)
                case .companyID: result = companyID?.search(_search)
                case .taxLevelCode: result = taxLevelCode?.search(_search)
                case .exemptionReasonCode: result = exemptionReasonCode?.search(_search)
                case .exemptionReason: result = exemptionReason.compactMap({ $0.search(_search) }).first
                case .registrationAddress: result = registrationAddress?.search(_search)
                case .taxScheme: result = taxScheme.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .registrationName: presentableView(for: registrationName, title: _title, details: _details, show: show, search: _search)
                case .companyID: presentableView(for: companyID, title: _title, details: _details, show: show, search: _search)
                case .taxLevelCode: presentableView(for: taxLevelCode, title: _title, details: _details, show: show, search: _search)
                case .exemptionReasonCode: presentableView(for: exemptionReasonCode, title: _title, details: _details, show: show, search: _search)
                case .exemptionReason: presentableView(for: exemptionReason.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .registrationAddress: presentableView(for: registrationAddress, title: _title, details: _details, show: show, search: _search)
                case .taxScheme: presentableView(for: taxScheme, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PartyTaxScheme: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = registrationName?.search(string) { return result }
        if let result = companyID?.search(string) { return result }
        if let result = taxLevelCode?.search(string) { return result }
        if let result = exemptionReasonCode?.search(string) { return result }
        if let result = exemptionReason.compactMap({ $0.search(string)  }).first { return result }
        if let result = registrationAddress?.search(string) { return result }
        if let result = taxScheme.search(string) { return result }
        return nil
    }
}
