// 
// Payment.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Payment: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .paidAmount,
        .receivedDate,
        .paidDate,
        .paidTime,
        .instructionID
]
    public var hasNilValue: Bool {(id == nil)
        || (paidAmount == nil)
        || (receivedDate == nil)
        || (paidDate == nil)
        || (paidTime == nil)
        || (instructionID == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .paidAmount: return "Paid Amount"
            case .receivedDate: return "Received Date"
            case .paidDate: return "Paid Date"
            case .paidTime: return "Paid Time"
            case .instructionID: return "Instruction Identifier"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this payment."
            case .paidAmount: return "The amount of this payment."
            case .receivedDate: return "The date on which this payment was received."
            case .paidDate: return "The date on which this payment was made."
            case .paidTime: return "The time at which this payment was made."
            case .instructionID: return "An identifier for the payment instruction."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .paidAmount: result = paidAmount?.search(_search)
                case .receivedDate: result = receivedDate?.search(_search)
                case .paidDate: result = paidDate?.search(_search)
                case .paidTime: result = paidTime?.search(_search)
                case .instructionID: result = instructionID?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .paidAmount: presentableView(for: paidAmount, title: _title, details: _details, show: show, search: _search)
                case .receivedDate: anyView(for: receivedDate, title: _title, details: _details, show: show, search: _search)
                case .paidDate: anyView(for: paidDate, title: _title, details: _details, show: show, search: _search)
                case .paidTime: anyView(for: paidTime, title: _title, details: _details, show: show, search: _search)
                case .instructionID: presentableView(for: instructionID, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Payment: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = receivedDate?.search(string) { return result }
        if let result = paidDate?.search(string) { return result }
        if let result = paidTime?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = paidAmount?.search(string) { return result }
        if let result = instructionID?.search(string) { return result }
        return nil
    }
}
