// 
// PaymentMandate.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PaymentMandate: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .mandateTypeCode,
        .maximumPaymentInstructionsNumeric,
        .maximumPaidAmount,
        .signatureID,
        .payerParty,
        .payerFinancialAccount,
        .validityPeriod,
        .paymentReversalPeriod,
        .clause
]
    public var hasNilValue: Bool {(id == nil)
        || (mandateTypeCode == nil)
        || (maximumPaymentInstructionsNumeric == nil)
        || (maximumPaidAmount == nil)
        || (signatureID == nil)
        || (payerParty == nil)
        || (payerFinancialAccount == nil)
        || (validityPeriod == nil)
        || (paymentReversalPeriod == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .mandateTypeCode: return "Mandate Type Code"
            case .maximumPaymentInstructionsNumeric: return "Maximum Payment Instructions"
            case .maximumPaidAmount: return "Maximum Paid Amount"
            case .signatureID: return "Signature Identifier"
            case .payerParty: return "Payer Party"
            case .payerFinancialAccount: return "Payer Financial Account"
            case .validityPeriod: return "Validity Period"
            case .paymentReversalPeriod: return "Payment Reversal Period"
            case .clause: return "Clause"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this payment mandate."
            case .mandateTypeCode: return "A code signifying the type of this payment mandate."
            case .maximumPaymentInstructionsNumeric: return "The number of maximum payment instructions allowed within the validity period."
            case .maximumPaidAmount: return "The maximum amount to be paid within a single instruction."
            case .signatureID: return "An identifier for a signature applied by a signatory party."
            case .payerParty: return "The payer party (if different from the debtor)."
            case .payerFinancialAccount: return "The payer's financial account."
            case .validityPeriod: return "The period during which this mandate is valid."
            case .paymentReversalPeriod: return "The period of the reverse payment."
            case .clause: return "A clause applicable to this payment mandate."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .mandateTypeCode: result = mandateTypeCode?.search(_search)
                case .maximumPaymentInstructionsNumeric: result = maximumPaymentInstructionsNumeric?.search(_search)
                case .maximumPaidAmount: result = maximumPaidAmount?.search(_search)
                case .signatureID: result = signatureID?.search(_search)
                case .payerParty: result = payerParty?.search(_search)
                case .payerFinancialAccount: result = payerFinancialAccount?.search(_search)
                case .validityPeriod: result = validityPeriod?.search(_search)
                case .paymentReversalPeriod: result = paymentReversalPeriod?.search(_search)
                case .clause: result = clause.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .mandateTypeCode: presentableView(for: mandateTypeCode, title: _title, details: _details, show: show, search: _search)
                case .maximumPaymentInstructionsNumeric: presentableView(for: maximumPaymentInstructionsNumeric, title: _title, details: _details, show: show, search: _search)
                case .maximumPaidAmount: presentableView(for: maximumPaidAmount, title: _title, details: _details, show: show, search: _search)
                case .signatureID: presentableView(for: signatureID, title: _title, details: _details, show: show, search: _search)
                case .payerParty: presentableView(for: payerParty, title: _title, details: _details, show: show, search: _search)
                case .payerFinancialAccount: presentableView(for: payerFinancialAccount, title: _title, details: _details, show: show, search: _search)
                case .validityPeriod: presentableView(for: validityPeriod, title: _title, details: _details, show: show, search: _search)
                case .paymentReversalPeriod: presentableView(for: paymentReversalPeriod, title: _title, details: _details, show: show, search: _search)
                case .clause: presentableView(for: clause.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PaymentMandate: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = mandateTypeCode?.search(string) { return result }
        if let result = maximumPaymentInstructionsNumeric?.search(string) { return result }
        if let result = maximumPaidAmount?.search(string) { return result }
        if let result = signatureID?.search(string) { return result }
        if let result = payerParty?.search(string) { return result }
        if let result = payerFinancialAccount?.search(string) { return result }
        if let result = validityPeriod?.search(string) { return result }
        if let result = paymentReversalPeriod?.search(string) { return result }
        if let result = clause.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
