// 
// PaymentMeans.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PaymentMeans: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .paymentMeansCode,
        .paymentDueDate,
        .paymentChannelCode,
        .instructionID,
        .instructionNote,
        .paymentID,
        .cardAccount,
        .payerFinancialAccount,
        .payeeFinancialAccount,
        .creditAccount,
        .paymentMandate,
        .tradeFinancing
]
    public var hasNilValue: Bool {(id == nil)
        || (paymentDueDate == nil)
        || (paymentChannelCode == nil)
        || (instructionID == nil)
        || (cardAccount == nil)
        || (payerFinancialAccount == nil)
        || (payeeFinancialAccount == nil)
        || (creditAccount == nil)
        || (paymentMandate == nil)
        || (tradeFinancing == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .paymentMeansCode: return "Payment Means Code"
            case .paymentDueDate: return "Payment Due Date"
            case .paymentChannelCode: return "Payment Channel Code"
            case .instructionID: return "Instruction Identifier"
            case .instructionNote: return "Instruction Note"
            case .paymentID: return "Payment Identifier"
            case .cardAccount: return "Card Account"
            case .payerFinancialAccount: return "Payer Financial Account"
            case .payeeFinancialAccount: return "Payee Financial Account"
            case .creditAccount: return "Credit Account"
            case .paymentMandate: return "Payment Mandate"
            case .tradeFinancing: return "Trade Financing"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this means of payment."
            case .paymentMeansCode: return "A code signifying the type of this means of payment."
            case .paymentDueDate: return "The date on which payment is due for this means of payment."
            case .paymentChannelCode: return "A code signifying the payment channel for this means of payment."
            case .instructionID: return "An identifier for the payment instruction."
            case .instructionNote: return "Free-form text conveying information that is not contained explicitly in other structures."
            case .paymentID: return "An identifier for a payment made using this means of payment."
            case .cardAccount: return "A credit card, debit card, or charge card account that constitutes this means of payment."
            case .payerFinancialAccount: return "The payer's financial account."
            case .payeeFinancialAccount: return "The payee's financial account."
            case .creditAccount: return "A credit account associated with this means of payment."
            case .paymentMandate: return "The payment mandate associated with this means of payment."
            case .tradeFinancing: return "A trade finance agreement applicable to this means of payment."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .paymentMeansCode: result = paymentMeansCode.search(_search)
                case .paymentDueDate: result = paymentDueDate?.search(_search)
                case .paymentChannelCode: result = paymentChannelCode?.search(_search)
                case .instructionID: result = instructionID?.search(_search)
                case .instructionNote: result = instructionNote.compactMap({ $0.search(_search) }).first
                case .paymentID: result = paymentID.compactMap({ $0.search(_search) }).first
                case .cardAccount: result = cardAccount?.search(_search)
                case .payerFinancialAccount: result = payerFinancialAccount?.search(_search)
                case .payeeFinancialAccount: result = payeeFinancialAccount?.search(_search)
                case .creditAccount: result = creditAccount?.search(_search)
                case .paymentMandate: result = paymentMandate?.search(_search)
                case .tradeFinancing: result = tradeFinancing?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .paymentMeansCode: presentableView(for: paymentMeansCode, title: _title, details: _details, show: show, search: _search)
                case .paymentDueDate: anyView(for: paymentDueDate, title: _title, details: _details, show: show, search: _search)
                case .paymentChannelCode: presentableView(for: paymentChannelCode, title: _title, details: _details, show: show, search: _search)
                case .instructionID: presentableView(for: instructionID, title: _title, details: _details, show: show, search: _search)
                case .instructionNote: presentableView(for: instructionNote.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .paymentID: presentableView(for: paymentID.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .cardAccount: presentableView(for: cardAccount, title: _title, details: _details, show: show, search: _search)
                case .payerFinancialAccount: presentableView(for: payerFinancialAccount, title: _title, details: _details, show: show, search: _search)
                case .payeeFinancialAccount: presentableView(for: payeeFinancialAccount, title: _title, details: _details, show: show, search: _search)
                case .creditAccount: presentableView(for: creditAccount, title: _title, details: _details, show: show, search: _search)
                case .paymentMandate: presentableView(for: paymentMandate, title: _title, details: _details, show: show, search: _search)
                case .tradeFinancing: presentableView(for: tradeFinancing, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PaymentMeans: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = paymentDueDate?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = paymentMeansCode.search(string) { return result }
        if let result = paymentChannelCode?.search(string) { return result }
        if let result = instructionID?.search(string) { return result }
        if let result = instructionNote.compactMap({ $0.search(string)  }).first { return result }
        if let result = paymentID.compactMap({ $0.search(string)  }).first { return result }
        if let result = cardAccount?.search(string) { return result }
        if let result = payerFinancialAccount?.search(string) { return result }
        if let result = payeeFinancialAccount?.search(string) { return result }
        if let result = creditAccount?.search(string) { return result }
        if let result = paymentMandate?.search(string) { return result }
        if let result = tradeFinancing?.search(string) { return result }
        return nil
    }
}
