// 
// PaymentTerms.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PaymentTerms: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .paymentMeansID,
        .prepaidPaymentReferenceID,
        .note,
        .referenceEventCode,
        .settlementDiscountPercent,
        .penaltySurchargePercent,
        .paymentPercent,
        .amount,
        .settlementDiscountAmount,
        .penaltyAmount,
        .paymentTermsDetailsURI,
        .paymentDueDate,
        .installmentDueDate,
        .invoicingPartyReference,
        .settlementPeriod,
        .penaltyPeriod,
        .exchangeRate,
        .validityPeriod
]
    public var hasNilValue: Bool {(id == nil)
        || (prepaidPaymentReferenceID == nil)
        || (referenceEventCode == nil)
        || (settlementDiscountPercent == nil)
        || (penaltySurchargePercent == nil)
        || (paymentPercent == nil)
        || (amount == nil)
        || (settlementDiscountAmount == nil)
        || (penaltyAmount == nil)
        || (paymentTermsDetailsURI == nil)
        || (paymentDueDate == nil)
        || (installmentDueDate == nil)
        || (invoicingPartyReference == nil)
        || (settlementPeriod == nil)
        || (penaltyPeriod == nil)
        || (exchangeRate == nil)
        || (validityPeriod == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .paymentMeansID: return "Payment Means Identifier"
            case .prepaidPaymentReferenceID: return "Prepaid Payment Reference Identifier"
            case .note: return "Note"
            case .referenceEventCode: return "Reference Event Code"
            case .settlementDiscountPercent: return "Settlement Discount Percent"
            case .penaltySurchargePercent: return "Penalty Surcharge Percent"
            case .paymentPercent: return "Payment Percent"
            case .amount: return "Amount"
            case .settlementDiscountAmount: return "Settlement Discount Amount"
            case .penaltyAmount: return "Penalty Amount"
            case .paymentTermsDetailsURI: return "Payment Terms Details URI"
            case .paymentDueDate: return "Payment Due Date"
            case .installmentDueDate: return "Installment Due Date"
            case .invoicingPartyReference: return "Invoicing Party Reference"
            case .settlementPeriod: return "Settlement Period"
            case .penaltyPeriod: return "Penalty Period"
            case .exchangeRate: return "Exchange Rate"
            case .validityPeriod: return "Validity Period"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this set of payment terms."
            case .paymentMeansID: return "An identifier for a means of payment associated with these payment terms."
            case .prepaidPaymentReferenceID: return "An identifier for a reference to a prepaid payment."
            case .note: return "Free-form text conveying information that is not contained explicitly in other structures."
            case .referenceEventCode: return "A code signifying the event during which these terms are offered."
            case .settlementDiscountPercent: return "The percentage for the settlement discount that is offered for payment under these payment terms."
            case .penaltySurchargePercent: return "The penalty for payment after the settlement period, expressed as a percentage of the payment."
            case .paymentPercent: return "The part of a payment, expressed as a percent, relevant for these payment terms."
            case .amount: return "The monetary amount covered by these payment terms."
            case .settlementDiscountAmount: return "The amount of a settlement discount offered for payment under these payment terms."
            case .penaltyAmount: return "The monetary amount of the penalty for payment after the settlement period."
            case .paymentTermsDetailsURI: return "The Uniform Resource Identifier (URI) of a document providing additional details regarding these payment terms."
            case .paymentDueDate: return "The due date for these payment terms."
            case .installmentDueDate: return "The due date for an installment payment for these payment terms."
            case .invoicingPartyReference: return "A reference to the payment terms used by the invoicing party. This may have been requested of the payer by the payee to accompany its remittance."
            case .settlementPeriod: return "The period during which settlement may occur."
            case .penaltyPeriod: return "The period during which penalties may apply."
            case .exchangeRate: return "The currency exchange rate for purposes of these payment terms."
            case .validityPeriod: return "The period during which these payment terms are valid."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .paymentMeansID: result = paymentMeansID.compactMap({ $0.search(_search) }).first
                case .prepaidPaymentReferenceID: result = prepaidPaymentReferenceID?.search(_search)
                case .note: result = note.compactMap({ $0.search(_search) }).first
                case .referenceEventCode: result = referenceEventCode?.search(_search)
                case .settlementDiscountPercent: result = settlementDiscountPercent?.search(_search)
                case .penaltySurchargePercent: result = penaltySurchargePercent?.search(_search)
                case .paymentPercent: result = paymentPercent?.search(_search)
                case .amount: result = amount?.search(_search)
                case .settlementDiscountAmount: result = settlementDiscountAmount?.search(_search)
                case .penaltyAmount: result = penaltyAmount?.search(_search)
                case .paymentTermsDetailsURI: result = paymentTermsDetailsURI?.search(_search)
                case .paymentDueDate: result = paymentDueDate?.search(_search)
                case .installmentDueDate: result = installmentDueDate?.search(_search)
                case .invoicingPartyReference: result = invoicingPartyReference?.search(_search)
                case .settlementPeriod: result = settlementPeriod?.search(_search)
                case .penaltyPeriod: result = penaltyPeriod?.search(_search)
                case .exchangeRate: result = exchangeRate?.search(_search)
                case .validityPeriod: result = validityPeriod?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .paymentMeansID: presentableView(for: paymentMeansID.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .prepaidPaymentReferenceID: presentableView(for: prepaidPaymentReferenceID, title: _title, details: _details, show: show, search: _search)
                case .note: presentableView(for: note.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .referenceEventCode: presentableView(for: referenceEventCode, title: _title, details: _details, show: show, search: _search)
                case .settlementDiscountPercent: presentableView(for: settlementDiscountPercent, title: _title, details: _details, show: show, search: _search)
                case .penaltySurchargePercent: presentableView(for: penaltySurchargePercent, title: _title, details: _details, show: show, search: _search)
                case .paymentPercent: presentableView(for: paymentPercent, title: _title, details: _details, show: show, search: _search)
                case .amount: presentableView(for: amount, title: _title, details: _details, show: show, search: _search)
                case .settlementDiscountAmount: presentableView(for: settlementDiscountAmount, title: _title, details: _details, show: show, search: _search)
                case .penaltyAmount: presentableView(for: penaltyAmount, title: _title, details: _details, show: show, search: _search)
                case .paymentTermsDetailsURI: presentableView(for: paymentTermsDetailsURI, title: _title, details: _details, show: show, search: _search)
                case .paymentDueDate: anyView(for: paymentDueDate, title: _title, details: _details, show: show, search: _search)
                case .installmentDueDate: anyView(for: installmentDueDate, title: _title, details: _details, show: show, search: _search)
                case .invoicingPartyReference: presentableView(for: invoicingPartyReference, title: _title, details: _details, show: show, search: _search)
                case .settlementPeriod: presentableView(for: settlementPeriod, title: _title, details: _details, show: show, search: _search)
                case .penaltyPeriod: presentableView(for: penaltyPeriod, title: _title, details: _details, show: show, search: _search)
                case .exchangeRate: presentableView(for: exchangeRate, title: _title, details: _details, show: show, search: _search)
                case .validityPeriod: presentableView(for: validityPeriod, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PaymentTerms: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = paymentDueDate?.search(string) { return result }
        if let result = installmentDueDate?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = paymentMeansID.compactMap({ $0.search(string)  }).first { return result }
        if let result = prepaidPaymentReferenceID?.search(string) { return result }
        if let result = note.compactMap({ $0.search(string)  }).first { return result }
        if let result = referenceEventCode?.search(string) { return result }
        if let result = settlementDiscountPercent?.search(string) { return result }
        if let result = penaltySurchargePercent?.search(string) { return result }
        if let result = paymentPercent?.search(string) { return result }
        if let result = amount?.search(string) { return result }
        if let result = settlementDiscountAmount?.search(string) { return result }
        if let result = penaltyAmount?.search(string) { return result }
        if let result = paymentTermsDetailsURI?.search(string) { return result }
        if let result = invoicingPartyReference?.search(string) { return result }
        if let result = settlementPeriod?.search(string) { return result }
        if let result = penaltyPeriod?.search(string) { return result }
        if let result = exchangeRate?.search(string) { return result }
        if let result = validityPeriod?.search(string) { return result }
        return nil
    }
}
