// 
// Period.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Period: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .startDate,
        .startTime,
        .endDate,
        .endTime,
        .durationMeasure,
        .descriptionCode,
        .description
]
    public var hasNilValue: Bool {(startDate == nil)
        || (startTime == nil)
        || (endDate == nil)
        || (endTime == nil)
        || (durationMeasure == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .startDate: return "Start Date"
            case .startTime: return "Start Time"
            case .endDate: return "End Date"
            case .endTime: return "End Time"
            case .durationMeasure: return "Duration"
            case .descriptionCode: return "Description Code"
            case .description: return "Description"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .startDate: return "The date on which this period begins."
            case .startTime: return "The time at which this period begins."
            case .endDate: return "The date on which this period ends."
            case .endTime: return "The time at which this period ends."
            case .durationMeasure: return "The duration of this period, expressed as an ISO 8601 code."
            case .descriptionCode: return "A description of this period, expressed as a code."
            case .description: return "A description of this period, expressed as text."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .startDate: result = startDate?.search(_search)
                case .startTime: result = startTime?.search(_search)
                case .endDate: result = endDate?.search(_search)
                case .endTime: result = endTime?.search(_search)
                case .durationMeasure: result = durationMeasure?.search(_search)
                case .descriptionCode: result = descriptionCode.compactMap({ $0.search(_search) }).first
                case .description: result = description.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .startDate: anyView(for: startDate, title: _title, details: _details, show: show, search: _search)
                case .startTime: anyView(for: startTime, title: _title, details: _details, show: show, search: _search)
                case .endDate: anyView(for: endDate, title: _title, details: _details, show: show, search: _search)
                case .endTime: anyView(for: endTime, title: _title, details: _details, show: show, search: _search)
                case .durationMeasure: presentableView(for: durationMeasure, title: _title, details: _details, show: show, search: _search)
                case .descriptionCode: presentableView(for: descriptionCode.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Period: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = startDate?.search(string) { return result }
        if let result = startTime?.search(string) { return result }
        if let result = endDate?.search(string) { return result }
        if let result = endTime?.search(string) { return result }
        if let result = durationMeasure?.search(string) { return result }
        if let result = descriptionCode.compactMap({ $0.search(string)  }).first { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
