// 
// Person.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Person: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .firstName,
        .familyName,
        .title,
        .middleName,
        .otherName,
        .nameSuffix,
        .jobTitle,
        .nationalityID,
        .genderCode,
        .birthDate,
        .birthplaceName,
        .organizationDepartment,
        .contact,
        .financialAccount,
        .identityDocumentReference,
        .residenceAddress
]
    public var hasNilValue: Bool {(id == nil)
        || (firstName == nil)
        || (familyName == nil)
        || (title == nil)
        || (middleName == nil)
        || (otherName == nil)
        || (nameSuffix == nil)
        || (jobTitle == nil)
        || (nationalityID == nil)
        || (genderCode == nil)
        || (birthDate == nil)
        || (birthplaceName == nil)
        || (organizationDepartment == nil)
        || (contact == nil)
        || (financialAccount == nil)
        || (residenceAddress == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .firstName: return "First Name"
            case .familyName: return "Family Name"
            case .title: return "Title"
            case .middleName: return "Middle Name"
            case .otherName: return "Other Name"
            case .nameSuffix: return "Name Suffix"
            case .jobTitle: return "Job Title"
            case .nationalityID: return "Nationality"
            case .genderCode: return "Gender Code"
            case .birthDate: return "Birth Date"
            case .birthplaceName: return "Birthplace Name"
            case .organizationDepartment: return "Organization Department"
            case .contact: return "Contact"
            case .financialAccount: return "Financial Account"
            case .identityDocumentReference: return "Identity Document Reference"
            case .residenceAddress: return "Residence Address"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this person."
            case .firstName: return "This person's given name."
            case .familyName: return "This person's family name."
            case .title: return "This person's title of address (e.g., Mr, Ms, Dr, Sir)."
            case .middleName: return "This person's middle name(s) or initials."
            case .otherName: return "This person's second family name."
            case .nameSuffix: return "A suffix to this person's name (e.g., PhD, OBE, Jr)."
            case .jobTitle: return "This person's job title (for a particular role) within an organization."
            case .nationalityID: return "An identifier for this person's nationality."
            case .genderCode: return "A code (e.g., ISO 5218) signifying the gender of this person."
            case .birthDate: return "This person's date of birth."
            case .birthplaceName: return "The name of the place where this person was born, expressed as text."
            case .organizationDepartment: return "The department or subdivision of an organization that this person belongs to (in a particular role)."
            case .contact: return "Contact information for this person."
            case .financialAccount: return "The financial account associated with this person."
            case .identityDocumentReference: return "A reference to a document that can precisely identify this person (e.g., a driver's license)."
            case .residenceAddress: return "This person's address of residence."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .firstName: result = firstName?.search(_search)
                case .familyName: result = familyName?.search(_search)
                case .title: result = title?.search(_search)
                case .middleName: result = middleName?.search(_search)
                case .otherName: result = otherName?.search(_search)
                case .nameSuffix: result = nameSuffix?.search(_search)
                case .jobTitle: result = jobTitle?.search(_search)
                case .nationalityID: result = nationalityID?.search(_search)
                case .genderCode: result = genderCode?.search(_search)
                case .birthDate: result = birthDate?.search(_search)
                case .birthplaceName: result = birthplaceName?.search(_search)
                case .organizationDepartment: result = organizationDepartment?.search(_search)
                case .contact: result = contact?.search(_search)
                case .financialAccount: result = financialAccount?.search(_search)
                case .identityDocumentReference: result = identityDocumentReference.compactMap({ $0.search(_search) }).first
                case .residenceAddress: result = residenceAddress?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .firstName: presentableView(for: firstName, title: _title, details: _details, show: show, search: _search)
                case .familyName: presentableView(for: familyName, title: _title, details: _details, show: show, search: _search)
                case .title: presentableView(for: title, title: _title, details: _details, show: show, search: _search)
                case .middleName: presentableView(for: middleName, title: _title, details: _details, show: show, search: _search)
                case .otherName: presentableView(for: otherName, title: _title, details: _details, show: show, search: _search)
                case .nameSuffix: presentableView(for: nameSuffix, title: _title, details: _details, show: show, search: _search)
                case .jobTitle: presentableView(for: jobTitle, title: _title, details: _details, show: show, search: _search)
                case .nationalityID: presentableView(for: nationalityID, title: _title, details: _details, show: show, search: _search)
                case .genderCode: presentableView(for: genderCode, title: _title, details: _details, show: show, search: _search)
                case .birthDate: anyView(for: birthDate, title: _title, details: _details, show: show, search: _search)
                case .birthplaceName: presentableView(for: birthplaceName, title: _title, details: _details, show: show, search: _search)
                case .organizationDepartment: presentableView(for: organizationDepartment, title: _title, details: _details, show: show, search: _search)
                case .contact: presentableView(for: contact, title: _title, details: _details, show: show, search: _search)
                case .financialAccount: presentableView(for: financialAccount, title: _title, details: _details, show: show, search: _search)
                case .identityDocumentReference: presentableView(for: identityDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .residenceAddress: presentableView(for: residenceAddress, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Person: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = birthDate?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = firstName?.search(string) { return result }
        if let result = familyName?.search(string) { return result }
        if let result = title?.search(string) { return result }
        if let result = middleName?.search(string) { return result }
        if let result = otherName?.search(string) { return result }
        if let result = nameSuffix?.search(string) { return result }
        if let result = jobTitle?.search(string) { return result }
        if let result = nationalityID?.search(string) { return result }
        if let result = genderCode?.search(string) { return result }
        if let result = birthplaceName?.search(string) { return result }
        if let result = organizationDepartment?.search(string) { return result }
        if let result = contact?.search(string) { return result }
        if let result = financialAccount?.search(string) { return result }
        if let result = identityDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = residenceAddress?.search(string) { return result }
        return nil
    }
}
