// 
// PhysicalAttribute.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PhysicalAttribute: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .attributeID,
        .positionCode,
        .descriptionCode,
        .description
]
    public var hasNilValue: Bool {(positionCode == nil)
        || (descriptionCode == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .attributeID: return "Attribute Identifier"
            case .positionCode: return "Position Code"
            case .descriptionCode: return "Description Code"
            case .description: return "Description"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .attributeID: return "An identifier for this physical attribute."
            case .positionCode: return "A code signifying the position of this physical attribute."
            case .descriptionCode: return "A description of the physical attribute, expressed as a code."
            case .description: return "A description of the physical attribute, expressed as text."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .attributeID: result = attributeID.search(_search)
                case .positionCode: result = positionCode?.search(_search)
                case .descriptionCode: result = descriptionCode?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .attributeID: presentableView(for: attributeID, title: _title, details: _details, show: show, search: _search)
                case .positionCode: presentableView(for: positionCode, title: _title, details: _details, show: show, search: _search)
                case .descriptionCode: presentableView(for: descriptionCode, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PhysicalAttribute: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = attributeID.search(string) { return result }
        if let result = positionCode?.search(string) { return result }
        if let result = descriptionCode?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
