// 
// PowerOfAttorney.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PowerOfAttorney: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .issueDate,
        .issueTime,
        .description,
        .notaryParty,
        .agentParty,
        .witnessParty,
        .mandateDocumentReference
]
    public var hasNilValue: Bool {(id == nil)
        || (issueDate == nil)
        || (issueTime == nil)
        || (notaryParty == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .issueDate: return "Issue Date"
            case .issueTime: return "Issue Time"
            case .description: return "Description"
            case .notaryParty: return "Notary Party"
            case .agentParty: return "Agent Party"
            case .witnessParty: return "Witness Party"
            case .mandateDocumentReference: return "Mandate Document Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this power of attorney."
            case .issueDate: return "The date on which this power of attorney was issued."
            case .issueTime: return "The time at which this power of attorney was issued."
            case .description: return "Text describing this power of attorney."
            case .notaryParty: return "The party notarizing this power of attorney."
            case .agentParty: return "The party who acts as an agent or fiduciary for the principal and who holds this power of attorney on behalf of the principal."
            case .witnessParty: return "An association to a WitnessParty."
            case .mandateDocumentReference: return "A reference to a mandate associated with this power of attorney."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .issueDate: result = issueDate?.search(_search)
                case .issueTime: result = issueTime?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
                case .notaryParty: result = notaryParty?.search(_search)
                case .agentParty: result = agentParty.search(_search)
                case .witnessParty: result = witnessParty.compactMap({ $0.search(_search) }).first
                case .mandateDocumentReference: result = mandateDocumentReference.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .issueDate: anyView(for: issueDate, title: _title, details: _details, show: show, search: _search)
                case .issueTime: anyView(for: issueTime, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .notaryParty: presentableView(for: notaryParty, title: _title, details: _details, show: show, search: _search)
                case .agentParty: presentableView(for: agentParty, title: _title, details: _details, show: show, search: _search)
                case .witnessParty: presentableView(for: witnessParty.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .mandateDocumentReference: presentableView(for: mandateDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PowerOfAttorney: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = issueDate?.search(string) { return result }
        if let result = issueTime?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        if let result = notaryParty?.search(string) { return result }
        if let result = agentParty.search(string) { return result }
        if let result = witnessParty.compactMap({ $0.search(string)  }).first { return result }
        if let result = mandateDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
