// 
// Price.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Price: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .priceAmount,
        .baseQuantity,
        .priceChangeReason,
        .priceTypeCode,
        .priceType,
        .orderableUnitFactorRate,
        .validityPeriod,
        .priceList,
        .allowanceCharge,
        .pricingExchangeRate
]
    public var hasNilValue: Bool {(baseQuantity == nil)
        || (priceTypeCode == nil)
        || (priceType == nil)
        || (orderableUnitFactorRate == nil)
        || (priceList == nil)
        || (pricingExchangeRate == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .priceAmount: return "Price Amount"
            case .baseQuantity: return "Base Quantity"
            case .priceChangeReason: return "Price Change Reason"
            case .priceTypeCode: return "Price Type Code"
            case .priceType: return "Price Type"
            case .orderableUnitFactorRate: return "Orderable Unit Factor"
            case .validityPeriod: return "Validity Period"
            case .priceList: return "Price List"
            case .allowanceCharge: return "Allowance Charge"
            case .pricingExchangeRate: return "Pricing Exchange Rate"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .priceAmount: return "The amount of the price."
            case .baseQuantity: return "The quantity at which this price applies."
            case .priceChangeReason: return "A reason for a price change."
            case .priceTypeCode: return "The type of price, expressed as a code."
            case .priceType: return "The type of price, expressed as text."
            case .orderableUnitFactorRate: return "The factor by which the base price unit can be converted to the orderable unit."
            case .validityPeriod: return "A period during which this price is valid."
            case .priceList: return "Information about a price list applicable to this price."
            case .allowanceCharge: return "An allowance or charge associated with this price."
            case .pricingExchangeRate: return "The exchange rate applicable to this price, if it differs from the exchange rate applicable to the document as a whole."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .priceAmount: result = priceAmount.search(_search)
                case .baseQuantity: result = baseQuantity?.search(_search)
                case .priceChangeReason: result = priceChangeReason.compactMap({ $0.search(_search) }).first
                case .priceTypeCode: result = priceTypeCode?.search(_search)
                case .priceType: result = priceType?.search(_search)
                case .orderableUnitFactorRate: result = orderableUnitFactorRate?.search(_search)
                case .validityPeriod: result = validityPeriod.compactMap({ $0.search(_search) }).first
                case .priceList: result = priceList?.search(_search)
                case .allowanceCharge: result = allowanceCharge.compactMap({ $0.search(_search) }).first
                case .pricingExchangeRate: result = pricingExchangeRate?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .priceAmount: presentableView(for: priceAmount, title: _title, details: _details, show: show, search: _search)
                case .baseQuantity: presentableView(for: baseQuantity, title: _title, details: _details, show: show, search: _search)
                case .priceChangeReason: presentableView(for: priceChangeReason.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .priceTypeCode: presentableView(for: priceTypeCode, title: _title, details: _details, show: show, search: _search)
                case .priceType: presentableView(for: priceType, title: _title, details: _details, show: show, search: _search)
                case .orderableUnitFactorRate: presentableView(for: orderableUnitFactorRate, title: _title, details: _details, show: show, search: _search)
                case .validityPeriod: presentableView(for: validityPeriod.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .priceList: presentableView(for: priceList, title: _title, details: _details, show: show, search: _search)
                case .allowanceCharge: presentableView(for: allowanceCharge.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .pricingExchangeRate: presentableView(for: pricingExchangeRate, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Price: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = priceAmount.search(string) { return result }
        if let result = baseQuantity?.search(string) { return result }
        if let result = priceChangeReason.compactMap({ $0.search(string)  }).first { return result }
        if let result = priceTypeCode?.search(string) { return result }
        if let result = priceType?.search(string) { return result }
        if let result = orderableUnitFactorRate?.search(string) { return result }
        if let result = validityPeriod.compactMap({ $0.search(string)  }).first { return result }
        if let result = priceList?.search(string) { return result }
        if let result = allowanceCharge.compactMap({ $0.search(string)  }).first { return result }
        if let result = pricingExchangeRate?.search(string) { return result }
        return nil
    }
}
