// 
// PriceExtension.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PriceExtension: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .amount,
        .taxTotal
]
    public var hasNilValue: Bool {
        false
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .amount: return "Amount"
            case .taxTotal: return "Tax Total"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .amount: return "The amount of this price extension."
            case .taxTotal: return "A total amount of taxes of a particular kind applicable to this price extension."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .amount: result = amount.search(_search)
                case .taxTotal: result = taxTotal.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .amount: presentableView(for: amount, title: _title, details: _details, show: show, search: _search)
                case .taxTotal: presentableView(for: taxTotal.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PriceExtension: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = amount.search(string) { return result }
        if let result = taxTotal.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
