// 
// PriceList.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PriceList: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .statusCode,
        .validityPeriod,
        .previousPriceList
]
    public var hasNilValue: Bool {(id == nil)
        || (statusCode == nil)
        || (previousPriceList == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .statusCode: return "Status Code"
            case .validityPeriod: return "Validity Period"
            case .previousPriceList: return "Previous Price List"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this price list."
            case .statusCode: return "A code signifying whether this price list is an original, copy, revision, or cancellation."
            case .validityPeriod: return "A period during which this price list is valid."
            case .previousPriceList: return "The previous price list."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .statusCode: result = statusCode?.search(_search)
                case .validityPeriod: result = validityPeriod.compactMap({ $0.search(_search) }).first
                case .previousPriceList: result = previousPriceList?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .statusCode: presentableView(for: statusCode, title: _title, details: _details, show: show, search: _search)
                case .validityPeriod: presentableView(for: validityPeriod.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .previousPriceList: presentableView(for: previousPriceList, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PriceList: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = statusCode?.search(string) { return result }
        if let result = validityPeriod.compactMap({ $0.search(string)  }).first { return result }
        if let result = previousPriceList?.search(string) { return result }
        return nil
    }
}
