// 
// PricingReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.PricingReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .originalItemLocationQuantity,
        .alternativeConditionPrice
]
    public var hasNilValue: Bool {(originalItemLocationQuantity == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .originalItemLocationQuantity: return "Original Item Location Quantity"
            case .alternativeConditionPrice: return "Alternative Condition Price"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .originalItemLocationQuantity: return "An original set of location-specific properties (e.g., price and quantity) associated with this item."
            case .alternativeConditionPrice: return "The price expressed in terms other than the actual price, e.g., the list price v. the contracted price, or the price in bags v. the price in kilos, or the list price in bags v. the contracted price in kilos."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .originalItemLocationQuantity: result = originalItemLocationQuantity?.search(_search)
                case .alternativeConditionPrice: result = alternativeConditionPrice.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .originalItemLocationQuantity: presentableView(for: originalItemLocationQuantity, title: _title, details: _details, show: show, search: _search)
                case .alternativeConditionPrice: presentableView(for: alternativeConditionPrice.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.PricingReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = originalItemLocationQuantity?.search(string) { return result }
        if let result = alternativeConditionPrice.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
