// 
// ProjectReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ProjectReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .uuid,
        .issueDate,
        .workPhaseReference
]
    public var hasNilValue: Bool {(uuid == nil)
        || (issueDate == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .uuid: return "UUID"
            case .issueDate: return "Issue Date"
            case .workPhaseReference: return "Work Phase Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for the referenced project."
            case .uuid: return "A universally unique identifier for the referenced project."
            case .issueDate: return "The date on which the referenced project was issued."
            case .workPhaseReference: return "A specific phase of work in the referenced project."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .issueDate: result = issueDate?.search(_search)
                case .workPhaseReference: result = workPhaseReference.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .issueDate: anyView(for: issueDate, title: _title, details: _details, show: show, search: _search)
                case .workPhaseReference: presentableView(for: workPhaseReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ProjectReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = issueDate?.search(string) { return result }
        if let result = id.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = workPhaseReference.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
