// 
// ReceiptLine.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ReceiptLine: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .uuid,
        .note,
        .receivedQuantity,
        .shortQuantity,
        .shortageActionCode,
        .rejectedQuantity,
        .rejectReasonCode,
        .rejectReason,
        .rejectActionCode,
        .quantityDiscrepancyCode,
        .oversupplyQuantity,
        .receivedDate,
        .timingComplaintCode,
        .timingComplaint,
        .orderLineReference,
        .despatchLineReference,
        .documentReference,
        .item,
        .shipment
]
    public var hasNilValue: Bool {(uuid == nil)
        || (receivedQuantity == nil)
        || (shortQuantity == nil)
        || (shortageActionCode == nil)
        || (rejectedQuantity == nil)
        || (rejectReasonCode == nil)
        || (rejectActionCode == nil)
        || (quantityDiscrepancyCode == nil)
        || (oversupplyQuantity == nil)
        || (receivedDate == nil)
        || (timingComplaintCode == nil)
        || (timingComplaint == nil)
        || (orderLineReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .uuid: return "UUID"
            case .note: return "Note"
            case .receivedQuantity: return "Received Quantity"
            case .shortQuantity: return "Short Quantity"
            case .shortageActionCode: return "Shortage Action Code"
            case .rejectedQuantity: return "Rejected Quantity"
            case .rejectReasonCode: return "Reject Reason Code"
            case .rejectReason: return "Reject Reason"
            case .rejectActionCode: return "Reject Action Code"
            case .quantityDiscrepancyCode: return "Quantity Discrepancy Code"
            case .oversupplyQuantity: return "Oversupply Quantity"
            case .receivedDate: return "Received Date"
            case .timingComplaintCode: return "Timing Complaint Code"
            case .timingComplaint: return "Timing Complaint"
            case .orderLineReference: return "Order Line Reference"
            case .despatchLineReference: return "Despatch Line Reference"
            case .documentReference: return "Document Reference"
            case .item: return "Item"
            case .shipment: return "Shipment"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this receipt line."
            case .uuid: return "A universally unique identifier for this receipt line."
            case .note: return "Free-form text conveying information that is not contained explicitly in other structures."
            case .receivedQuantity: return "The quantity received."
            case .shortQuantity: return "The quantity received short; the difference between the quantity reported despatched and the quantity actually received."
            case .shortageActionCode: return "A code signifying the action that the delivery party wishes the despatch party to take as the result of a shortage."
            case .rejectedQuantity: return "The quantity rejected."
            case .rejectReasonCode: return "The reason for a rejection, expressed as a code."
            case .rejectReason: return "The reason for a rejection, expressed as text."
            case .rejectActionCode: return "A code signifying the action that the delivery party wishes the despatch party to take as the result of a rejection."
            case .quantityDiscrepancyCode: return "A code signifying the type of a discrepancy in quantity."
            case .oversupplyQuantity: return "The quantity over-supplied, i.e., the quantity over and above the quantity ordered."
            case .receivedDate: return "The date on which the goods or services were received."
            case .timingComplaintCode: return "A complaint about the timing of delivery, expressed as a code."
            case .timingComplaint: return "A complaint about the timing of delivery, expressed as text."
            case .orderLineReference: return "A reference to the order line associated with this receipt line."
            case .despatchLineReference: return "A reference to a despatch line associated with this receipt line."
            case .documentReference: return "A reference to a document associated with this receipt line."
            case .item: return "An item associated with this receipt line."
            case .shipment: return "A shipment associated with this receipt line."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .uuid: result = uuid?.search(_search)
                case .note: result = note.compactMap({ $0.search(_search) }).first
                case .receivedQuantity: result = receivedQuantity?.search(_search)
                case .shortQuantity: result = shortQuantity?.search(_search)
                case .shortageActionCode: result = shortageActionCode?.search(_search)
                case .rejectedQuantity: result = rejectedQuantity?.search(_search)
                case .rejectReasonCode: result = rejectReasonCode?.search(_search)
                case .rejectReason: result = rejectReason.compactMap({ $0.search(_search) }).first
                case .rejectActionCode: result = rejectActionCode?.search(_search)
                case .quantityDiscrepancyCode: result = quantityDiscrepancyCode?.search(_search)
                case .oversupplyQuantity: result = oversupplyQuantity?.search(_search)
                case .receivedDate: result = receivedDate?.search(_search)
                case .timingComplaintCode: result = timingComplaintCode?.search(_search)
                case .timingComplaint: result = timingComplaint?.search(_search)
                case .orderLineReference: result = orderLineReference?.search(_search)
                case .despatchLineReference: result = despatchLineReference.compactMap({ $0.search(_search) }).first
                case .documentReference: result = documentReference.compactMap({ $0.search(_search) }).first
                case .item: result = item.compactMap({ $0.search(_search) }).first
                case .shipment: result = shipment.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .uuid: presentableView(for: uuid, title: _title, details: _details, show: show, search: _search)
                case .note: presentableView(for: note.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .receivedQuantity: presentableView(for: receivedQuantity, title: _title, details: _details, show: show, search: _search)
                case .shortQuantity: presentableView(for: shortQuantity, title: _title, details: _details, show: show, search: _search)
                case .shortageActionCode: presentableView(for: shortageActionCode, title: _title, details: _details, show: show, search: _search)
                case .rejectedQuantity: presentableView(for: rejectedQuantity, title: _title, details: _details, show: show, search: _search)
                case .rejectReasonCode: presentableView(for: rejectReasonCode, title: _title, details: _details, show: show, search: _search)
                case .rejectReason: presentableView(for: rejectReason.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .rejectActionCode: presentableView(for: rejectActionCode, title: _title, details: _details, show: show, search: _search)
                case .quantityDiscrepancyCode: presentableView(for: quantityDiscrepancyCode, title: _title, details: _details, show: show, search: _search)
                case .oversupplyQuantity: presentableView(for: oversupplyQuantity, title: _title, details: _details, show: show, search: _search)
                case .receivedDate: anyView(for: receivedDate, title: _title, details: _details, show: show, search: _search)
                case .timingComplaintCode: presentableView(for: timingComplaintCode, title: _title, details: _details, show: show, search: _search)
                case .timingComplaint: presentableView(for: timingComplaint, title: _title, details: _details, show: show, search: _search)
                case .orderLineReference: presentableView(for: orderLineReference, title: _title, details: _details, show: show, search: _search)
                case .despatchLineReference: presentableView(for: despatchLineReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .documentReference: presentableView(for: documentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .item: presentableView(for: item.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .shipment: presentableView(for: shipment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ReceiptLine: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = receivedDate?.search(string) { return result }
        if let result = id.search(string) { return result }
        if let result = uuid?.search(string) { return result }
        if let result = note.compactMap({ $0.search(string)  }).first { return result }
        if let result = receivedQuantity?.search(string) { return result }
        if let result = shortQuantity?.search(string) { return result }
        if let result = shortageActionCode?.search(string) { return result }
        if let result = rejectedQuantity?.search(string) { return result }
        if let result = rejectReasonCode?.search(string) { return result }
        if let result = rejectReason.compactMap({ $0.search(string)  }).first { return result }
        if let result = rejectActionCode?.search(string) { return result }
        if let result = quantityDiscrepancyCode?.search(string) { return result }
        if let result = oversupplyQuantity?.search(string) { return result }
        if let result = timingComplaintCode?.search(string) { return result }
        if let result = timingComplaint?.search(string) { return result }
        if let result = orderLineReference?.search(string) { return result }
        if let result = despatchLineReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = documentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = item.compactMap({ $0.search(string)  }).first { return result }
        if let result = shipment.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
