// 
// ResultOfVerification.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ResultOfVerification: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .validatorID,
        .validationResultCode,
        .validationDate,
        .validationTime,
        .validateProcess,
        .validateTool,
        .validateToolVersion,
        .signatoryParty
]
    public var hasNilValue: Bool {(validatorID == nil)
        || (validationResultCode == nil)
        || (validationDate == nil)
        || (validationTime == nil)
        || (validateProcess == nil)
        || (validateTool == nil)
        || (validateToolVersion == nil)
        || (signatoryParty == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .validatorID: return "Validator"
            case .validationResultCode: return "Validation Result Code"
            case .validationDate: return "Validation Date"
            case .validationTime: return "Validation Time"
            case .validateProcess: return "Validate Process"
            case .validateTool: return "Validate Tool"
            case .validateToolVersion: return "Validate Tool Version"
            case .signatoryParty: return "Signatory Party"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .validatorID: return "An identifier for the organization, person, service, or server that verified the signature."
            case .validationResultCode: return "A code signifying the result of the verification."
            case .validationDate: return "The date upon which verification took place."
            case .validationTime: return "The time at which verification took place."
            case .validateProcess: return "The verification process."
            case .validateTool: return "The tool used to verify the signature."
            case .validateToolVersion: return "The version of the tool used to verify the signature."
            case .signatoryParty: return "The signing party."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .validatorID: result = validatorID?.search(_search)
                case .validationResultCode: result = validationResultCode?.search(_search)
                case .validationDate: result = validationDate?.search(_search)
                case .validationTime: result = validationTime?.search(_search)
                case .validateProcess: result = validateProcess?.search(_search)
                case .validateTool: result = validateTool?.search(_search)
                case .validateToolVersion: result = validateToolVersion?.search(_search)
                case .signatoryParty: result = signatoryParty?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .validatorID: presentableView(for: validatorID, title: _title, details: _details, show: show, search: _search)
                case .validationResultCode: presentableView(for: validationResultCode, title: _title, details: _details, show: show, search: _search)
                case .validationDate: anyView(for: validationDate, title: _title, details: _details, show: show, search: _search)
                case .validationTime: anyView(for: validationTime, title: _title, details: _details, show: show, search: _search)
                case .validateProcess: presentableView(for: validateProcess, title: _title, details: _details, show: show, search: _search)
                case .validateTool: presentableView(for: validateTool, title: _title, details: _details, show: show, search: _search)
                case .validateToolVersion: presentableView(for: validateToolVersion, title: _title, details: _details, show: show, search: _search)
                case .signatoryParty: presentableView(for: signatoryParty, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ResultOfVerification: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = validationDate?.search(string) { return result }
        if let result = validationTime?.search(string) { return result }
        if let result = validatorID?.search(string) { return result }
        if let result = validationResultCode?.search(string) { return result }
        if let result = validateProcess?.search(string) { return result }
        if let result = validateTool?.search(string) { return result }
        if let result = validateToolVersion?.search(string) { return result }
        if let result = signatoryParty?.search(string) { return result }
        return nil
    }
}
