// 
// SecondaryHazard.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.SecondaryHazard: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .placardNotation,
        .placardEndorsement,
        .emergencyProceduresCode,
        .extensionVar
]
    public var hasNilValue: Bool {(id == nil)
        || (placardNotation == nil)
        || (placardEndorsement == nil)
        || (emergencyProceduresCode == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .placardNotation: return "Placard Notation"
            case .placardEndorsement: return "Placard Endorsement"
            case .emergencyProceduresCode: return "Emergency Procedures Code"
            case .extensionVar: return "Extension"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this secondary hazard."
            case .placardNotation: return "Text of the placard notation corresponding to the hazard class of this secondary hazard. Can also be the hazard identification number of the orange placard (upper part) required on the means of transport."
            case .placardEndorsement: return "Text of the placard endorsement for this secondary hazard that is to be shown on the shipping papers for a hazardous item. Can also be used for the number of the orange placard (lower part) required on the means of transport."
            case .emergencyProceduresCode: return "A code signifying the emergency procedures for this secondary hazard."
            case .extensionVar: return "Additional information about the hazardous substance, which can be used (for example) to specify the type of regulatory requirements that apply to this secondary hazard."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .placardNotation: result = placardNotation?.search(_search)
                case .placardEndorsement: result = placardEndorsement?.search(_search)
                case .emergencyProceduresCode: result = emergencyProceduresCode?.search(_search)
                case .extensionVar: result = extensionVar.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .placardNotation: presentableView(for: placardNotation, title: _title, details: _details, show: show, search: _search)
                case .placardEndorsement: presentableView(for: placardEndorsement, title: _title, details: _details, show: show, search: _search)
                case .emergencyProceduresCode: presentableView(for: emergencyProceduresCode, title: _title, details: _details, show: show, search: _search)
                case .extensionVar: presentableView(for: extensionVar.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.SecondaryHazard: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = placardNotation?.search(string) { return result }
        if let result = placardEndorsement?.search(string) { return result }
        if let result = emergencyProceduresCode?.search(string) { return result }
        if let result = extensionVar.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
