// 
// ServiceProviderParty.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.ServiceProviderParty: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .serviceTypeCode,
        .serviceType,
        .party,
        .sellerContact
]
    public var hasNilValue: Bool {(id == nil)
        || (serviceTypeCode == nil)
        || (sellerContact == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .serviceTypeCode: return "Service Type Code"
            case .serviceType: return "Service Type"
            case .party: return "Party"
            case .sellerContact: return "Seller Contact"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this service provider."
            case .serviceTypeCode: return "The type of service provided, expressed as a code."
            case .serviceType: return "The type of service provided, expressed as text."
            case .party: return "The party providing the service."
            case .sellerContact: return "The contact for the service provider."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .serviceTypeCode: result = serviceTypeCode?.search(_search)
                case .serviceType: result = serviceType.compactMap({ $0.search(_search) }).first
                case .party: result = party.search(_search)
                case .sellerContact: result = sellerContact?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .serviceTypeCode: presentableView(for: serviceTypeCode, title: _title, details: _details, show: show, search: _search)
                case .serviceType: presentableView(for: serviceType.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .party: presentableView(for: party, title: _title, details: _details, show: show, search: _search)
                case .sellerContact: presentableView(for: sellerContact, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.ServiceProviderParty: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = serviceTypeCode?.search(string) { return result }
        if let result = serviceType.compactMap({ $0.search(string)  }).first { return result }
        if let result = party.search(string) { return result }
        if let result = sellerContact?.search(string) { return result }
        return nil
    }
}
