// 
// Shipment.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Shipment: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .shippingPriorityLevelCode,
        .handlingCode,
        .handlingInstructions,
        .information,
        .grossWeightMeasure,
        .netWeightMeasure,
        .netNetWeightMeasure,
        .grossVolumeMeasure,
        .netVolumeMeasure,
        .totalGoodsItemQuantity,
        .totalTransportHandlingUnitQuantity,
        .insuranceValueAmount,
        .declaredCustomsValueAmount,
        .declaredForCarriageValueAmount,
        .declaredStatisticsValueAmount,
        .freeOnBoardValueAmount,
        .specialInstructions,
        .deliveryInstructions,
        .splitConsignmentIndicator,
        .consignmentQuantity,
        .consignment,
        .goodsItem,
        .shipmentStage,
        .delivery,
        .transportHandlingUnit,
        .returnAddress,
        .originAddress,
        .firstArrivalPortLocation,
        .lastExitPortLocation,
        .exportCountry,
        .freightAllowanceCharge
]
    public var hasNilValue: Bool {(shippingPriorityLevelCode == nil)
        || (handlingCode == nil)
        || (grossWeightMeasure == nil)
        || (netWeightMeasure == nil)
        || (netNetWeightMeasure == nil)
        || (grossVolumeMeasure == nil)
        || (netVolumeMeasure == nil)
        || (totalGoodsItemQuantity == nil)
        || (totalTransportHandlingUnitQuantity == nil)
        || (insuranceValueAmount == nil)
        || (declaredCustomsValueAmount == nil)
        || (declaredForCarriageValueAmount == nil)
        || (declaredStatisticsValueAmount == nil)
        || (freeOnBoardValueAmount == nil)
        || (splitConsignmentIndicator == nil)
        || (consignmentQuantity == nil)
        || (delivery == nil)
        || (returnAddress == nil)
        || (originAddress == nil)
        || (firstArrivalPortLocation == nil)
        || (lastExitPortLocation == nil)
        || (exportCountry == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .shippingPriorityLevelCode: return "Shipping Priority Level Code"
            case .handlingCode: return "Handling Code"
            case .handlingInstructions: return "Handling Instructions"
            case .information: return "Information"
            case .grossWeightMeasure: return "Gross Weight"
            case .netWeightMeasure: return "Net Weight"
            case .netNetWeightMeasure: return "Net Net Weight"
            case .grossVolumeMeasure: return "Gross Volume"
            case .netVolumeMeasure: return "Net Volume"
            case .totalGoodsItemQuantity: return "Total Goods Item Quantity"
            case .totalTransportHandlingUnitQuantity: return "Total Transport Handling Unit Quantity"
            case .insuranceValueAmount: return "Insurance Value"
            case .declaredCustomsValueAmount: return "Declared Customs Value"
            case .declaredForCarriageValueAmount: return "Declared For Carriage Value"
            case .declaredStatisticsValueAmount: return "Declared Statistics Value"
            case .freeOnBoardValueAmount: return "Free On Board Value"
            case .specialInstructions: return "Special Instructions"
            case .deliveryInstructions: return "Delivery Instructions"
            case .splitConsignmentIndicator: return "Split Consignment Indicator"
            case .consignmentQuantity: return "Consignment Quantity"
            case .consignment: return "Consignment"
            case .goodsItem: return "Goods Item"
            case .shipmentStage: return "Shipment Stage"
            case .delivery: return "Delivery"
            case .transportHandlingUnit: return "Transport Handling Unit"
            case .returnAddress: return "Return Address"
            case .originAddress: return "Origin Address"
            case .firstArrivalPortLocation: return "First Arrival Port Location"
            case .lastExitPortLocation: return "Last Exit Port Location"
            case .exportCountry: return "Export Country"
            case .freightAllowanceCharge: return "Freight Allowance Charge"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this shipment."
            case .shippingPriorityLevelCode: return "A code signifying the priority or level of service required for this shipment."
            case .handlingCode: return "The handling required for this shipment, expressed as a code."
            case .handlingInstructions: return "The handling required for this shipment, expressed as text."
            case .information: return "Free-form text pertinent to this shipment, conveying information that is not contained explicitly in other structures."
            case .grossWeightMeasure: return "The total gross weight of a shipment; the weight of the goods plus packaging plus transport equipment."
            case .netWeightMeasure: return "The net weight of this shipment, excluding packaging."
            case .netNetWeightMeasure: return "The total net weight of this shipment, excluding packaging and transport equipment."
            case .grossVolumeMeasure: return "The total volume of the goods in this shipment, including packaging."
            case .netVolumeMeasure: return "The total volume of the goods in this shipment, excluding packaging and transport equipment."
            case .totalGoodsItemQuantity: return "The total number of goods items in this shipment."
            case .totalTransportHandlingUnitQuantity: return "The number of pieces of transport handling equipment (pallets, boxes, cases, etc.) in this shipment."
            case .insuranceValueAmount: return "The amount covered by insurance for this shipment."
            case .declaredCustomsValueAmount: return "The total declared value for customs purposes of those goods in this shipment that are subject to the same customs procedure and have the same tariff/statistical heading, country information, and duty regime."
            case .declaredForCarriageValueAmount: return "The value of this shipment, declared by the shipper or his agent solely for the purpose of varying the carrier's level of liability from that provided in the contract of carriage, in case of loss or damage to goods or delayed delivery."
            case .declaredStatisticsValueAmount: return "The value, declared for statistical purposes, of those goods in this shipment that have the same statistical heading."
            case .freeOnBoardValueAmount: return "The monetary amount that has to be or has been paid as calculated under the applicable trade delivery."
            case .specialInstructions: return "Special instructions relating to this shipment."
            case .deliveryInstructions: return "Delivery instructions relating to this shipment."
            case .splitConsignmentIndicator: return "An indicator that the consignment has been split in transit (true) or not (false)."
            case .consignmentQuantity: return "The total number of consignments within this shipment."
            case .consignment: return "A consignment covering this shipment."
            case .goodsItem: return "A goods item included in this shipment."
            case .shipmentStage: return "A stage in the transport movement of this shipment."
            case .delivery: return "The delivery of this shipment."
            case .transportHandlingUnit: return "A transport handling unit associated with this shipment."
            case .returnAddress: return "The address to which a shipment should be returned."
            case .originAddress: return "The region in which the goods have been produced or manufactured, according to criteria laid down for the purposes of application of the customs tariff, or of quantitative restrictions, or of any other measure related to trade."
            case .firstArrivalPortLocation: return "The first arrival location of a shipment. This would be a port for sea, an airport for air, a terminal for rail, or a border post for land crossing."
            case .lastExitPortLocation: return "The final exporting location for a shipment. This would be a port for sea, an airport for air, a terminal for rail, or a border post for land crossing."
            case .exportCountry: return "The country from which the goods were originally exported, without any commercial transaction taking place in intermediate countries."
            case .freightAllowanceCharge: return "A cost incurred by the shipper in moving goods, by whatever means, from one place to another under the terms of the contract of carriage. In addition to transport costs, this may include such elements as packing, documentation, loading, unloading, and insurance to the extent that they relate to the freight costs."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .shippingPriorityLevelCode: result = shippingPriorityLevelCode?.search(_search)
                case .handlingCode: result = handlingCode?.search(_search)
                case .handlingInstructions: result = handlingInstructions.compactMap({ $0.search(_search) }).first
                case .information: result = information.compactMap({ $0.search(_search) }).first
                case .grossWeightMeasure: result = grossWeightMeasure?.search(_search)
                case .netWeightMeasure: result = netWeightMeasure?.search(_search)
                case .netNetWeightMeasure: result = netNetWeightMeasure?.search(_search)
                case .grossVolumeMeasure: result = grossVolumeMeasure?.search(_search)
                case .netVolumeMeasure: result = netVolumeMeasure?.search(_search)
                case .totalGoodsItemQuantity: result = totalGoodsItemQuantity?.search(_search)
                case .totalTransportHandlingUnitQuantity: result = totalTransportHandlingUnitQuantity?.search(_search)
                case .insuranceValueAmount: result = insuranceValueAmount?.search(_search)
                case .declaredCustomsValueAmount: result = declaredCustomsValueAmount?.search(_search)
                case .declaredForCarriageValueAmount: result = declaredForCarriageValueAmount?.search(_search)
                case .declaredStatisticsValueAmount: result = declaredStatisticsValueAmount?.search(_search)
                case .freeOnBoardValueAmount: result = freeOnBoardValueAmount?.search(_search)
                case .specialInstructions: result = specialInstructions.compactMap({ $0.search(_search) }).first
                case .deliveryInstructions: result = deliveryInstructions.compactMap({ $0.search(_search) }).first
                case .splitConsignmentIndicator: result = splitConsignmentIndicator?.search(_search)
                case .consignmentQuantity: result = consignmentQuantity?.search(_search)
                case .consignment: result = consignment.compactMap({ $0.search(_search) }).first
                case .goodsItem: result = goodsItem.compactMap({ $0.search(_search) }).first
                case .shipmentStage: result = shipmentStage.compactMap({ $0.search(_search) }).first
                case .delivery: result = delivery?.search(_search)
                case .transportHandlingUnit: result = transportHandlingUnit.compactMap({ $0.search(_search) }).first
                case .returnAddress: result = returnAddress?.search(_search)
                case .originAddress: result = originAddress?.search(_search)
                case .firstArrivalPortLocation: result = firstArrivalPortLocation?.search(_search)
                case .lastExitPortLocation: result = lastExitPortLocation?.search(_search)
                case .exportCountry: result = exportCountry?.search(_search)
                case .freightAllowanceCharge: result = freightAllowanceCharge.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .shippingPriorityLevelCode: presentableView(for: shippingPriorityLevelCode, title: _title, details: _details, show: show, search: _search)
                case .handlingCode: presentableView(for: handlingCode, title: _title, details: _details, show: show, search: _search)
                case .handlingInstructions: presentableView(for: handlingInstructions.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .information: presentableView(for: information.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .grossWeightMeasure: presentableView(for: grossWeightMeasure, title: _title, details: _details, show: show, search: _search)
                case .netWeightMeasure: presentableView(for: netWeightMeasure, title: _title, details: _details, show: show, search: _search)
                case .netNetWeightMeasure: presentableView(for: netNetWeightMeasure, title: _title, details: _details, show: show, search: _search)
                case .grossVolumeMeasure: presentableView(for: grossVolumeMeasure, title: _title, details: _details, show: show, search: _search)
                case .netVolumeMeasure: presentableView(for: netVolumeMeasure, title: _title, details: _details, show: show, search: _search)
                case .totalGoodsItemQuantity: presentableView(for: totalGoodsItemQuantity, title: _title, details: _details, show: show, search: _search)
                case .totalTransportHandlingUnitQuantity: presentableView(for: totalTransportHandlingUnitQuantity, title: _title, details: _details, show: show, search: _search)
                case .insuranceValueAmount: presentableView(for: insuranceValueAmount, title: _title, details: _details, show: show, search: _search)
                case .declaredCustomsValueAmount: presentableView(for: declaredCustomsValueAmount, title: _title, details: _details, show: show, search: _search)
                case .declaredForCarriageValueAmount: presentableView(for: declaredForCarriageValueAmount, title: _title, details: _details, show: show, search: _search)
                case .declaredStatisticsValueAmount: presentableView(for: declaredStatisticsValueAmount, title: _title, details: _details, show: show, search: _search)
                case .freeOnBoardValueAmount: presentableView(for: freeOnBoardValueAmount, title: _title, details: _details, show: show, search: _search)
                case .specialInstructions: presentableView(for: specialInstructions.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .deliveryInstructions: presentableView(for: deliveryInstructions.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .splitConsignmentIndicator: presentableView(for: splitConsignmentIndicator, title: _title, details: _details, show: show, search: _search)
                case .consignmentQuantity: presentableView(for: consignmentQuantity, title: _title, details: _details, show: show, search: _search)
                case .consignment: presentableView(for: consignment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .goodsItem: presentableView(for: goodsItem.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .shipmentStage: presentableView(for: shipmentStage.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .delivery: presentableView(for: delivery, title: _title, details: _details, show: show, search: _search)
                case .transportHandlingUnit: presentableView(for: transportHandlingUnit.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .returnAddress: presentableView(for: returnAddress, title: _title, details: _details, show: show, search: _search)
                case .originAddress: presentableView(for: originAddress, title: _title, details: _details, show: show, search: _search)
                case .firstArrivalPortLocation: presentableView(for: firstArrivalPortLocation, title: _title, details: _details, show: show, search: _search)
                case .lastExitPortLocation: presentableView(for: lastExitPortLocation, title: _title, details: _details, show: show, search: _search)
                case .exportCountry: presentableView(for: exportCountry, title: _title, details: _details, show: show, search: _search)
                case .freightAllowanceCharge: presentableView(for: freightAllowanceCharge.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Shipment: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id.search(string) { return result }
        if let result = shippingPriorityLevelCode?.search(string) { return result }
        if let result = handlingCode?.search(string) { return result }
        if let result = handlingInstructions.compactMap({ $0.search(string)  }).first { return result }
        if let result = information.compactMap({ $0.search(string)  }).first { return result }
        if let result = grossWeightMeasure?.search(string) { return result }
        if let result = netWeightMeasure?.search(string) { return result }
        if let result = netNetWeightMeasure?.search(string) { return result }
        if let result = grossVolumeMeasure?.search(string) { return result }
        if let result = netVolumeMeasure?.search(string) { return result }
        if let result = totalGoodsItemQuantity?.search(string) { return result }
        if let result = totalTransportHandlingUnitQuantity?.search(string) { return result }
        if let result = insuranceValueAmount?.search(string) { return result }
        if let result = declaredCustomsValueAmount?.search(string) { return result }
        if let result = declaredForCarriageValueAmount?.search(string) { return result }
        if let result = declaredStatisticsValueAmount?.search(string) { return result }
        if let result = freeOnBoardValueAmount?.search(string) { return result }
        if let result = specialInstructions.compactMap({ $0.search(string)  }).first { return result }
        if let result = deliveryInstructions.compactMap({ $0.search(string)  }).first { return result }
        if let result = splitConsignmentIndicator?.search(string) { return result }
        if let result = consignmentQuantity?.search(string) { return result }
        if let result = consignment.compactMap({ $0.search(string)  }).first { return result }
        if let result = goodsItem.compactMap({ $0.search(string)  }).first { return result }
        if let result = shipmentStage.compactMap({ $0.search(string)  }).first { return result }
        if let result = delivery?.search(string) { return result }
        if let result = transportHandlingUnit.compactMap({ $0.search(string)  }).first { return result }
        if let result = returnAddress?.search(string) { return result }
        if let result = originAddress?.search(string) { return result }
        if let result = firstArrivalPortLocation?.search(string) { return result }
        if let result = lastExitPortLocation?.search(string) { return result }
        if let result = exportCountry?.search(string) { return result }
        if let result = freightAllowanceCharge.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
