// 
// Signature.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Signature: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .note,
        .validationDate,
        .validationTime,
        .validatorID,
        .canonicalizationMethod,
        .signatureMethod,
        .signatoryParty,
        .digitalSignatureAttachment,
        .originalDocumentReference
]
    public var hasNilValue: Bool {(validationDate == nil)
        || (validationTime == nil)
        || (validatorID == nil)
        || (canonicalizationMethod == nil)
        || (signatureMethod == nil)
        || (signatoryParty == nil)
        || (digitalSignatureAttachment == nil)
        || (originalDocumentReference == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .note: return "Note"
            case .validationDate: return "Validation Date"
            case .validationTime: return "Validation Time"
            case .validatorID: return "Validator"
            case .canonicalizationMethod: return "Canonicalization Method"
            case .signatureMethod: return "Signature Method"
            case .signatoryParty: return "Signatory Party"
            case .digitalSignatureAttachment: return "Digital Signature Attachment"
            case .originalDocumentReference: return "Original Document Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this signature."
            case .note: return "Free-form text conveying information that is not contained explicitly in other structures; in particular, information regarding the circumstances in which the signature is being used."
            case .validationDate: return "The date upon which this signature was verified."
            case .validationTime: return "The time at which this signature was verified."
            case .validatorID: return "An identifier for the organization, person, service, or server that verified this signature."
            case .canonicalizationMethod: return "The method used to perform XML canonicalization of this signature."
            case .signatureMethod: return "Text describing the method of signature."
            case .signatoryParty: return "The signing party."
            case .digitalSignatureAttachment: return "The actual encoded signature (e.g., in XMLDsig format)."
            case .originalDocumentReference: return "A reference to the document that the signature applies to. For evidentiary purposes, this may be the document image that the signatory party saw when applying their signature."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .note: result = note.compactMap({ $0.search(_search) }).first
                case .validationDate: result = validationDate?.search(_search)
                case .validationTime: result = validationTime?.search(_search)
                case .validatorID: result = validatorID?.search(_search)
                case .canonicalizationMethod: result = canonicalizationMethod?.search(_search)
                case .signatureMethod: result = signatureMethod?.search(_search)
                case .signatoryParty: result = signatoryParty?.search(_search)
                case .digitalSignatureAttachment: result = digitalSignatureAttachment?.search(_search)
                case .originalDocumentReference: result = originalDocumentReference?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .note: presentableView(for: note.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .validationDate: anyView(for: validationDate, title: _title, details: _details, show: show, search: _search)
                case .validationTime: anyView(for: validationTime, title: _title, details: _details, show: show, search: _search)
                case .validatorID: presentableView(for: validatorID, title: _title, details: _details, show: show, search: _search)
                case .canonicalizationMethod: presentableView(for: canonicalizationMethod, title: _title, details: _details, show: show, search: _search)
                case .signatureMethod: presentableView(for: signatureMethod, title: _title, details: _details, show: show, search: _search)
                case .signatoryParty: presentableView(for: signatoryParty, title: _title, details: _details, show: show, search: _search)
                case .digitalSignatureAttachment: presentableView(for: digitalSignatureAttachment, title: _title, details: _details, show: show, search: _search)
                case .originalDocumentReference: presentableView(for: originalDocumentReference, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Signature: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = validationDate?.search(string) { return result }
        if let result = validationTime?.search(string) { return result }
        if let result = id.search(string) { return result }
        if let result = note.compactMap({ $0.search(string)  }).first { return result }
        if let result = validatorID?.search(string) { return result }
        if let result = canonicalizationMethod?.search(string) { return result }
        if let result = signatureMethod?.search(string) { return result }
        if let result = signatoryParty?.search(string) { return result }
        if let result = digitalSignatureAttachment?.search(string) { return result }
        if let result = originalDocumentReference?.search(string) { return result }
        return nil
    }
}
