// 
// Status.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Status: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .conditionCode,
        .referenceDate,
        .referenceTime,
        .description,
        .statusReasonCode,
        .statusReason,
        .sequenceID,
        .text,
        .indicationIndicator,
        .percent,
        .reliabilityPercent,
        .condition
]
    public var hasNilValue: Bool {(conditionCode == nil)
        || (referenceDate == nil)
        || (referenceTime == nil)
        || (statusReasonCode == nil)
        || (sequenceID == nil)
        || (indicationIndicator == nil)
        || (percent == nil)
        || (reliabilityPercent == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .conditionCode: return "Condition Code"
            case .referenceDate: return "Reference Date"
            case .referenceTime: return "Reference Time"
            case .description: return "Description"
            case .statusReasonCode: return "Status Reason Code"
            case .statusReason: return "Status Reason"
            case .sequenceID: return "Sequence Identifier"
            case .text: return "Text"
            case .indicationIndicator: return "Indication Indicator"
            case .percent: return "Percent"
            case .reliabilityPercent: return "Reliability Percent"
            case .condition: return "Condition"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .conditionCode: return "Specifies the status condition of the related object."
            case .referenceDate: return "The reference date for this status."
            case .referenceTime: return "The reference time for this status."
            case .description: return "Text describing this status."
            case .statusReasonCode: return "The reason for this status condition or position, expressed as a code."
            case .statusReason: return "The reason for this status condition or position, expressed as text."
            case .sequenceID: return "A sequence identifier for this status."
            case .text: return "Provides any textual information related to this status."
            case .indicationIndicator: return "Specifies an indicator relevant to a specific status."
            case .percent: return "A percentage meaningful in the context of this status."
            case .reliabilityPercent: return "The reliability of this status, expressed as a percentage."
            case .condition: return "Measurements that quantify the condition of the objects covered by the status."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .conditionCode: result = conditionCode?.search(_search)
                case .referenceDate: result = referenceDate?.search(_search)
                case .referenceTime: result = referenceTime?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
                case .statusReasonCode: result = statusReasonCode?.search(_search)
                case .statusReason: result = statusReason.compactMap({ $0.search(_search) }).first
                case .sequenceID: result = sequenceID?.search(_search)
                case .text: result = text.compactMap({ $0.search(_search) }).first
                case .indicationIndicator: result = indicationIndicator?.search(_search)
                case .percent: result = percent?.search(_search)
                case .reliabilityPercent: result = reliabilityPercent?.search(_search)
                case .condition: result = condition.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .conditionCode: presentableView(for: conditionCode, title: _title, details: _details, show: show, search: _search)
                case .referenceDate: anyView(for: referenceDate, title: _title, details: _details, show: show, search: _search)
                case .referenceTime: anyView(for: referenceTime, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .statusReasonCode: presentableView(for: statusReasonCode, title: _title, details: _details, show: show, search: _search)
                case .statusReason: presentableView(for: statusReason.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .sequenceID: presentableView(for: sequenceID, title: _title, details: _details, show: show, search: _search)
                case .text: presentableView(for: text.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .indicationIndicator: presentableView(for: indicationIndicator, title: _title, details: _details, show: show, search: _search)
                case .percent: presentableView(for: percent, title: _title, details: _details, show: show, search: _search)
                case .reliabilityPercent: presentableView(for: reliabilityPercent, title: _title, details: _details, show: show, search: _search)
                case .condition: presentableView(for: condition.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Status: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = referenceDate?.search(string) { return result }
        if let result = referenceTime?.search(string) { return result }
        if let result = conditionCode?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        if let result = statusReasonCode?.search(string) { return result }
        if let result = statusReason.compactMap({ $0.search(string)  }).first { return result }
        if let result = sequenceID?.search(string) { return result }
        if let result = text.compactMap({ $0.search(string)  }).first { return result }
        if let result = indicationIndicator?.search(string) { return result }
        if let result = percent?.search(string) { return result }
        if let result = reliabilityPercent?.search(string) { return result }
        if let result = condition.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
