// 
// Stowage.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.Stowage: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .locationID,
        .location,
        .measurementDimension
]
    public var hasNilValue: Bool {(locationID == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .locationID: return "Location Identifier"
            case .location: return "Location"
            case .measurementDimension: return "Measurement Dimension"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .locationID: return "An identifier for the location."
            case .location: return "Text describing the location."
            case .measurementDimension: return "A measurable dimension (length, mass, weight, or volume) of this stowage."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .locationID: result = locationID?.search(_search)
                case .location: result = location.compactMap({ $0.search(_search) }).first
                case .measurementDimension: result = measurementDimension.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .locationID: presentableView(for: locationID, title: _title, details: _details, show: show, search: _search)
                case .location: presentableView(for: location.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .measurementDimension: presentableView(for: measurementDimension.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.Stowage: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = locationID?.search(string) { return result }
        if let result = location.compactMap({ $0.search(string)  }).first { return result }
        if let result = measurementDimension.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
