// 
// SupplierParty.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.SupplierParty: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .customerAssignedAccountID,
        .additionalAccountID,
        .dataSendingCapability,
        .party,
        .despatchContact,
        .accountingContact,
        .sellerContact
]
    public var hasNilValue: Bool {(customerAssignedAccountID == nil)
        || (dataSendingCapability == nil)
        || (party == nil)
        || (despatchContact == nil)
        || (accountingContact == nil)
        || (sellerContact == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .customerAssignedAccountID: return "Customer Assigned Account Identifier"
            case .additionalAccountID: return "Additional Account Identifier"
            case .dataSendingCapability: return "Data Sending Capability"
            case .party: return "Party"
            case .despatchContact: return "Despatch Contact"
            case .accountingContact: return "Accounting Contact"
            case .sellerContact: return "Seller Contact"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .customerAssignedAccountID: return "An identifier for this supplier party, assigned by the customer."
            case .additionalAccountID: return "An additional identifier for this supplier party."
            case .dataSendingCapability: return "Text describing the supplier's ability to send invoice data via a purchase card provider (e.g., VISA, MasterCard, American Express)."
            case .party: return "The supplier party itself."
            case .despatchContact: return "A contact at this supplier party for despatches (pickups)."
            case .accountingContact: return "A contact at this supplier party for accounting."
            case .sellerContact: return "The primary contact for this supplier party."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .customerAssignedAccountID: result = customerAssignedAccountID?.search(_search)
                case .additionalAccountID: result = additionalAccountID.compactMap({ $0.search(_search) }).first
                case .dataSendingCapability: result = dataSendingCapability?.search(_search)
                case .party: result = party?.search(_search)
                case .despatchContact: result = despatchContact?.search(_search)
                case .accountingContact: result = accountingContact?.search(_search)
                case .sellerContact: result = sellerContact?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .customerAssignedAccountID: presentableView(for: customerAssignedAccountID, title: _title, details: _details, show: show, search: _search)
                case .additionalAccountID: presentableView(for: additionalAccountID.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .dataSendingCapability: presentableView(for: dataSendingCapability, title: _title, details: _details, show: show, search: _search)
                case .party: presentableView(for: party, title: _title, details: _details, show: show, search: _search)
                case .despatchContact: presentableView(for: despatchContact, title: _title, details: _details, show: show, search: _search)
                case .accountingContact: presentableView(for: accountingContact, title: _title, details: _details, show: show, search: _search)
                case .sellerContact: presentableView(for: sellerContact, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.SupplierParty: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = customerAssignedAccountID?.search(string) { return result }
        if let result = additionalAccountID.compactMap({ $0.search(string)  }).first { return result }
        if let result = dataSendingCapability?.search(string) { return result }
        if let result = party?.search(string) { return result }
        if let result = despatchContact?.search(string) { return result }
        if let result = accountingContact?.search(string) { return result }
        if let result = sellerContact?.search(string) { return result }
        return nil
    }
}
