// 
// TaxCategory.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TaxCategory: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .name,
        .percent,
        .baseUnitMeasure,
        .perUnitAmount,
        .taxExemptionReasonCode,
        .taxExemptionReason,
        .tierRange,
        .tierRatePercent,
        .taxScheme
]
    public var hasNilValue: Bool {(id == nil)
        || (name == nil)
        || (percent == nil)
        || (baseUnitMeasure == nil)
        || (perUnitAmount == nil)
        || (taxExemptionReasonCode == nil)
        || (tierRange == nil)
        || (tierRatePercent == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .name: return "Name"
            case .percent: return "Percent"
            case .baseUnitMeasure: return "Base Unit Measure"
            case .perUnitAmount: return "Per Unit Amount"
            case .taxExemptionReasonCode: return "Tax Exemption Reason Code"
            case .taxExemptionReason: return "Tax Exemption Reason"
            case .tierRange: return "Tier Range"
            case .tierRatePercent: return "Tier Rate"
            case .taxScheme: return "Tax Scheme"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this tax category."
            case .name: return "The name of this tax category."
            case .percent: return "The tax rate for this category, expressed as a percentage."
            case .baseUnitMeasure: return "A Unit of Measures used as the basic for the tax calculation applied at a certain rate per unit."
            case .perUnitAmount: return "Where a tax is applied at a certain rate per unit, the rate per unit applied."
            case .taxExemptionReasonCode: return "The reason for tax being exempted, expressed as a code."
            case .taxExemptionReason: return "The reason for tax being exempted, expressed as text."
            case .tierRange: return "Where a tax is tiered, the range of taxable amounts that determines the rate of tax applicable to this tax category."
            case .tierRatePercent: return "Where a tax is tiered, the tax rate that applies within the specified range of taxable amounts for this tax category."
            case .taxScheme: return "The taxation scheme within which this tax category is defined."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .name: result = name?.search(_search)
                case .percent: result = percent?.search(_search)
                case .baseUnitMeasure: result = baseUnitMeasure?.search(_search)
                case .perUnitAmount: result = perUnitAmount?.search(_search)
                case .taxExemptionReasonCode: result = taxExemptionReasonCode?.search(_search)
                case .taxExemptionReason: result = taxExemptionReason.compactMap({ $0.search(_search) }).first
                case .tierRange: result = tierRange?.search(_search)
                case .tierRatePercent: result = tierRatePercent?.search(_search)
                case .taxScheme: result = taxScheme.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .percent: presentableView(for: percent, title: _title, details: _details, show: show, search: _search)
                case .baseUnitMeasure: presentableView(for: baseUnitMeasure, title: _title, details: _details, show: show, search: _search)
                case .perUnitAmount: presentableView(for: perUnitAmount, title: _title, details: _details, show: show, search: _search)
                case .taxExemptionReasonCode: presentableView(for: taxExemptionReasonCode, title: _title, details: _details, show: show, search: _search)
                case .taxExemptionReason: presentableView(for: taxExemptionReason.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .tierRange: presentableView(for: tierRange, title: _title, details: _details, show: show, search: _search)
                case .tierRatePercent: presentableView(for: tierRatePercent, title: _title, details: _details, show: show, search: _search)
                case .taxScheme: presentableView(for: taxScheme, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TaxCategory: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = name?.search(string) { return result }
        if let result = percent?.search(string) { return result }
        if let result = baseUnitMeasure?.search(string) { return result }
        if let result = perUnitAmount?.search(string) { return result }
        if let result = taxExemptionReasonCode?.search(string) { return result }
        if let result = taxExemptionReason.compactMap({ $0.search(string)  }).first { return result }
        if let result = tierRange?.search(string) { return result }
        if let result = tierRatePercent?.search(string) { return result }
        if let result = taxScheme.search(string) { return result }
        return nil
    }
}
