// 
// TaxSubtotal.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TaxSubtotal: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .taxableAmount,
        .taxAmount,
        .calculationSequenceNumeric,
        .transactionCurrencyTaxAmount,
        .percent,
        .baseUnitMeasure,
        .perUnitAmount,
        .tierRange,
        .tierRatePercent,
        .taxCategory
]
    public var hasNilValue: Bool {(taxableAmount == nil)
        || (calculationSequenceNumeric == nil)
        || (transactionCurrencyTaxAmount == nil)
        || (percent == nil)
        || (baseUnitMeasure == nil)
        || (perUnitAmount == nil)
        || (tierRange == nil)
        || (tierRatePercent == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .taxableAmount: return "Taxable Amount"
            case .taxAmount: return "Tax Amount"
            case .calculationSequenceNumeric: return "Calculation Sequence"
            case .transactionCurrencyTaxAmount: return "Transaction Currency Tax Amount"
            case .percent: return "Percent"
            case .baseUnitMeasure: return "Base Unit Measure"
            case .perUnitAmount: return "Per Unit Amount"
            case .tierRange: return "Tier Range"
            case .tierRatePercent: return "Tier Rate"
            case .taxCategory: return "Tax Category"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .taxableAmount: return "The net amount to which the tax percent (rate) is applied to calculate the tax amount."
            case .taxAmount: return "The amount of this tax subtotal."
            case .calculationSequenceNumeric: return "The number of this tax subtotal in the sequence of subtotals corresponding to the order in which multiple taxes are applied. If all taxes are applied to the same taxable amount (i.e., their order of application is inconsequential), then CalculationSequenceNumeric is 1 for all tax subtotals applied to a given amount."
            case .transactionCurrencyTaxAmount: return "The amount of this tax subtotal, expressed in the currency used for invoicing."
            case .percent: return "The tax rate of the tax category applied to this tax subtotal, expressed as a percentage."
            case .baseUnitMeasure: return "The unit of measure on which the tax calculation is based"
            case .perUnitAmount: return "Where a tax is applied at a certain rate per unit, the rate per unit applied."
            case .tierRange: return "Where a tax is tiered, the range of taxable amounts that determines the rate of tax applicable to this tax subtotal."
            case .tierRatePercent: return "Where a tax is tiered, the tax rate that applies within a specified range of taxable amounts for this tax subtotal."
            case .taxCategory: return "The tax category applicable to this subtotal."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .taxableAmount: result = taxableAmount?.search(_search)
                case .taxAmount: result = taxAmount.search(_search)
                case .calculationSequenceNumeric: result = calculationSequenceNumeric?.search(_search)
                case .transactionCurrencyTaxAmount: result = transactionCurrencyTaxAmount?.search(_search)
                case .percent: result = percent?.search(_search)
                case .baseUnitMeasure: result = baseUnitMeasure?.search(_search)
                case .perUnitAmount: result = perUnitAmount?.search(_search)
                case .tierRange: result = tierRange?.search(_search)
                case .tierRatePercent: result = tierRatePercent?.search(_search)
                case .taxCategory: result = taxCategory.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .taxableAmount: presentableView(for: taxableAmount, title: _title, details: _details, show: show, search: _search)
                case .taxAmount: presentableView(for: taxAmount, title: _title, details: _details, show: show, search: _search)
                case .calculationSequenceNumeric: presentableView(for: calculationSequenceNumeric, title: _title, details: _details, show: show, search: _search)
                case .transactionCurrencyTaxAmount: presentableView(for: transactionCurrencyTaxAmount, title: _title, details: _details, show: show, search: _search)
                case .percent: presentableView(for: percent, title: _title, details: _details, show: show, search: _search)
                case .baseUnitMeasure: presentableView(for: baseUnitMeasure, title: _title, details: _details, show: show, search: _search)
                case .perUnitAmount: presentableView(for: perUnitAmount, title: _title, details: _details, show: show, search: _search)
                case .tierRange: presentableView(for: tierRange, title: _title, details: _details, show: show, search: _search)
                case .tierRatePercent: presentableView(for: tierRatePercent, title: _title, details: _details, show: show, search: _search)
                case .taxCategory: presentableView(for: taxCategory, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TaxSubtotal: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = taxableAmount?.search(string) { return result }
        if let result = taxAmount.search(string) { return result }
        if let result = calculationSequenceNumeric?.search(string) { return result }
        if let result = transactionCurrencyTaxAmount?.search(string) { return result }
        if let result = percent?.search(string) { return result }
        if let result = baseUnitMeasure?.search(string) { return result }
        if let result = perUnitAmount?.search(string) { return result }
        if let result = tierRange?.search(string) { return result }
        if let result = tierRatePercent?.search(string) { return result }
        if let result = taxCategory.search(string) { return result }
        return nil
    }
}
