// 
// TaxTotal.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TaxTotal: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .taxAmount,
        .roundingAmount,
        .taxEvidenceIndicator,
        .taxIncludedIndicator,
        .taxSubtotal
]
    public var hasNilValue: Bool {(roundingAmount == nil)
        || (taxEvidenceIndicator == nil)
        || (taxIncludedIndicator == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .taxAmount: return "Tax Amount"
            case .roundingAmount: return "Rounding Amount"
            case .taxEvidenceIndicator: return "Tax Evidence Indicator"
            case .taxIncludedIndicator: return "Tax Included Indicator"
            case .taxSubtotal: return "Tax Subtotal"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .taxAmount: return "The total tax amount for a particular taxation scheme, e.g., VAT; the sum of the tax subtotals for each tax category within the taxation scheme."
            case .roundingAmount: return "The rounding amount (positive or negative) added to the calculated tax total to produce the rounded TaxAmount."
            case .taxEvidenceIndicator: return "An indicator that this total is recognized as legal evidence for taxation purposes (true) or not (false)."
            case .taxIncludedIndicator: return "An indicator that tax is included in the calculation (true) or not (false)."
            case .taxSubtotal: return "One of the subtotals the sum of which equals the total tax amount for a particular taxation scheme."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .taxAmount: result = taxAmount.search(_search)
                case .roundingAmount: result = roundingAmount?.search(_search)
                case .taxEvidenceIndicator: result = taxEvidenceIndicator?.search(_search)
                case .taxIncludedIndicator: result = taxIncludedIndicator?.search(_search)
                case .taxSubtotal: result = taxSubtotal.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .taxAmount: presentableView(for: taxAmount, title: _title, details: _details, show: show, search: _search)
                case .roundingAmount: presentableView(for: roundingAmount, title: _title, details: _details, show: show, search: _search)
                case .taxEvidenceIndicator: presentableView(for: taxEvidenceIndicator, title: _title, details: _details, show: show, search: _search)
                case .taxIncludedIndicator: presentableView(for: taxIncludedIndicator, title: _title, details: _details, show: show, search: _search)
                case .taxSubtotal: presentableView(for: taxSubtotal.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TaxTotal: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = taxAmount.search(string) { return result }
        if let result = roundingAmount?.search(string) { return result }
        if let result = taxEvidenceIndicator?.search(string) { return result }
        if let result = taxIncludedIndicator?.search(string) { return result }
        if let result = taxSubtotal.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
