// 
// TradeFinancing.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TradeFinancing: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .financingInstrumentCode,
        .contractDocumentReference,
        .documentReference,
        .financingParty,
        .financingFinancialAccount,
        .clause
]
    public var hasNilValue: Bool {(id == nil)
        || (financingInstrumentCode == nil)
        || (contractDocumentReference == nil)
        || (financingFinancialAccount == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .financingInstrumentCode: return "Financing Instrument Code"
            case .contractDocumentReference: return "Contract Document Reference"
            case .documentReference: return "Document Reference"
            case .financingParty: return "Financing Party"
            case .financingFinancialAccount: return "Financing Financial Account"
            case .clause: return "Clause"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this trade financing instrument."
            case .financingInstrumentCode: return "A code signifying the type of this financing instrument."
            case .contractDocumentReference: return "A reference to a contract document."
            case .documentReference: return "A reference to a document associated with this trade financing instrument."
            case .financingParty: return "The financing party (bank or other enabled party)."
            case .financingFinancialAccount: return "An internal bank account used by the bank or its first agent to manage the line of credit granted to the financing requester."
            case .clause: return "A clause applicable to this trade financing instrument."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .financingInstrumentCode: result = financingInstrumentCode?.search(_search)
                case .contractDocumentReference: result = contractDocumentReference?.search(_search)
                case .documentReference: result = documentReference.compactMap({ $0.search(_search) }).first
                case .financingParty: result = financingParty.search(_search)
                case .financingFinancialAccount: result = financingFinancialAccount?.search(_search)
                case .clause: result = clause.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .financingInstrumentCode: presentableView(for: financingInstrumentCode, title: _title, details: _details, show: show, search: _search)
                case .contractDocumentReference: presentableView(for: contractDocumentReference, title: _title, details: _details, show: show, search: _search)
                case .documentReference: presentableView(for: documentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .financingParty: presentableView(for: financingParty, title: _title, details: _details, show: show, search: _search)
                case .financingFinancialAccount: presentableView(for: financingFinancialAccount, title: _title, details: _details, show: show, search: _search)
                case .clause: presentableView(for: clause.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TradeFinancing: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = financingInstrumentCode?.search(string) { return result }
        if let result = contractDocumentReference?.search(string) { return result }
        if let result = documentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = financingParty.search(string) { return result }
        if let result = financingFinancialAccount?.search(string) { return result }
        if let result = clause.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
