// 
// TradingTerms.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TradingTerms: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .information,
        .reference,
        .applicableAddress
]
    public var hasNilValue: Bool {(reference == nil)
        || (applicableAddress == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .information: return "Information"
            case .reference: return "Reference"
            case .applicableAddress: return "Applicable Address"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .information: return "Text describing the terms of a trade agreement."
            case .reference: return "A reference quoting the basis of the terms"
            case .applicableAddress: return "The address at which these trading terms apply."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .information: result = information.compactMap({ $0.search(_search) }).first
                case .reference: result = reference?.search(_search)
                case .applicableAddress: result = applicableAddress?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .information: presentableView(for: information.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .reference: presentableView(for: reference, title: _title, details: _details, show: show, search: _search)
                case .applicableAddress: presentableView(for: applicableAddress, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TradingTerms: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = information.compactMap({ $0.search(string)  }).first { return result }
        if let result = reference?.search(string) { return result }
        if let result = applicableAddress?.search(string) { return result }
        return nil
    }
}
