// 
// TransactionConditions.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TransactionConditions: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .actionCode,
        .description,
        .documentReference
]
    public var hasNilValue: Bool {(id == nil)
        || (actionCode == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .actionCode: return "Action Code"
            case .description: return "Description"
            case .documentReference: return "Document Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for conditions of the transaction, typically purchase/sales conditions."
            case .actionCode: return "A code signifying a type of action relating to sales or payment conditions."
            case .description: return "Text describing the transaction conditions."
            case .documentReference: return "A document associated with these transaction conditions."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .actionCode: result = actionCode?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
                case .documentReference: result = documentReference.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .actionCode: presentableView(for: actionCode, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .documentReference: presentableView(for: documentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TransactionConditions: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = actionCode?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        if let result = documentReference.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
