// 
// TransportEquipmentSeal.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TransportEquipmentSeal: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .sealIssuerTypeCode,
        .condition,
        .sealStatusCode,
        .sealingPartyType
]
    public var hasNilValue: Bool {(sealIssuerTypeCode == nil)
        || (condition == nil)
        || (sealStatusCode == nil)
        || (sealingPartyType == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .sealIssuerTypeCode: return "Seal Issuer Type Code"
            case .condition: return "Condition"
            case .sealStatusCode: return "Seal Status Code"
            case .sealingPartyType: return "Sealing Party Type"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this transport equipment seal."
            case .sealIssuerTypeCode: return "A code signifying the type of party that issues and is responsible for this transport equipment seal."
            case .condition: return "The condition of this transport equipment seal."
            case .sealStatusCode: return "A code signifying the condition of this transport equipment seal."
            case .sealingPartyType: return "The role of the sealing party."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id.search(_search)
                case .sealIssuerTypeCode: result = sealIssuerTypeCode?.search(_search)
                case .condition: result = condition?.search(_search)
                case .sealStatusCode: result = sealStatusCode?.search(_search)
                case .sealingPartyType: result = sealingPartyType?.search(_search)
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .sealIssuerTypeCode: presentableView(for: sealIssuerTypeCode, title: _title, details: _details, show: show, search: _search)
                case .condition: presentableView(for: condition, title: _title, details: _details, show: show, search: _search)
                case .sealStatusCode: presentableView(for: sealStatusCode, title: _title, details: _details, show: show, search: _search)
                case .sealingPartyType: presentableView(for: sealingPartyType, title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TransportEquipmentSeal: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id.search(string) { return result }
        if let result = sealIssuerTypeCode?.search(string) { return result }
        if let result = condition?.search(string) { return result }
        if let result = sealStatusCode?.search(string) { return result }
        if let result = sealingPartyType?.search(string) { return result }
        return nil
    }
}
