// 
// TransportEvent.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TransportEvent: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .identificationID,
        .occurrenceDate,
        .occurrenceTime,
        .transportEventTypeCode,
        .description,
        .completionIndicator,
        .reportedShipment,
        .currentStatus,
        .contact,
        .location,
        .signature,
        .period
]
    public var hasNilValue: Bool {(identificationID == nil)
        || (occurrenceDate == nil)
        || (occurrenceTime == nil)
        || (transportEventTypeCode == nil)
        || (completionIndicator == nil)
        || (reportedShipment == nil)
        || (location == nil)
        || (signature == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .identificationID: return "Identification"
            case .occurrenceDate: return "Occurrence Date"
            case .occurrenceTime: return "Occurrence Time"
            case .transportEventTypeCode: return "Transport Event Type Code"
            case .description: return "Description"
            case .completionIndicator: return "Completion Indicator"
            case .reportedShipment: return "Reported Shipment"
            case .currentStatus: return "Current Status"
            case .contact: return "Contact"
            case .location: return "Location"
            case .signature: return "Signature"
            case .period: return "Period"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .identificationID: return "An identifier for this transport event within an agreed event identification scheme."
            case .occurrenceDate: return "The date of this transport event."
            case .occurrenceTime: return "The time of this transport event."
            case .transportEventTypeCode: return "A code signifying the type of this transport event."
            case .description: return "Text describing this transport event."
            case .completionIndicator: return "An indicator that this transport event has been completed (true) or not (false)."
            case .reportedShipment: return "The shipment involved in this transport event."
            case .currentStatus: return "The current status of this transport event."
            case .contact: return "A contact associated with this transport event."
            case .location: return "The location associated with this transport event."
            case .signature: return "A signature that can be used to sign for an entry or an exit at a transport location (e.g., port terminal)."
            case .period: return "A period of time associated with this transport event."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .identificationID: result = identificationID?.search(_search)
                case .occurrenceDate: result = occurrenceDate?.search(_search)
                case .occurrenceTime: result = occurrenceTime?.search(_search)
                case .transportEventTypeCode: result = transportEventTypeCode?.search(_search)
                case .description: result = description.compactMap({ $0.search(_search) }).first
                case .completionIndicator: result = completionIndicator?.search(_search)
                case .reportedShipment: result = reportedShipment?.search(_search)
                case .currentStatus: result = currentStatus.compactMap({ $0.search(_search) }).first
                case .contact: result = contact.compactMap({ $0.search(_search) }).first
                case .location: result = location?.search(_search)
                case .signature: result = signature?.search(_search)
                case .period: result = period.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .identificationID: presentableView(for: identificationID, title: _title, details: _details, show: show, search: _search)
                case .occurrenceDate: anyView(for: occurrenceDate, title: _title, details: _details, show: show, search: _search)
                case .occurrenceTime: anyView(for: occurrenceTime, title: _title, details: _details, show: show, search: _search)
                case .transportEventTypeCode: presentableView(for: transportEventTypeCode, title: _title, details: _details, show: show, search: _search)
                case .description: presentableView(for: description.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .completionIndicator: presentableView(for: completionIndicator, title: _title, details: _details, show: show, search: _search)
                case .reportedShipment: presentableView(for: reportedShipment, title: _title, details: _details, show: show, search: _search)
                case .currentStatus: presentableView(for: currentStatus.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .contact: presentableView(for: contact.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .location: presentableView(for: location, title: _title, details: _details, show: show, search: _search)
                case .signature: presentableView(for: signature, title: _title, details: _details, show: show, search: _search)
                case .period: presentableView(for: period.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TransportEvent: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = occurrenceDate?.search(string) { return result }
        if let result = occurrenceTime?.search(string) { return result }
        if let result = identificationID?.search(string) { return result }
        if let result = transportEventTypeCode?.search(string) { return result }
        if let result = description.compactMap({ $0.search(string)  }).first { return result }
        if let result = completionIndicator?.search(string) { return result }
        if let result = reportedShipment?.search(string) { return result }
        if let result = currentStatus.compactMap({ $0.search(string)  }).first { return result }
        if let result = contact.compactMap({ $0.search(string)  }).first { return result }
        if let result = location?.search(string) { return result }
        if let result = signature?.search(string) { return result }
        if let result = period.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
