// 
// TransportHandlingUnit.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TransportHandlingUnit: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .transportHandlingUnitTypeCode,
        .handlingCode,
        .handlingInstructions,
        .hazardousRiskIndicator,
        .totalGoodsItemQuantity,
        .totalPackageQuantity,
        .damageRemarks,
        .shippingMarks,
        .traceID,
        .handlingUnitDespatchLine,
        .actualPackage,
        .receivedHandlingUnitReceiptLine,
        .transportEquipment,
        .transportMeans,
        .hazardousGoodsTransit,
        .measurementDimension,
        .minimumTemperature,
        .maximumTemperature,
        .goodsItem,
        .floorSpaceMeasurementDimension,
        .palletSpaceMeasurementDimension,
        .shipmentDocumentReference,
        .status,
        .customsDeclaration,
        .referencedShipment,
        .package
]
    public var hasNilValue: Bool {(id == nil)
        || (transportHandlingUnitTypeCode == nil)
        || (handlingCode == nil)
        || (hazardousRiskIndicator == nil)
        || (totalGoodsItemQuantity == nil)
        || (totalPackageQuantity == nil)
        || (traceID == nil)
        || (minimumTemperature == nil)
        || (maximumTemperature == nil)
        || (floorSpaceMeasurementDimension == nil)
        || (palletSpaceMeasurementDimension == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .transportHandlingUnitTypeCode: return "Transport Handling Unit Type Code"
            case .handlingCode: return "Handling Code"
            case .handlingInstructions: return "Handling Instructions"
            case .hazardousRiskIndicator: return "Hazardous Risk Indicator"
            case .totalGoodsItemQuantity: return "Total Goods Item Quantity"
            case .totalPackageQuantity: return "Total Package Quantity"
            case .damageRemarks: return "Damage Remarks"
            case .shippingMarks: return "Shipping Marks"
            case .traceID: return "Trace Identifier"
            case .handlingUnitDespatchLine: return "Handling Unit Despatch Line"
            case .actualPackage: return "Actual Package"
            case .receivedHandlingUnitReceiptLine: return "Received Handling Unit Receipt Line"
            case .transportEquipment: return "Transport Equipment"
            case .transportMeans: return "Transport Means"
            case .hazardousGoodsTransit: return "Hazardous Goods Transit"
            case .measurementDimension: return "Measurement Dimension"
            case .minimumTemperature: return "Minimum Temperature"
            case .maximumTemperature: return "Maximum Temperature"
            case .goodsItem: return "Goods Item"
            case .floorSpaceMeasurementDimension: return "Floor Space Measurement Dimension"
            case .palletSpaceMeasurementDimension: return "Pallet Space Measurement Dimension"
            case .shipmentDocumentReference: return "Shipment Document Reference"
            case .status: return "Status"
            case .customsDeclaration: return "Customs Declaration"
            case .referencedShipment: return "Referenced Shipment"
            case .package: return "Package"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this transport handling unit."
            case .transportHandlingUnitTypeCode: return "A code signifying the type of this transport handling unit."
            case .handlingCode: return "The handling required for this transport handling unit, expressed as a code."
            case .handlingInstructions: return "The handling required for this transport handling unit, expressed as text."
            case .hazardousRiskIndicator: return "An indicator that the materials contained in this transport handling unit are subject to an international regulation concerning the carriage of dangerous goods (true) or not (false)."
            case .totalGoodsItemQuantity: return "The total number of goods items in this transport handling unit."
            case .totalPackageQuantity: return "The total number of packages in this transport handling unit."
            case .damageRemarks: return "Text describing damage associated with this transport handling unit."
            case .shippingMarks: return "Text describing the marks and numbers on this transport handling unit."
            case .traceID: return "An identifier for use in tracing this transport handling unit, such as the EPC number used in RFID."
            case .handlingUnitDespatchLine: return "A despatch line associated with this transport handling unit."
            case .actualPackage: return "A package contained in this transport handling unit."
            case .receivedHandlingUnitReceiptLine: return "A receipt line associated with this transport handling unit."
            case .transportEquipment: return "A piece of transport equipment associated with this transport handling unit."
            case .transportMeans: return "A means of transport associated with this transport handling unit."
            case .hazardousGoodsTransit: return "Transit-related information regarding a type of hazardous goods contained in this transport handling unit."
            case .measurementDimension: return "A measurable dimension (length, mass, weight, or volume) of this transport handling unit."
            case .minimumTemperature: return "The minimum required operating temperature of this transport handling unit."
            case .maximumTemperature: return "The maximum allowable operating temperature of this transport handling unit."
            case .goodsItem: return "A goods item contained in this transport handling unit."
            case .floorSpaceMeasurementDimension: return "The floor space measurement dimension associated with this transport handling unit."
            case .palletSpaceMeasurementDimension: return "The pallet space measurement dimension associated to this transport handling unit."
            case .shipmentDocumentReference: return "A reference to a shipping document associated with this transport handling unit."
            case .status: return "The status of this transport handling unit."
            case .customsDeclaration: return "Describes identifiers or references relating to customs procedures."
            case .referencedShipment: return "A shipment associated with this transport handling unit."
            case .package: return "A package contained in this transport handling unit."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .transportHandlingUnitTypeCode: result = transportHandlingUnitTypeCode?.search(_search)
                case .handlingCode: result = handlingCode?.search(_search)
                case .handlingInstructions: result = handlingInstructions.compactMap({ $0.search(_search) }).first
                case .hazardousRiskIndicator: result = hazardousRiskIndicator?.search(_search)
                case .totalGoodsItemQuantity: result = totalGoodsItemQuantity?.search(_search)
                case .totalPackageQuantity: result = totalPackageQuantity?.search(_search)
                case .damageRemarks: result = damageRemarks.compactMap({ $0.search(_search) }).first
                case .shippingMarks: result = shippingMarks.compactMap({ $0.search(_search) }).first
                case .traceID: result = traceID?.search(_search)
                case .handlingUnitDespatchLine: result = handlingUnitDespatchLine.compactMap({ $0.search(_search) }).first
                case .actualPackage: result = actualPackage.compactMap({ $0.search(_search) }).first
                case .receivedHandlingUnitReceiptLine: result = receivedHandlingUnitReceiptLine.compactMap({ $0.search(_search) }).first
                case .transportEquipment: result = transportEquipment.compactMap({ $0.search(_search) }).first
                case .transportMeans: result = transportMeans.compactMap({ $0.search(_search) }).first
                case .hazardousGoodsTransit: result = hazardousGoodsTransit.compactMap({ $0.search(_search) }).first
                case .measurementDimension: result = measurementDimension.compactMap({ $0.search(_search) }).first
                case .minimumTemperature: result = minimumTemperature?.search(_search)
                case .maximumTemperature: result = maximumTemperature?.search(_search)
                case .goodsItem: result = goodsItem.compactMap({ $0.search(_search) }).first
                case .floorSpaceMeasurementDimension: result = floorSpaceMeasurementDimension?.search(_search)
                case .palletSpaceMeasurementDimension: result = palletSpaceMeasurementDimension?.search(_search)
                case .shipmentDocumentReference: result = shipmentDocumentReference.compactMap({ $0.search(_search) }).first
                case .status: result = status.compactMap({ $0.search(_search) }).first
                case .customsDeclaration: result = customsDeclaration.compactMap({ $0.search(_search) }).first
                case .referencedShipment: result = referencedShipment.compactMap({ $0.search(_search) }).first
                case .package: result = package.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .transportHandlingUnitTypeCode: presentableView(for: transportHandlingUnitTypeCode, title: _title, details: _details, show: show, search: _search)
                case .handlingCode: presentableView(for: handlingCode, title: _title, details: _details, show: show, search: _search)
                case .handlingInstructions: presentableView(for: handlingInstructions.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .hazardousRiskIndicator: presentableView(for: hazardousRiskIndicator, title: _title, details: _details, show: show, search: _search)
                case .totalGoodsItemQuantity: presentableView(for: totalGoodsItemQuantity, title: _title, details: _details, show: show, search: _search)
                case .totalPackageQuantity: presentableView(for: totalPackageQuantity, title: _title, details: _details, show: show, search: _search)
                case .damageRemarks: presentableView(for: damageRemarks.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .shippingMarks: presentableView(for: shippingMarks.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .traceID: presentableView(for: traceID, title: _title, details: _details, show: show, search: _search)
                case .handlingUnitDespatchLine: presentableView(for: handlingUnitDespatchLine.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .actualPackage: presentableView(for: actualPackage.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .receivedHandlingUnitReceiptLine: presentableView(for: receivedHandlingUnitReceiptLine.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .transportEquipment: presentableView(for: transportEquipment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .transportMeans: presentableView(for: transportMeans.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .hazardousGoodsTransit: presentableView(for: hazardousGoodsTransit.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .measurementDimension: presentableView(for: measurementDimension.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .minimumTemperature: presentableView(for: minimumTemperature, title: _title, details: _details, show: show, search: _search)
                case .maximumTemperature: presentableView(for: maximumTemperature, title: _title, details: _details, show: show, search: _search)
                case .goodsItem: presentableView(for: goodsItem.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .floorSpaceMeasurementDimension: presentableView(for: floorSpaceMeasurementDimension, title: _title, details: _details, show: show, search: _search)
                case .palletSpaceMeasurementDimension: presentableView(for: palletSpaceMeasurementDimension, title: _title, details: _details, show: show, search: _search)
                case .shipmentDocumentReference: presentableView(for: shipmentDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .status: presentableView(for: status.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .customsDeclaration: presentableView(for: customsDeclaration.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .referencedShipment: presentableView(for: referencedShipment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .package: presentableView(for: package.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TransportHandlingUnit: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = id?.search(string) { return result }
        if let result = transportHandlingUnitTypeCode?.search(string) { return result }
        if let result = handlingCode?.search(string) { return result }
        if let result = handlingInstructions.compactMap({ $0.search(string)  }).first { return result }
        if let result = hazardousRiskIndicator?.search(string) { return result }
        if let result = totalGoodsItemQuantity?.search(string) { return result }
        if let result = totalPackageQuantity?.search(string) { return result }
        if let result = damageRemarks.compactMap({ $0.search(string)  }).first { return result }
        if let result = shippingMarks.compactMap({ $0.search(string)  }).first { return result }
        if let result = traceID?.search(string) { return result }
        if let result = handlingUnitDespatchLine.compactMap({ $0.search(string)  }).first { return result }
        if let result = actualPackage.compactMap({ $0.search(string)  }).first { return result }
        if let result = receivedHandlingUnitReceiptLine.compactMap({ $0.search(string)  }).first { return result }
        if let result = transportEquipment.compactMap({ $0.search(string)  }).first { return result }
        if let result = transportMeans.compactMap({ $0.search(string)  }).first { return result }
        if let result = hazardousGoodsTransit.compactMap({ $0.search(string)  }).first { return result }
        if let result = measurementDimension.compactMap({ $0.search(string)  }).first { return result }
        if let result = minimumTemperature?.search(string) { return result }
        if let result = maximumTemperature?.search(string) { return result }
        if let result = goodsItem.compactMap({ $0.search(string)  }).first { return result }
        if let result = floorSpaceMeasurementDimension?.search(string) { return result }
        if let result = palletSpaceMeasurementDimension?.search(string) { return result }
        if let result = shipmentDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        if let result = status.compactMap({ $0.search(string)  }).first { return result }
        if let result = customsDeclaration.compactMap({ $0.search(string)  }).first { return result }
        if let result = referencedShipment.compactMap({ $0.search(string)  }).first { return result }
        if let result = package.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
