// 
// TransportMeans.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TransportMeans: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .journeyID,
        .registrationNationalityID,
        .registrationNationality,
        .directionCode,
        .transportMeansTypeCode,
        .tradeServiceCode,
        .stowage,
        .airTransport,
        .roadTransport,
        .railTransport,
        .maritimeTransport,
        .ownerParty,
        .measurementDimension
]
    public var hasNilValue: Bool {(journeyID == nil)
        || (registrationNationalityID == nil)
        || (directionCode == nil)
        || (transportMeansTypeCode == nil)
        || (tradeServiceCode == nil)
        || (stowage == nil)
        || (airTransport == nil)
        || (roadTransport == nil)
        || (railTransport == nil)
        || (maritimeTransport == nil)
        || (ownerParty == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .journeyID: return "Journey Identifier"
            case .registrationNationalityID: return "Registration Nationality Identifier"
            case .registrationNationality: return "Registration Nationality"
            case .directionCode: return "Direction Code"
            case .transportMeansTypeCode: return "Transport Means Type Code"
            case .tradeServiceCode: return "Trade Service Code"
            case .stowage: return "Stowage"
            case .airTransport: return "Air Transport"
            case .roadTransport: return "Road Transport"
            case .railTransport: return "Rail Transport"
            case .maritimeTransport: return "Maritime Transport"
            case .ownerParty: return "Owner Party"
            case .measurementDimension: return "Measurement Dimension"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .journeyID: return "An identifier for the regular service schedule of this means of transport."
            case .registrationNationalityID: return "An identifier for the country in which this means of transport is registered."
            case .registrationNationality: return "Text describing the country in which this means of transport is registered."
            case .directionCode: return "A code signifying the direction of this means of transport."
            case .transportMeansTypeCode: return "A code signifying the type of this means of transport (truck, vessel, etc.)."
            case .tradeServiceCode: return "A code signifying the service regularly provided by the carrier operating this means of transport."
            case .stowage: return "The location within the means of transport where goods are to be or have been stowed."
            case .airTransport: return "An aircraft used for transport."
            case .roadTransport: return "A vehicle used for road transport."
            case .railTransport: return "Equipment used for rail transport."
            case .maritimeTransport: return "A vessel used for transport by water (not only by sea)."
            case .ownerParty: return "The party that owns this means of transport."
            case .measurementDimension: return "A measurable dimension (length, mass, weight, or volume) of this means of transport."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .journeyID: result = journeyID?.search(_search)
                case .registrationNationalityID: result = registrationNationalityID?.search(_search)
                case .registrationNationality: result = registrationNationality.compactMap({ $0.search(_search) }).first
                case .directionCode: result = directionCode?.search(_search)
                case .transportMeansTypeCode: result = transportMeansTypeCode?.search(_search)
                case .tradeServiceCode: result = tradeServiceCode?.search(_search)
                case .stowage: result = stowage?.search(_search)
                case .airTransport: result = airTransport?.search(_search)
                case .roadTransport: result = roadTransport?.search(_search)
                case .railTransport: result = railTransport?.search(_search)
                case .maritimeTransport: result = maritimeTransport?.search(_search)
                case .ownerParty: result = ownerParty?.search(_search)
                case .measurementDimension: result = measurementDimension.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .journeyID: presentableView(for: journeyID, title: _title, details: _details, show: show, search: _search)
                case .registrationNationalityID: presentableView(for: registrationNationalityID, title: _title, details: _details, show: show, search: _search)
                case .registrationNationality: presentableView(for: registrationNationality.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .directionCode: presentableView(for: directionCode, title: _title, details: _details, show: show, search: _search)
                case .transportMeansTypeCode: presentableView(for: transportMeansTypeCode, title: _title, details: _details, show: show, search: _search)
                case .tradeServiceCode: presentableView(for: tradeServiceCode, title: _title, details: _details, show: show, search: _search)
                case .stowage: presentableView(for: stowage, title: _title, details: _details, show: show, search: _search)
                case .airTransport: presentableView(for: airTransport, title: _title, details: _details, show: show, search: _search)
                case .roadTransport: presentableView(for: roadTransport, title: _title, details: _details, show: show, search: _search)
                case .railTransport: presentableView(for: railTransport, title: _title, details: _details, show: show, search: _search)
                case .maritimeTransport: presentableView(for: maritimeTransport, title: _title, details: _details, show: show, search: _search)
                case .ownerParty: presentableView(for: ownerParty, title: _title, details: _details, show: show, search: _search)
                case .measurementDimension: presentableView(for: measurementDimension.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TransportMeans: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = journeyID?.search(string) { return result }
        if let result = registrationNationalityID?.search(string) { return result }
        if let result = registrationNationality.compactMap({ $0.search(string)  }).first { return result }
        if let result = directionCode?.search(string) { return result }
        if let result = transportMeansTypeCode?.search(string) { return result }
        if let result = tradeServiceCode?.search(string) { return result }
        if let result = stowage?.search(string) { return result }
        if let result = airTransport?.search(string) { return result }
        if let result = roadTransport?.search(string) { return result }
        if let result = railTransport?.search(string) { return result }
        if let result = maritimeTransport?.search(string) { return result }
        if let result = ownerParty?.search(string) { return result }
        if let result = measurementDimension.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
