// 
// TransportationService.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.TransportationService: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .transportServiceCode,
        .tariffClassCode,
        .priority,
        .freightRateClassCode,
        .transportationServiceDescription,
        .transportationServiceDetailsURI,
        .nominationDate,
        .nominationTime,
        .name,
        .sequenceNumeric,
        .transportEquipment,
        .supportedTransportEquipment,
        .unsupportedTransportEquipment,
        .commodityClassification,
        .supportedCommodityClassification,
        .unsupportedCommodityClassification,
        .totalCapacityDimension,
        .shipmentStage,
        .transportEvent,
        .responsibleTransportServiceProviderParty,
        .environmentalEmission,
        .estimatedDurationPeriod,
        .scheduledServiceFrequency
]
    public var hasNilValue: Bool {(tariffClassCode == nil)
        || (priority == nil)
        || (freightRateClassCode == nil)
        || (transportationServiceDetailsURI == nil)
        || (nominationDate == nil)
        || (nominationTime == nil)
        || (name == nil)
        || (sequenceNumeric == nil)
        || (totalCapacityDimension == nil)
        || (responsibleTransportServiceProviderParty == nil)
        || (estimatedDurationPeriod == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .transportServiceCode: return "Transport Service Code"
            case .tariffClassCode: return "Tariff Class Code"
            case .priority: return "Priority"
            case .freightRateClassCode: return "Freight Rate Class Code"
            case .transportationServiceDescription: return "Transportation Service Description"
            case .transportationServiceDetailsURI: return "Transportation Service Details URI"
            case .nominationDate: return "Nomination Date"
            case .nominationTime: return "Nomination Time"
            case .name: return "Name"
            case .sequenceNumeric: return "Sequence"
            case .transportEquipment: return "Transport Equipment"
            case .supportedTransportEquipment: return "Supported Transport Equipment"
            case .unsupportedTransportEquipment: return "Unsupported Transport Equipment"
            case .commodityClassification: return "Commodity Classification"
            case .supportedCommodityClassification: return "Supported Commodity Classification"
            case .unsupportedCommodityClassification: return "Unsupported Commodity Classification"
            case .totalCapacityDimension: return "Total Capacity Dimension"
            case .shipmentStage: return "Shipment Stage"
            case .transportEvent: return "Transport Event"
            case .responsibleTransportServiceProviderParty: return "Responsible Transport Service Provider Party"
            case .environmentalEmission: return "Environmental Emission"
            case .estimatedDurationPeriod: return "Estimated Duration Period"
            case .scheduledServiceFrequency: return "Scheduled Service Frequency"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .transportServiceCode: return "A code signifying the extent of this transportation service (e.g., door-to-door, port-to-port)."
            case .tariffClassCode: return "A code signifying the tariff class applicable to this transportation service."
            case .priority: return "The priority of this transportation service."
            case .freightRateClassCode: return "A code signifying the rate class for freight in this transportation service."
            case .transportationServiceDescription: return "Text describing this transportation service."
            case .transportationServiceDetailsURI: return "The Uniform Resource Identifier (URI) of a document providing additional details regarding this transportation service."
            case .nominationDate: return "In a transport contract, the deadline date by which this transportation service has to be booked. For example, if this service is scheduled for Wednesday 16 February 2011 at 10 a.m. CET, the nomination date might be Tuesday15 February 2011."
            case .nominationTime: return "In a transport contract, the deadline time by which this transportation service has to be booked. For example, if this service is scheduled for Wednesday 16 February 2011 at 10 a.m. CET, the nomination date might be Tuesday15 February 2011 and the nomination time 4 p.m. at the latest."
            case .name: return "The name of this transportation service."
            case .sequenceNumeric: return "A number indicating the order of this transportation service in a sequence of transportation services."
            case .transportEquipment: return "A piece of transport equipment used in this transportation service."
            case .supportedTransportEquipment: return "A piece of transport equipment supported in this transportation service."
            case .unsupportedTransportEquipment: return "A piece of transport equipment that is not supported in this transportation service."
            case .commodityClassification: return "A classification of this transportation service."
            case .supportedCommodityClassification: return "A classification (e.g., general cargo) for commodities that can be handled in this transportation service."
            case .unsupportedCommodityClassification: return "A classification for commodities that cannot be handled in this transportation service."
            case .totalCapacityDimension: return "The total capacity or volume available in this transportation service."
            case .shipmentStage: return "One of the stages of shipment in this transportation service."
            case .transportEvent: return "One of the transport events taking place in this transportation service."
            case .responsibleTransportServiceProviderParty: return "The transport service provider responsible for this transportation service."
            case .environmentalEmission: return "An environmental emission resulting from this transportation service."
            case .estimatedDurationPeriod: return "The estimated duration of this transportation service."
            case .scheduledServiceFrequency: return "A class to specify which day of the week a transport service is operational."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .transportServiceCode: result = transportServiceCode.search(_search)
                case .tariffClassCode: result = tariffClassCode?.search(_search)
                case .priority: result = priority?.search(_search)
                case .freightRateClassCode: result = freightRateClassCode?.search(_search)
                case .transportationServiceDescription: result = transportationServiceDescription.compactMap({ $0.search(_search) }).first
                case .transportationServiceDetailsURI: result = transportationServiceDetailsURI?.search(_search)
                case .nominationDate: result = nominationDate?.search(_search)
                case .nominationTime: result = nominationTime?.search(_search)
                case .name: result = name?.search(_search)
                case .sequenceNumeric: result = sequenceNumeric?.search(_search)
                case .transportEquipment: result = transportEquipment.compactMap({ $0.search(_search) }).first
                case .supportedTransportEquipment: result = supportedTransportEquipment.compactMap({ $0.search(_search) }).first
                case .unsupportedTransportEquipment: result = unsupportedTransportEquipment.compactMap({ $0.search(_search) }).first
                case .commodityClassification: result = commodityClassification.compactMap({ $0.search(_search) }).first
                case .supportedCommodityClassification: result = supportedCommodityClassification.compactMap({ $0.search(_search) }).first
                case .unsupportedCommodityClassification: result = unsupportedCommodityClassification.compactMap({ $0.search(_search) }).first
                case .totalCapacityDimension: result = totalCapacityDimension?.search(_search)
                case .shipmentStage: result = shipmentStage.compactMap({ $0.search(_search) }).first
                case .transportEvent: result = transportEvent.compactMap({ $0.search(_search) }).first
                case .responsibleTransportServiceProviderParty: result = responsibleTransportServiceProviderParty?.search(_search)
                case .environmentalEmission: result = environmentalEmission.compactMap({ $0.search(_search) }).first
                case .estimatedDurationPeriod: result = estimatedDurationPeriod?.search(_search)
                case .scheduledServiceFrequency: result = scheduledServiceFrequency.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .transportServiceCode: presentableView(for: transportServiceCode, title: _title, details: _details, show: show, search: _search)
                case .tariffClassCode: presentableView(for: tariffClassCode, title: _title, details: _details, show: show, search: _search)
                case .priority: presentableView(for: priority, title: _title, details: _details, show: show, search: _search)
                case .freightRateClassCode: presentableView(for: freightRateClassCode, title: _title, details: _details, show: show, search: _search)
                case .transportationServiceDescription: presentableView(for: transportationServiceDescription.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .transportationServiceDetailsURI: presentableView(for: transportationServiceDetailsURI, title: _title, details: _details, show: show, search: _search)
                case .nominationDate: anyView(for: nominationDate, title: _title, details: _details, show: show, search: _search)
                case .nominationTime: anyView(for: nominationTime, title: _title, details: _details, show: show, search: _search)
                case .name: presentableView(for: name, title: _title, details: _details, show: show, search: _search)
                case .sequenceNumeric: presentableView(for: sequenceNumeric, title: _title, details: _details, show: show, search: _search)
                case .transportEquipment: presentableView(for: transportEquipment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .supportedTransportEquipment: presentableView(for: supportedTransportEquipment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .unsupportedTransportEquipment: presentableView(for: unsupportedTransportEquipment.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .commodityClassification: presentableView(for: commodityClassification.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .supportedCommodityClassification: presentableView(for: supportedCommodityClassification.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .unsupportedCommodityClassification: presentableView(for: unsupportedCommodityClassification.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .totalCapacityDimension: presentableView(for: totalCapacityDimension, title: _title, details: _details, show: show, search: _search)
                case .shipmentStage: presentableView(for: shipmentStage.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .transportEvent: presentableView(for: transportEvent.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .responsibleTransportServiceProviderParty: presentableView(for: responsibleTransportServiceProviderParty, title: _title, details: _details, show: show, search: _search)
                case .environmentalEmission: presentableView(for: environmentalEmission.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .estimatedDurationPeriod: presentableView(for: estimatedDurationPeriod, title: _title, details: _details, show: show, search: _search)
                case .scheduledServiceFrequency: presentableView(for: scheduledServiceFrequency.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.TransportationService: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = nominationDate?.search(string) { return result }
        if let result = nominationTime?.search(string) { return result }
        if let result = transportServiceCode.search(string) { return result }
        if let result = tariffClassCode?.search(string) { return result }
        if let result = priority?.search(string) { return result }
        if let result = freightRateClassCode?.search(string) { return result }
        if let result = transportationServiceDescription.compactMap({ $0.search(string)  }).first { return result }
        if let result = transportationServiceDetailsURI?.search(string) { return result }
        if let result = name?.search(string) { return result }
        if let result = sequenceNumeric?.search(string) { return result }
        if let result = transportEquipment.compactMap({ $0.search(string)  }).first { return result }
        if let result = supportedTransportEquipment.compactMap({ $0.search(string)  }).first { return result }
        if let result = unsupportedTransportEquipment.compactMap({ $0.search(string)  }).first { return result }
        if let result = commodityClassification.compactMap({ $0.search(string)  }).first { return result }
        if let result = supportedCommodityClassification.compactMap({ $0.search(string)  }).first { return result }
        if let result = unsupportedCommodityClassification.compactMap({ $0.search(string)  }).first { return result }
        if let result = totalCapacityDimension?.search(string) { return result }
        if let result = shipmentStage.compactMap({ $0.search(string)  }).first { return result }
        if let result = transportEvent.compactMap({ $0.search(string)  }).first { return result }
        if let result = responsibleTransportServiceProviderParty?.search(string) { return result }
        if let result = environmentalEmission.compactMap({ $0.search(string)  }).first { return result }
        if let result = estimatedDurationPeriod?.search(string) { return result }
        if let result = scheduledServiceFrequency.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
