// 
// WorkPhaseReference.swift
// 
// 
// Generated by Christoph Vogelbusch on 26. Nov 2024
//

import Foundation
import SwiftUI
import ViewPresentable
import UBLInvoice

extension UBLInvoice.WorkPhaseReference: KeyViewPresentable {
    public static let elementKeys: [CodingKeys] = [
        .id,
        .workPhaseCode,
        .workPhase,
        .progressPercent,
        .startDate,
        .endDate,
        .workOrderDocumentReference
]
    public var hasNilValue: Bool {(id == nil)
        || (workPhaseCode == nil)
        || (progressPercent == nil)
        || (startDate == nil)
        || (endDate == nil)
    }
    
    public func title(for key: CodingKeys) -> String {
        switch key {
            case .id: return "Identifier"
            case .workPhaseCode: return "Work Phase Code"
            case .workPhase: return "Work Phase"
            case .progressPercent: return "Progress Percent"
            case .startDate: return "Start Date"
            case .endDate: return "End Date"
            case .workOrderDocumentReference: return "Work Order Document Reference"
        }
    }

    public func details(for key: CodingKeys) -> String? {
        switch key {
            case .id: return "An identifier for this phase of work."
            case .workPhaseCode: return "A code signifying this phase of work."
            case .workPhase: return "Text describing this phase of work."
            case .progressPercent: return "The progress percentage of the work phase."
            case .startDate: return "The date on which this phase of work begins."
            case .endDate: return "The date on which this phase of work ends."
            case .workOrderDocumentReference: return "A reference to a document regarding the work order for the project in which this phase of work takes place."
        }
    }
    func skipView(title _title: String, key: CodingKeys, search _search: String?) -> Bool {
        if let _search,
            (_title.search(_search) == nil) {
            let result: SearchResult?
            switch key {
                case .id: result = id?.search(_search)
                case .workPhaseCode: result = workPhaseCode?.search(_search)
                case .workPhase: result = workPhase.compactMap({ $0.search(_search) }).first
                case .progressPercent: result = progressPercent?.search(_search)
                case .startDate: result = startDate?.search(_search)
                case .endDate: result = endDate?.search(_search)
                case .workOrderDocumentReference: result = workOrderDocumentReference.compactMap({ $0.search(_search) }).first
            }
            return (result == nil)
        }
        return false
    }

    @MainActor @ViewBuilder public func view(for key: CodingKeys, show: Bool = false, search _search: String? = nil) -> some View {
        let _title = title(for: key)
        if !skipView(title: _title, key: key, search: _search) {
        let _details = details(for: key)
            switch key {
                case .id: presentableView(for: id, title: _title, details: _details, show: show, search: _search)
                case .workPhaseCode: presentableView(for: workPhaseCode, title: _title, details: _details, show: show, search: _search)
                case .workPhase: presentableView(for: workPhase.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
                case .progressPercent: presentableView(for: progressPercent, title: _title, details: _details, show: show, search: _search)
                case .startDate: anyView(for: startDate, title: _title, details: _details, show: show, search: _search)
                case .endDate: anyView(for: endDate, title: _title, details: _details, show: show, search: _search)
                case .workOrderDocumentReference: presentableView(for: workOrderDocumentReference.filter({ if let _search { return ($0.search(_search) != nil) } else { return true } }), title: _title, details: _details, show: show, search: _search)
            }
        }
    }
}

extension UBLInvoice.WorkPhaseReference: Searchable {
    public func search(_ string: String) -> SearchResult? {
        for codingKey in Self.elementKeys {
            let _title = self.title(for: codingKey) 
            if _title.contains(string) {
                return .title(_title)
            }
        }
        if let result = startDate?.search(string) { return result }
        if let result = endDate?.search(string) { return result }
        if let result = id?.search(string) { return result }
        if let result = workPhaseCode?.search(string) { return result }
        if let result = workPhase.compactMap({ $0.search(string)  }).first { return result }
        if let result = progressPercent?.search(string) { return result }
        if let result = workOrderDocumentReference.compactMap({ $0.search(string)  }).first { return result }
        return nil
    }
}
