//
//  TitledDisclosureView.swift
//  XMLInvoice
//
//  Created by Christoph Vogelbusch on 30.10.24.
//
import SwiftUI

public struct TitledDisclosureView<Content: View>: View {
    private let title: String
    private let details: String?
    private let canHide: Bool
    @State private var showHidden: Bool = false
    private let content: (Bool) -> Content
    @State private var show: Bool = true
    public init(title: String, details: String? = nil, canHide: Bool, content: @escaping (Bool) -> Content) {
        self.title = title
        self.details = details
        self.content = content
        self.canHide = canHide
    }
    
    public var body: some View {
        Group {
            HStack {
                Button {
                    withAnimation {
                        show.toggle()
                    }
                } label: {
                    HStack(spacing: 1) {
                        Image(systemName: "chevron.down")
                            .rotationEffect(.degrees(show ? 0 : -90), anchor: .center)
                            .font(.system(size: 10, weight: .bold))
                        Text(title)
                            .font(.system(size: 12, weight: .bold))
                            .optionalHelp(details)
                    }
                    .foregroundStyle(.primary)
                }
                .buttonStyle(.borderless)
                Spacer()
                if canHide {
                    Button {
                        withAnimation {
                            showHidden.toggle()
                        }
                    } label: {
                        Image(systemName: showHidden ? "eye" : "eye.slash")
                            .font(.footnote)
                    }
                    .buttonStyle(.plain)
                }
            }
            if show {
                content(showHidden)
                    .padding(.leading, 12)
            }
        }
    }
}
